package org.ffd2.skeletonx.compile.base;

import org.ffd2.skeletonx.austenx.peg.base.GeneralTypePeer;
import org.ffd2.skeletonx.compile.impl.DesignBlock;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.JavaExternalBlock;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.language.SpecificCommonErrorOutput;


public class GeneralTypeBlock implements GeneralTypePeer.Indirect{
  private SpecificCommonErrorOutput error_;
  private final SpecificCommonErrorOutput baseError_;
  private Handler builtHandler_;
  public GeneralTypeBlock(SpecificCommonErrorOutput baseError) {
    this.baseError_ = baseError;
  }
  public String getTypeDescriptor() {
    return builtHandler_.getTypeDescriptor();
  }
  public DesignBlock getDesignBlock(FileEnvironment environment) throws ParsingException {
    return builtHandler_.getDesign(environment);
  }
  public JavaExternalBlock getJavaExternalsBlock(FileEnvironment environment) throws ParsingException {
    return builtHandler_.getJavaExternal(environment);
  }

//  /**
//   * Get a tokens block from this general type
//   * @return TokensBlock found block
//   * @throws ParsingException if problem get block
//   * @note must be called post initial pass (when entities settled,eg entityResolve pass fine)
//   */
//  public TokensBlock getTokens(BaseLexEnvironment environment) throws ParsingException {
//    return this.builtHandler_.getTokens(environment);
//  }
//
//  public PackratBlock getLibraryPackrat(FileRoot environment) throws ParsingException {
//    return this.builtHandler_.getLibraryPackrat(environment);
//  }
//  /**
//   * Get a library tokens block from this general type
//   * @return LibraryTokensBlock found block
//   * @throws ParsingException if problem get block
//   * @note must be called post initial pass (when entities settled, eg entityResolve pass fine)
//   */
//  public LibraryTokensBlock getLibraryTokens(BaseLexEnvironment environment) throws ParsingException {
//    return this.builtHandler_.getLibraryTokens(environment);
//  }
//  public GeneralTokensStore getGeneralTokens(BaseLexEnvironment environment) throws ParsingException {
//    return this.builtHandler_.getGeneralTokens(environment);
//  }
//    /**
//   * Get a library lex block from this general type
//   * @return LibraryLexBlock found block
//   * @throws ParsingException if problem get block
//   * @note must be called post initial pass (when entities settled, eg entityResolve pass fine)
//   */
//  public PatternLibraryBlock getLexPatternLibrary(BaseLexEnvironment environment) throws ParsingException {
//    return this.builtHandler_.getLexPatternLibrary(environment);
//  }
//
//  public LexDefineBlock getLexDefineLibrary(BaseLexEnvironment environment) throws ParsingException {
//    return this.builtHandler_.getLexDefineLibrary(environment);
//  }

  public void createDirect(final String nameParameter, final int nameLineNumber, final int nameCharacterNumber) {
    this.error_ = this.baseError_.createAdjusted(nameLineNumber,nameCharacterNumber);
    this.builtHandler_ = new Handler() {
      public DesignBlock getDesign(FileEnvironment environment) throws ParsingException {
        return environment.getDesign(nameParameter);
      }
      public String getTypeDescriptor() {
        return  nameParameter;
      }
      public JavaExternalBlock getJavaExternal(FileEnvironment environment) throws ParsingException {
        return environment.getJavaExternal(nameParameter);
      }
    };
  }
  public void createImport(final String namespaceParameter, final String nameParameter, final int nameLineNumber, final int nameCharacterNumber) {
    this.error_ = this.baseError_.createAdjusted(nameLineNumber,nameCharacterNumber);
    this.builtHandler_ = new Handler() {
      public DesignBlock getDesign(FileEnvironment environment) throws ParsingException {
        return environment.getExternalEnvironment(namespaceParameter).getDesign(nameParameter);
      }
      public String getTypeDescriptor() {
        return namespaceParameter+":"+nameParameter;
      }
      public JavaExternalBlock getJavaExternal(FileEnvironment environment) throws ParsingException {
        return environment.getExternalEnvironment(namespaceParameter).getJavaExternal(nameParameter);
      }


    };
  }
  public SpecificCommonErrorOutput getTypeErrorAccess() { return this.error_; }
  // --------------------------------------------------------------------------
  private static interface Handler {
    public DesignBlock getDesign(FileEnvironment environment) throws ParsingException;
    public JavaExternalBlock getJavaExternal(FileEnvironment environment) throws ParsingException;
//    public TokensBlock getTokens(BaseLexEnvironment environment) throws ParsingException;
//    public LibraryTokensBlock getLibraryTokens(BaseLexEnvironment environment) throws ParsingException;
//    public GeneralTokensStore getGeneralTokens(BaseLexEnvironment environment) throws ParsingException;
//
//    public PatternLibraryBlock getLexPatternLibrary(BaseLexEnvironment environment) throws ParsingException;
//    public LexDefineBlock getLexDefineLibrary(BaseLexEnvironment environment) throws ParsingException;
//    public PackratBlock getLibraryPackrat(FileRoot environment) throws ParsingException;

    public String getTypeDescriptor();

  }
}
