package org.ffd2.skeletonx.compile.design;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

import org.ffd2.skeletonx.compile.design.DesignType.ChainLinkSkeletonSource;
import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.ChainImplementationTarget;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.TemplateRootBlock;
import org.ffd2.skeletonx.compile.java.ImplementationParameterTarget;
import org.ffd2.skeletonx.skeleton.BaseTrackers.DesignVariablePath;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder.ChainBlockFormBlock.LinkTypeDataBlock;
import org.ffd2.skeletonx.skeleton.DataBlockFormHolder;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder;

public class BasicDesignTypes implements DesignTypeAccess {
  public static final BasicDesignTypes INSTANCE = new BasicDesignTypes();
  
  private BasicDesignTypes() { }
  public ChainDefinitionBlock getChain(String name) throws ParsingException {
  	throw new ItemNotFoundException(name, "chain",name);
  }
  public MappingDefinitionBlock getMapping(String name) throws ParsingException {
  	throw new ItemNotFoundException(name, "mapping",name);
  }
  public KeyDefinitionBlock getKey(String name) throws ParsingException {
  	throw new ItemNotFoundException(name, "key",name);

	}
  public TemplateRootBlock getTemplate(String name) throws ParsingException {
  	throw new ItemNotFoundException(name, "template",name);
	}
  public DesignType getDesignType(String name) throws ItemNotFoundException {
    if(name.equals("int")||name.equals("Int")) {
      return INT_TYPE;
    } else if(name.equals("double")||name.equals("Double")) {
      return DOUBLE_TYPE;
    } else  if(name.equals("boolean")||name.equals("Boolean")) {
      return BOOLEAN_TYPE;
    } else if(name.equals("string")||name.equals("String")) {
      return STRING_TYPE;
    } else {
      throw new ItemNotFoundException(name,"type",name);
    }
    
  }
  private final static DesignType INT_TYPE = new BasePrimitiveType() {

    public DesignType getDirectSubDesignType(String name) throws ParsingException {
      throw ParsingException.createInvalidType("name", "Int type does not have sub types");
    }
    public void updateParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addInt();
    }
    public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
      target.newIntParameter(parameterName);
    }
    public void createParameterType(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
    	targetDataSkeleton.addInt();
    }
    public void updateJavaParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addInt(skeleton.getRootBuilder().createIntBuilderVariableAccessChild());			
		}
		public DesignType getAsArrayType() throws ParsingException {
			return INT_ARRAY_TYPE;
		}
    public GivenParameterInstantiation createParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
    	final ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.IntDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addInt();
    	return new GivenParameterInstantiation() {
				public void requestRecordFromVariableFinal(	IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
					fixedSkeleton.getArgumentsParameter().addArgumentValue().addInt(instantiated, variablePath);
				}
    	};
    }
		public String getNameForError() { return "int"; }
    
  };
  private final static DesignType INT_ARRAY_TYPE = new BasePrimitiveType() {

    public DesignType getDirectSubDesignType(String name) throws ParsingException {
      throw ParsingException.createInvalidType("name", "Int array type does not have sub types");
    }
    public void updateParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addIntArray();
    }
    public void createParameterType(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
    	targetDataSkeleton.addIntArray();
    }
    public void updateJavaParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addIntArray(skeleton.getRootBuilder().createIntArrayBuilderVariableAccessChild());			
		}
    public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
      target.newIntArrayParameter(parameterName);
    }
    public DesignType getAsArrayType() throws ParsingException {
			return INT_TYPE;
		}
    public GivenParameterInstantiation createParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
    	final ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.IntArrayDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addIntArray();
    	return new GivenParameterInstantiation() {
				public void requestRecordFromVariableFinal(	IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
					fixedSkeleton.getArgumentsParameter().addArgumentValue().addIntArray(instantiated, variablePath);
				}
    	};
    }
		public String getNameForError() { return "int[]"; }
    
  };
  private final static DesignType STRING_TYPE = new BasePrimitiveType() {

    public DesignType getDirectSubDesignType(String name) throws ParsingException {
      throw ParsingException.createInvalidType("name", "String type does not have sub types");
    }
    public void updateParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addString();
    }
    public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
      target.newStringParameter(parameterName);
    }
    public void createParameterType(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
    	targetDataSkeleton.addString();
    }
    public DesignType getAsArrayType() throws ParsingException {
    	Debug.finishMeMarker();
			throw ParsingException.createGeneralSyntax("Array type not allowed here (yet)!");
		}
    public GivenParameterInstantiation createParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
    	final ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addString();
    	return new GivenParameterInstantiation() {
				public void requestRecordFromVariableFinal(	IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
					fixedSkeleton.getArgumentsParameter().addArgumentValue().addString(instantiated, variablePath);
				}
    	};
    }
		public String getNameForError() { return "String"; }
		public void updateJavaParameter(JavaParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addString(skeleton.getRootBuilder().createStringBuilderVariableAccessChild());			
		}
	
  };
  private final static DesignType DOUBLE_TYPE = new BasePrimitiveType() {

    public DesignType getDirectSubDesignType(String name) throws ParsingException {
      throw ParsingException.createInvalidType("name", "Double type does not have sub types");
    }
    public void updateParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addDouble();
    }
    public void createParameterType(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
    	targetDataSkeleton.addDouble();
    }
    public void updateJavaParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addDouble(skeleton.getRootBuilder().createDoubleBuilderVariableAccessChild());			
		}
    public GivenParameterInstantiation createParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
    	final ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.DoubleDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addDouble();
    	return new GivenParameterInstantiation() {
				public void requestRecordFromVariableFinal(	ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
					fixedSkeleton.getArgumentsParameter().addArgumentValue().addDouble(instantiated, variablePath);
				}
    	};
    }

		public String getNameForError() { return "double"; }

    public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
      target.newDoubleParameter(parameterName);
    }
    public DesignType getAsArrayType() throws ParsingException {
    	Debug.finishMeMarker();
			throw ParsingException.createGeneralSyntax("Array type not allowed here (yet)!");
		}
  };
  public final static DesignType BOOLEAN_TYPE = new BasePrimitiveType() {

    public DesignType getDirectSubDesignType(String name) throws ParsingException {
      throw ParsingException.createInvalidType("name", "Boolean type does not have sub types");
    }
    public void updateParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock skeleton, String parameterName) {
      skeleton.addParameter(parameterName).addBoolean();
    }
    public void createParameterType(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
    	final JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock.BooleanDataBlock instantiated = targetDataSkeleton.addBoolean(targetDataSkeleton.getRootBuilder().createBooleanBuilderVariableAccessChild());
    }
//    public GivenParameterInstantiation createParameter(BaseBuilder.JavaParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
//    	final BaseBuilder.JavaParameterSettingsFormBlock.ParameterDataBlock.BooleanDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addBoolean(targetDataSkeleton.getRootBuilder().createBooleanBuilderVariableAccessChild());
//    	final BaseBuilder.JavaParameterSettingsFormBlock.ParameterDataBlock.BooleanDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addBoolean(targetDataSkeleton.getRootBuilder().createBooleanBuilderVariableAccessChild());
//
//    	return new GivenParameterInstantiation() {
//				public void requestRecordFromVariableFinal(	IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
//					fixedSkeleton.getArgumentsParameter().addArgumentValue().addBooleanBlock(instantiated, variablePath);
//				}
//    	};
//    }
		public String getNameForError() { return "boolean"; }

    public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
      target.newBooleanParameter(parameterName);
    }
    public DesignType getAsArrayType() throws ParsingException {
    	Debug.finishMeMarker();
			throw ParsingException.createGeneralSyntax("Array type not allowed here (yet)!");
		}
		public void updateJavaParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock skeleton, String parameterName) {
			// TODO Auto-generated method stub
			
		}
		public GivenParameterInstantiation createParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
			final ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.BooleanDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addBoolean();//(targetDataSkeleton.getRootBuilder().createBooleanBuilderVariableAccessChild());
    	return new GivenParameterInstantiation() {
				public void requestRecordFromVariableFinal(	IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
					fixedSkeleton.getArgumentsParameter().addArgumentValue().addBoolean(instantiated, variablePath);
				}
    	};
		}
		public void createParameterType(ParameterDataBlock targetDataSkeleton) {
			// TODO Auto-generated method stub
    	targetDataSkeleton.addBoolean();//(targetDataSkeleton.getRootBuilder().createBooleanBuilderVariableAccessChild());

		}
		
  };
  private static abstract class BasePrimitiveType implements DesignType {
  	public final LinkTypeDataBlock addChainLink(String linkName,ChainLinkSkeletonSource skeletonSource) throws ParsingException {
  		throw ParsingException.createGeneralSyntax("Primitive types cannot provide links");
  	}
   
  	public final void doAddChainLink(ChainImplementationTarget target, String linkName, int linkIndex) {
  		//Will never happend because the addChainLink() bit will catch the problem in an earlier pass
  	}
   
//    public final FixedValueAccess createFixed(BaseBuilder.DataBlockForm targetDataSkeleton, String parameterName) throws ParsingException {
//    	throw ParsingException.createGeneralSyntax("not supported for given type");
//    }
    public FixedValueAccess createFixed(DataBlockFormHolder.DataBlockFormBlock.IsNotRootDataBlock targetDataSkeleton, String parameterName) throws ParsingException {
    	throw ParsingException.createGeneralSyntax("not supported for given type");

		}
   

  }
	
	
}
