package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer.FixedPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.DesignBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ParameterBlockPatternPeer;
import org.ffd2.skeletonx.compile.design.BasicDesignTypes;
import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.skeletonx.compile.java.ImplementationParameterTarget;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.DataBlockFormHolder.DataBlockFormBlock;
import org.ffd2.skeletonx.skeleton.DesignFormHolder;
import org.ffd2.skeletonx.skeleton.DesignFormHolder.DesignFormBlock;
import org.ffd2.skeletonx.skeleton.IDTrackerFormHolder.IDTrackerFormBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class DesignBlock implements DesignBlockPatternPeer, DesignTypeAccess,  DesignRecordLink {
  private final String name_;
  private final SpecificCommonErrorOutput error_;
  private final FileRoot encompassingFile_;
  private final SubBlockStore subBlocks_;
  private final BlockParameters parameters_;
  private final BaseBuilder baseBuilder_;
  private final DesignFormHolder.DesignFormBlock designSkeleton_;
  private final DesignBridgeComponents designBridge_;
  private final DesignBridgeComponents.DesignRecordReference rootType_;
  private final DesignTypeAccess fallbackTypeAccess_ = BasicDesignTypes.INSTANCE;
  private final AccessStoreX<ChainDefinitionBlock> chainBlocks_;
  private final AccessStoreX<MappingDefinitionBlock> mappingBlocks_;
  private final AccessStoreX<KeyDefinitionBlock> keyBlocks_;
  private final AccessStoreX<TemplateRootBlock> templates_;

  private final SimpleVector<ChainDefinitionBlock> orderedChainBlocks_ = new SimpleVector<ChainDefinitionBlock>();
  private final SimpleVector<MappingDefinitionBlock> orderedMappingBlocks_ = new SimpleVector<MappingDefinitionBlock>();
  private final SimpleVector<KeyDefinitionBlock> orderedKeyBlocks_ = new SimpleVector<KeyDefinitionBlock>();

  public DesignBlock(FileRoot encompassingFile, String name, BaseBuilder baseBuilder,  SpecificCommonErrorOutput error) {
    this.name_ = name;
    this.error_ = error;
    this.baseBuilder_ = baseBuilder;
    this.designBridge_ = new DesignBridgeComponents(baseBuilder,name,encompassingFile.getDefintionGlobalEnvironment());
    this.parameters_ = new BlockParameters(encompassingFile.getDefintionGlobalEnvironment(),baseBuilder,error);
    this.chainBlocks_ = encompassingFile.getDefintionGlobalEnvironment().createEntityStore("chain");
    this.mappingBlocks_ = encompassingFile.getDefintionGlobalEnvironment().createEntityStore("mapping");
    this.keyBlocks_ = encompassingFile.getDefintionGlobalEnvironment().createEntityStore("key");
    this.templates_ = encompassingFile.getDefintionGlobalEnvironment().createEntityStore("template");
    final String fullName = "ROOT:"+name;
		rootType_ = designBridge_.addNewRecord(name,fullName,this,this.parameters_);
    this.rootType_.setIsInvisibleRoot();
    this.designSkeleton_ = rootType_.createDesignChild(baseBuilder);
    
    this.designSkeleton_.getRootBlockDetailsParameter().addIsRoot(this.designSkeleton_);
    
    this.encompassingFile_ = encompassingFile;
    this.subBlocks_ = new SubBlockStore(new BlockParent() {
			public DesignBlock getRootDesignBlock() {
				return DesignBlock.this;
			}
			public DataBlockFormBlock.IsNotRootDataBlock updateDataBlockAsNotRoot(DataBlockFormBlock targetSkeleton) {
				//May be wrong
				return targetSkeleton.addIsNotRoot(designSkeleton_.getRootBlockDetailsParameter(), designSkeleton_.getRootBlockDetailsParameter()).addParentIsRoot().getParent();
			}
    },fullName, encompassingFile.getDefintionGlobalEnvironment());
  
  }

  public void resolutionPass() {
  	
  	for(TemplateRootBlock template : this.templates_ ) {
  		template.resolutionPass(this);
  	}
  	
  	this.subBlocks_.resolveTemplates();
  	
  	for(ChainDefinitionBlock chain : this.chainBlocks_) {
  		chain.resolutionPass();
  	}
  	for(KeyDefinitionBlock key : this.orderedKeyBlocks_ ) {
    	key.basicBuild(this);
  	}
  	for(MappingDefinitionBlock mapping : this.orderedMappingBlocks_ ) {
  		mapping.basicBuild(this);
  	}
 

  	DesignVariableEnvironment withParameters = this.parameters_.basicBuildFromRoot(DesignVariableEnvironment.NULL_ENVIRONMENT,this);

  	this.subBlocks_.basicBuild(withParameters);


  }
  public void intermediatePass() {
//    this.parameters_.updateDesignForm(this.designSkeleton_);
//    for(MappingDefinitionBlock mapping : this.orderedMappingBlocks_ ) {
//  		mapping.infinalBuild();
//  	}
//    for(KeyDefinitionBlock key : this.orderedKeyBlocks_ ) {
//    	key.ininalBuild();
//  	}
//    for(ChainDefinitionBlock chain : orderedChainBlocks_) {
//    	chain.inresolutionPass();
//    }
  }
  public void finalPass() {
  	this.parameters_.finalBuild();
    for(MappingDefinitionBlock mapping : this.orderedMappingBlocks_ ) {
   		mapping.finalBuild();
   	}
    //this.designBridge_.finalPass();
    for(KeyDefinitionBlock key : this.orderedKeyBlocks_ ) {
     	key.finalBuild();
   	}
  	this.subBlocks_.finalPass();

  }
  public DataBlockFormBlock getRootDataBlockSkeleton() {
  	return designSkeleton_.getRootBlockDetailsParameter();
  }
  public boolean isTopLevel() {
  	return false;
  }

  
  //--------------------------------------------------------------------------
  public void buildImplementationParameters(ImplementationParameterTarget target) {
    this.parameters_.buildImplementationParameters(target);
  }
  //--------------------------------------------------------------------------
  public ChainDefinitionBlock getChain(String name) throws ParsingException {
  	return this.chainBlocks_.get(name);
  }
  public TemplateRootBlock getTemplate(String name) throws ParsingException {
  	return this.templates_.get(name);
  }
  public MappingDefinitionBlock getMapping(String name) throws ParsingException {
  	return this.mappingBlocks_.get(name);
  }
  public KeyDefinitionBlock getKey(String name) throws ParsingException {
   	return this.keyBlocks_.get(name); 	
  }
  public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
  	return this.keyBlocks_.get(name); 	
  }
  public DesignType getDesignType(String name) throws ItemNotFoundException {
     try {
       return subBlocks_.getDesignType(name);
     } catch(ItemNotFoundException e) {
       return fallbackTypeAccess_.getDesignType(name);
     }
  }
  // --------------------------------------------------------------------------
  
  public int getTotalNumberOfRecords() {
    return this.designBridge_.getTotalNumberOfRecords();
  }
  public int getTotalNumberOfChains() {
  	return this.orderedChainBlocks_.size();
  }
//  public void addJavaChildrenReferences(BaseBuilder.JavaImplementationForm.BlockDataBlock.IsDesignLinkedDataBlock skeleton, FormSubRecord.JavaSkeletonLinkup linkup) {
//    this.subBlocks_.addJavaChildrenReferences(skeleton, linkup);
//    final BaseBuilder.JavaImplementationForm baseImplementationSkeleton = skeleton.getParent().getParent();
//    
//    
//  }
	public ChainDefinitionBlock getChainDefinition(String name) throws ItemNotFoundException {
		return this.chainBlocks_.get("chain definition", name);
	}
	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
		return this.mappingBlocks_.get("mapping definition", name);

	}
  // ==========================================================================
  public DesignFormBlock getDesignSkeleton() {
    return this.designSkeleton_;
  }
  public TargetBaseFormBlock getTargetBaseSkeleton() {
    return this.designBridge_.getTargetBaseSkeleton();
  }
  public DesignBridgeComponents.DesignRecordReference getRootRecord() {
    return this.rootType_;
  }

//  //Disused
//  private void buildJavaImplementations(BaseBuilder.JavaImplementationForm javaImpl, JavaBlockTypeTarget target) {
//    Debug.println("Building java implementation!");
//  	this.designBridge_.buildJavaImplementations(javaImpl,target);
//    this.rootType_.buildJavaImplementations(javaImpl);
//    Debug.println("Chains:",this.chainBlocks_);
//  }
// 
  
  public void buildImplementations(DesignBaseImplementationTarget target) {
    subBlocks_.buildImplementations(target.getRootBlockTarget());
    for(ChainDefinitionBlock chain : this.chainBlocks_) {
    	chain.updateImplementation(target);
    }
  }
  // ==========================================================================
  public void end() {
    
  }
  public ParameterBlockPatternPeer addParameterBlockForParameters() {
    return this.parameters_;
  }
  public FixedPatternPeer addOptionFixedOfBlockElementForElement(
			String nameParameter, int nameLineNumber, int nameCharacterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
		return parameters_.addFixedElement(nameParameter, error);
	}

  public BlockElementPeer.BlockPatternPeer addOptionBlockOfBlockElementForElement(final String blockNameParameter, final int blockNameLineNumber, final int blockNameCharacterNumber) {
    SpecificCommonErrorOutput error = this.error_.createAdjusted(blockNameLineNumber, blockNameCharacterNumber);
    return this.subBlocks_.addSubBlock(this.encompassingFile_, blockNameParameter,true, this.designSkeleton_.getRootBlockDetailsParameter(), this.designBridge_, this.rootType_,this,error);
  }
	public BlockElementPeer.ChainPatternPeer addOptionChainOfBlockElementForElement(
			int keywordLineNumber, int keywordCharacterNumber,
			String chainNameParameter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		int idCount = this.orderedChainBlocks_.size();
		//this.baseBuilder_.addChainBlock(chainNameParameter, idCount);
		IDTrackerFormBlock.ChainDataBlock idSkeleton = this.designBridge_.newChainIDTrackerSkeleton(chainNameParameter, idCount);
		TargetBaseFormBlock.ChainDataBlock targetSkeleton = this.designBridge_.newChainTargetPeerSkeleton(chainNameParameter, idCount);
		ChainDefinitionBlock defBlock = new ChainDefinitionBlock(chainNameParameter, idCount, this,this.encompassingFile_.getDefintionGlobalEnvironment(),this.designBridge_,this.baseBuilder_,this.designSkeleton_,idSkeleton,targetSkeleton,error);
		this.orderedChainBlocks_.addElement(defBlock);
		try {
		 	this.chainBlocks_.put(chainNameParameter,defBlock);
		} catch(ItemAlreadyExistsException e) {
			error.repeatedObjectError("chain definition", chainNameParameter);
		}
		
		return defBlock;
	}
	
	public void addOptionKeyOfBlockElementForElement(int keywordLineNumber,
			int keywordCharacterNumber, String nameParameter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		KeyDefinitionBlock defBlock = new KeyDefinitionBlock(nameParameter,this.baseBuilder_,error);// this,this.encompassingFile_.getDefintionGlobalEnvironment(),this.designBridge_,this.baseBuilder_,this.designSkeleton_,idSkeleton,targetSkeleton,error);

		this.orderedKeyBlocks_.addElement(defBlock);
		try {
		 	this.keyBlocks_.put(nameParameter,defBlock);
		} catch(ItemAlreadyExistsException e) {
			error.repeatedObjectError("mapping definition", nameParameter);
		}
	}
	public void addOptionMappingOfBlockElementForElement(int keywordLineNumber, int keywordCharacterNumber, String nameParameter, String keyNameParameter, String targetParameter)  {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
		int idCount = this.orderedMappingBlocks_.size();
		//this.baseBuilder_.addChainBlock(chainNameParameter, idCount);
//		skeletonx.skeleton.BaseBuilder.IDTrackerForm.ChainDataBlock idSkeleton = this.designBridge_.newChainIDTrackerSkeleton(chainNameParameter, idCount);
//		skeletonx.skeleton.BaseBuilder.TargetBaseForm.ChainDataBlock targetSkeleton = this.designBridge_.newChainTargetPeerSkeleton(chainNameParameter, idCount);
		MappingDefinitionBlock defBlock = new MappingDefinitionBlock(nameParameter,keyNameParameter,targetParameter,this.baseBuilder_,error);// this,this.encompassingFile_.getDefintionGlobalEnvironment(),this.designBridge_,this.baseBuilder_,this.designSkeleton_,idSkeleton,targetSkeleton,error);
		this.orderedMappingBlocks_.addElement(defBlock);
		try {
		 	this.mappingBlocks_.put(nameParameter,defBlock);
		} catch(ItemAlreadyExistsException e) {
			error.repeatedObjectError("mapping definition", nameParameter);
		}
	}
	public BlockElementPeer.TemplatePatternPeer addOptionTemplateOfBlockElementForElement(String nameValue, int nameLine, int nameCharacter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLine, nameCharacter);
		
		TemplateRootBlock template = new TemplateRootBlock(nameValue,this.designSkeleton_,error,this.encompassingFile_.getDefintionGlobalEnvironment());
		try {
		 	this.templates_.put(nameValue,template);
		} catch(ItemAlreadyExistsException e) {
			error.repeatedObjectError("template", nameValue);
		}
		return template;
	}
	
	
	

	
//	public ChainBlockReference addNewChain(String name, ChainDefinitionBlock chainLink,BaseBuilder baseBuilder,BaseBuilder.DesignForm designSkeleton ) {
//    final int idCount = this.chainBlockReferences_.size();
// 	 ChainBlockReference ref = new ChainBlockReference(name,idCount, baseBuilder, designSkeleton,chainLink, this.baseSkeleton_.addChainBlock(name, idCount),this.targetBaseSkeleton_.addChainBlock(name, idCount));
// 	 this.chainBlockReferences_.addElement(ref);
// 	 return ref;
//  }
	
 


}
