package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.compile.java.ImplementationParameterTarget;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.DataBlockFormHolder.DataBlockFormBlock;
import org.ffd2.skeletonx.skeleton.DesignFormHolder.DesignFormBlock;
import org.ffd2.skeletonx.skeleton.IDTrackerFormHolder.IDTrackerFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.general.SimpleVector;

public class DesignBridgeComponents {
  private final SimpleVector<DesignRecordReference> recordReferences_ = new SimpleVector<DesignRecordReference>();

  
  private final IDTrackerFormBlock baseSkeleton_;
  private final TargetBaseFormBlock targetBaseSkeleton_;
  
  public DesignBridgeComponents(BaseBuilder baseBuilder, String name, GlobalEnvironment environment) {
    this.baseSkeleton_ = baseBuilder.createIDTrackerChild(name);
    this.targetBaseSkeleton_ = baseBuilder.createTargetBaseChild(name); 
  }
//  public final void finalPass() {
//    for(DesignRecordReference ref: this.recordReferences_) {
//      ref.doFinalStuff();
//    }
//  }
  public int getTotalNumberOfRecords() {
    return this.recordReferences_.size();
  }
  public TargetBaseFormBlock getTargetBaseSkeleton() {
    return this.targetBaseSkeleton_;
  }
  /**
   * Build java implementation blocks for root of java implementation, based on design blocks
   * @param javaImpl the target skeleton
   * @return an array of skeleton "blocks" for each type (by index)
   */
  public void buildJavaImplementations(JavaImplementationFormBlock javaImpl, JavaBlockTypeTarget target) {
 
    for(DesignRecordReference ref : this.recordReferences_) {
      JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock skeleton = ref.buildJavaImplementations(javaImpl);
      target.recordBuilt( ref.getRecordIndex(), skeleton, ref);
    }
    
  }
  /**
   * Assumes the parameters are managed elsewhere
   * @param name
   * @param fullName
   * @param designLink
   * @param parameters
   * @return
   */
   public DesignRecordReference addNewRecord(String name, String fullName, DesignRecordLink designLink,BlockParameters parameters) {
  	final BaseBuilder rootBuilder = this.baseSkeleton_.getRootBuilder();
    final int idCount = this.recordReferences_.size();
    DesignRecordReference ref = new DesignRecordReference(this.baseSkeleton_.addType(name, idCount),this.targetBaseSkeleton_.addBlock(name, idCount,parameters.getParametersSkeleton()),name,fullName,idCount,designLink,parameters);
    this.recordReferences_.addElement(ref);
    return ref;
  }
   
   //Hack - DesignBridgeComponents should move to design?
   public IDTrackerFormBlock.ChainDataBlock newChainIDTrackerSkeleton(String name, int index) {
  	 return this.baseSkeleton_.addChain(name, index);
   }
   public TargetBaseFormBlock.ChainDataBlock newChainTargetPeerSkeleton(String name, int index) {
  	 return this.targetBaseSkeleton_.addChain(name, index);
   }
   // =============================================================================================================
  
//  public static final class ChainBlockReference {
//  	private final ChainDataBlock targetPeerSkeleton_;
//		private final skeletonx.skeleton.IDTrackerForm.ChainDataBlock idSkeleton_;
//
//		private final String chainName_;
//		private final ChainBlockLink chainLink_;
//		private final ChainBlockForm chainBlockFormSkeleton_;
//		private final DesignForm.ChainDataBlock designChainSkeleton_;
//		private final int chainTypeID_; 
//		private final SimpleVector<ChainLinkElement> links_ = new SimpleVector<ChainLinkElement>();
//		public ChainBlockReference(String chainName, int chainTypeID, BaseBuilder baseBuilder, DesignForm designSkeleton, ChainBlockLink chainLink,IDTrackerForm.ChainDataBlock idSkeleton, TargetBaseForm.ChainDataBlock targetPeerSkeleton) {
//			
//			this.chainName_ = chainName;
//			this.chainTypeID_ = chainTypeID;
//			this.chainLink_ = chainLink;
//			this.idSkeleton_ = idSkeleton;
//			this.targetPeerSkeleton_ = targetPeerSkeleton;
//			this.chainBlockFormSkeleton_ = baseBuilder.createChainBlockChild(chainName, this.idSkeleton_, this.targetPeerSkeleton_);
//			this.designChainSkeleton_ = designSkeleton.addChainBlock(this.chainBlockFormSkeleton_, this.idSkeleton_, this.targetPeerSkeleton_);
//  	}
//		public DesignForm.ChainDataBlock getDesignChainSkeleton() {
//			return this.designChainSkeleton_;
//		}
//		public void updateImplementation(DesignBaseImplementationTarget target) {
//			ChainImplementationTarget newChain = target.newChain(this.chainTypeID_, this.chainName_, this,this.targetPeerSkeleton_);
//			for(ChainLinkElement e : this.links_) {
//				e.updateChain(newChain);
//			}
//		}
//		
//		public void addLink(String linkName, DesignType type) throws ParsingException {
//			LinkRefDataBlock linkRefBlock = targetPeerSkeleton_.addLinkRefBlock(linkName);
//			this.links_.addElement(new ChainLinkElement(linkName, type,linkRefBlock));
//			type.addChainLink(linkName, this.chainBlockFormSkeleton_,linkRefBlock);
//			
//		}
//  	
//  }
  // ==========================================================================
  
  public static final class DesignRecordReference {
    private final IDTrackerFormBlock.TypeDataBlock idSkeleton_;
    private final TargetBaseFormBlock.BlockDataBlock targetPeerSkeleton_;
    private final String name_;
    private final DesignRecordLink designLink_;
    private final int index_;
		private final BlockParameters	parameters_;
		private final String	fullName_;
    private DesignRecordReference(IDTrackerFormBlock.TypeDataBlock idSkeleton, TargetBaseFormBlock.BlockDataBlock targetPeerSkeleton, String name, String fullName,  int index,DesignRecordLink designLink,BlockParameters parameters) {
      this.idSkeleton_ = idSkeleton;
      this.name_ = name;
			this.fullName_ = fullName;
      this.index_ = index;
      this.designLink_ = designLink;
      this.targetPeerSkeleton_ = targetPeerSkeleton;
			this.parameters_ = parameters;
    }
    public void buildImplementationParameters(ImplementationParameterTarget target) {
      this.parameters_.buildImplementationParameters(target);
    }
    /**
     * Use to indicate that this is the very top level node (the one that isn't named as such - the design's peer). Should only be called once!
     */
    public void setIsInvisibleRoot() {
    	this.targetPeerSkeleton_.addIsRoot();
    }
    public int getRecordIndex() { return this.index_; }
    /**
     * Trying to get rid of this - I don't think any more than this bridge is needed (also need better names)
     * @return
     */
    public DesignRecordLink getDesignRecordBackLink() {
      return this.designLink_;
    }
//    private final void doFinalStuff() {
//    	this.parameters_.finalBuild();
//    	
//      this.parameters_.updateParametersSkeleton(this.targetPeerSkeleton_.getParametersParameter());
//    }
//    public void addParameters(BlockParameters parameters) {
//      parameters.updateParametersSkeleton(this.targetPeerSkeleton_);
//    }
   
    public TargetBaseFormBlock.BlockDataBlock getTargetPeerSkeleton() {
      return this.targetPeerSkeleton_;
    }
    
    // org.ffd2.skeletonx version
    public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock buildJavaImplementations(JavaImplementationFormBlock javaImpl) {
      return javaImpl.addBlock(this.name_, this.fullName_, this.index_,javaImpl.getRootBuilder().createJavaParameterSettingsChild()).addIsDesignLinked(this.targetPeerSkeleton_);
    } 
    
    public TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock createChildRef(String childName) {
      return this.targetPeerSkeleton_.addChildRef(childName);
    }
    public DesignFormBlock  createDesignChild(BaseBuilder baseBuilder) {
      ParameterSettingsFormBlock parameterDetailsSkeleton = baseBuilder.createParameterSettingsChild();
      DataBlockFormBlock dataBlockSkeleton =  baseBuilder.createDataBlockChild(name_,this.idSkeleton_, this.targetPeerSkeleton_, parameterDetailsSkeleton);
			return baseBuilder.createDesignChild(name_,this.idSkeleton_,parameterDetailsSkeleton,this.targetPeerSkeleton_,dataBlockSkeleton);
    }
    
    
    public  DataBlockFormBlock.GeneralSubDataBlock createSubChild(String subChildName,TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInContainerSkeleton, ParameterSettingsFormBlock parametersSkeleton , DataBlockFormBlock baseSkeleton) {
    	//public final DataBlockFormBlock createDataBlockChild(java.lang.String name, IDTrackerFormBlock.TypeDataBlock relatedTracker, TargetBaseFormBlock.BlockDataBlock targetPeer, ParameterSettingsFormBlock parameters) {
 
    	DataBlockFormBlock relatedBuilderSkeleton =
			      	childRefInContainerSkeleton.getRootBuilder().
			      	createDataBlockChild(subChildName,this.idSkeleton_, targetPeerSkeleton_, parametersSkeleton);

    	
			return baseSkeleton.addGeneralSub(subChildName,relatedBuilderSkeleton,childRefInContainerSkeleton,  this.idSkeleton_,this.targetPeerSkeleton_, parametersSkeleton);
    }
  }
  
}
