package org.ffd2.skeletonx.compile.impl;

import java.io.IOException;

import org.ffd2.austenx.runtime.BuildStats;
import org.ffd2.austenx.runtime.GeneralErrorCatcher;
import org.ffd2.skeletonx.austenx.lexi.main.SkeletonLexParser;
import org.ffd2.skeletonx.austenx.lexi.tokens.SkeletonTable;
import org.ffd2.skeletonx.austenx.lexi.tokens.SkeletonToken;
import org.ffd2.skeletonx.austenx.packrat.library.FileRootBuilderForSkeleton;
import org.ffd2.skeletonx.austenx.packrat.library.SkeletonPackrat;
import org.ffd2.skeletonx.austenx.peg.base.DesignBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.FilePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ImportPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExternalBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaImplementatonBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.PackagePathElementPeer.NormalPatternPeer;
import org.ffd2.skeletonx.compile.java.JavaExternalBlock;
import org.ffd2.skeletonx.compile.java.JavaImplementationBlock;
import org.ffd2.skeletonx.compile.library.ParserControl;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.compile.ParserStateCreator;
import org.ffd2.solar.compile.ParserStateInterface;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.general.Source;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.CharacterSource;
import org.ffd2.solar.language.GeneralCommonErrorOutput;
import org.ffd2.solar.language.SimpleEntityStore;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class FileRoot implements FileRootBuilderForSkeleton, FileEnvironment, FilePatternPeer, ParserStateInterface {

	private final GeneralCommonErrorOutput error_;
	private final ParserControl globalBase_;
	private final AccessStoreX<FileRoot> externals_;
	private final AccessStoreX<DesignBlock> designs_;
	private final AccessStoreX<JavaImplementationBlock> javaImplementations_;
	private final AccessStoreX<JavaExternalBlock> javaExternals_;
	private String[] currentOutputPath_;
	private OutputGrouping currentOutputGrouping_;
	private boolean doOutput_;
	private final String fileName_;

	public FileRoot(String fileName, ParserControl globalBase, boolean doOutput) {
		this.fileName_ = fileName;
		this.doOutput_ = doOutput;
		this.error_ = globalBase.getDefinitionEnvironment().formError(fileName);
		this.currentOutputPath_ = null;
		this.globalBase_ = globalBase;
		this.javaExternals_ = createEntityStore("java external");
		this.javaImplementations_ = createEntityStore("java implementation");
		this.externals_ = createEntityStore("external type def");
		this.designs_ = createEntityStore("design block");
	}

	public void updateDoOutputToTrue() {
		this.doOutput_ = true;
	}

	public String readFile(String fileName) throws IOException {
		return this.globalBase_.getParserBase().readTextFile(fileName);
	}

	public <Type> AccessStoreX<Type> createEntityStore(String typeName) {
		return new SimpleEntityStore<Type>(typeName);
	}

	public GlobalEnvironment getDefintionGlobalEnvironment() {
		return this.globalBase_.getDefinitionEnvironment();
	}

	public GlobalEnvironment getImplementationGlobalEnvironment() {
		return this.globalBase_.getImplementationEnvironment();
	}

	public FilePatternPeer buildFile() {
		return this;
	}

	// ==========================================================================
//	private BaseBuilder createNewBaseBuilder(String[] path) {
//		return globalBase_.formNewBuilder(path);
//	}
	private BaseBuilder getCurrentBuilder(SpecificCommonErrorOutput error) throws ParsingException {
		if(this.currentOutputGrouping_!=null) {
			return this.currentOutputGrouping_.getRootBuilder();
		} else {
			throw ParsingException.createGeneralSyntax("Output space not defined");
		}
	}


	// ==========================================================================
	public void end() {

	}

	public ImportPatternPeer addImportForImports(final int keywordLineNumber, final int keywordCharacterNumber, final String namespaceParameter) {
		ImportNode importNode = new ImportNode(namespaceParameter,
																						this,
																						createError(keywordLineNumber, keywordCharacterNumber));
		return importNode;
	}

	public void addImport(String nameSpace, String importFile, boolean doOutput, SpecificCommonErrorOutput error) throws ItemAlreadyExistsException {
		// environment_.addImport(namespace_,importFile,doOutput_,this.error_);
		try {
			FileRoot access = globalBase_.getParserBase().checkFile(importFile, new ParserStateCreator<FileRoot>() {

				public FileRoot generateParser(String fileName, boolean doOutput) {
					return new FileRoot(fileName, FileRoot.this.globalBase_, doOutput);
				}
			}, doOutput);
			this.externals_.put(nameSpace, access);
		} catch (IOException e) {
			error.generalSyntaxError("IO problems with file import of '" + importFile + "' (" + e + ")");
		}
	}
	private final void doSetCurrentPath(String[] path) {
		this.currentOutputPath_ = path;
		this.currentOutputGrouping_ = this.globalBase_.getOutputGrouping(path, this.doOutput_);
	}
	public NormalPatternPeer addOptionNormalOfPackagePathElementForOutputElement() {
		return new NormalPatternPeer() {
			SimpleVector<String> componentStore_ = new SimpleVector<String>();
			public void end() {
				doSetCurrentPath(componentStore_.toStringArray());
			}

			public void addComponent(String value) { this.componentStore_.addElement(value); }
			
		};
	}

	public JavaExternalBlockPatternPeer addJavaExternalBlockForJavaExternalBlock(final String keywordParameter, int keywordLineNumber, int keywordCharacterNumber, String nameParameter) {
		SpecificCommonErrorOutput error = createError(keywordLineNumber, keywordCharacterNumber);
		JavaExternalBlock external = new JavaExternalBlock(this, error);
		try {
			this.javaExternals_.put(nameParameter, external);
		} catch (ItemAlreadyExistsException e) {
			error.repeatedObjectError("java external block", nameParameter);
		}
		return external;
	}

	public DesignBlockPatternPeer addDesignBlockForDesignBlock(final int keywordLineNumber, final int keywordCharacterNumber, final String formNameParameter) {

		SpecificCommonErrorOutput error = createError(keywordLineNumber, keywordCharacterNumber);
		try {
			DesignBlock design = new DesignBlock(this, formNameParameter,getCurrentBuilder(error), error);
	
			try {
				this.designs_.put(formNameParameter, design);
			} catch (ItemAlreadyExistsException e) {
				error.repeatedObjectError("design block", formNameParameter);
			}
			return design;

		} catch (ParsingException e) {
			e.updateError(error);
			return null;
		}
	}

	public JavaImplementatonBlockPatternPeer addJavaImplementatonBlockForJavaBlock(final String keywordParameter, final int keywordLineNumber, final int keywordCharacterNumber, final String javaNameParameter) {
		SpecificCommonErrorOutput error = createError(keywordLineNumber, keywordCharacterNumber);
		try {
			JavaImplementationBlock impl = new JavaImplementationBlock(getCurrentBuilder(error), javaNameParameter, this, error);
		
			try {
				this.javaImplementations_.put(javaNameParameter, impl);
			} catch (ItemAlreadyExistsException e) {
				error.repeatedObjectError("java implementation block", javaNameParameter);
			}
			return impl;
		} catch (ParsingException e) {
			e.updateError(error);
			return null;
		}
	}

	// =========================================================================
	public void resolutionPassDesign() {
		for (DesignBlock design : this.designs_) {
			design.resolutionPass();
		}
		
	}
	public void resolutionPassImplementation() {
		// TODO MAKE SURE CALLED SOME WHERE!
		for (JavaExternalBlock external : this.javaExternals_) {
			external.resolutionPass();
		}
	
		for (JavaImplementationBlock block : this.javaImplementations_) {
			block.resolutionPassImplementation();
		}
	}
	public void intermediatePassDesign() {
	
	}
	public void intermediatePassImplementation() {
		for (JavaImplementationBlock block : this.javaImplementations_) {
			block.intermediatePassImplementation();
		}
	}

	public void finalPassImplementation() {
//		for (DesignBlock design : this.designs_) {
//			design.finalPass();
//		}
		for (JavaImplementationBlock block : this.javaImplementations_) {
			block.finalPassImplementation();
		}
		
	}
	public void finalPassDesign() {
		for (DesignBlock design : this.designs_) {
			design.finalPass();
		}
		for (JavaImplementationBlock block : this.javaImplementations_) {
			block.finalPassDesign();
		}
	}

	// ==========================================================================
	public SpecificCommonErrorOutput createError(int line, int character) {
		return new SpecificCommonErrorOutput(this.error_, line, character);
	}

	// ==========================================================================
	// File environment stuff
	public DesignBlock getDesign(String name) throws ParsingException {
		return this.designs_.get(name);
	}

	public FileEnvironment getExternalEnvironment(String namespace) throws ParsingException {
		return this.externals_.get(namespace);
	}

	public JavaExternalBlock getJavaExternal(String name) throws ItemNotFoundException {
		return this.javaExternals_.get(name);
	}

	public void startParse(CharacterSource fileText) {
		// Debug.println("Text to parse:",text);
		SkeletonTable table = new SkeletonTable();
		SkeletonLexParser lex = new SkeletonLexParser();

		GeneralErrorCatcher errorCatcher = new GeneralErrorCatcher(error_);
		lex.doParseForNormal(fileText, table, errorCatcher);
		if (!errorCatcher.isInError()) {
			Source<SkeletonToken> tokens = table.getTokens();
			Debug.println("Tokens:", tokens);
			// System.out.println("Read:"+tokens.length);

			// System.out.println("tokens:"+solar.general.StringUtils.toString(tokens));
			SkeletonPackrat builder = new SkeletonPackrat(tokens);
			BuildStats stats = builder.buildFile(this, errorCatcher);
			Debug.println("Stats for file:", this.fileName_);
			Debug.println(stats);
		}
	}

}
