package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer.TemplatePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ParameterBlockPatternPeer;
import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.skeletonx.compile.design.FixedValueAccess;
import org.ffd2.skeletonx.compile.java.ImplementationParameterTarget;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers.DesignVariablePath;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder.ChainBlockFormBlock;
import org.ffd2.skeletonx.skeleton.ChainBlockFormHolder.ChainBlockFormBlock.LinkTypeDataBlock;
import org.ffd2.skeletonx.skeleton.DataBlockFormHolder.DataBlockFormBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;
import org.ffd2.skeletonx.skeleton.TemplateBlockFormHolder.TemplateBlockFormBlock;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class FormSubRecord implements BlockElementPeer.BlockPatternPeer, DesignType, DesignRecordLink {
  private final String name_;
  private final SpecificCommonErrorOutput error_;
  private final FileRoot encompassingFile_;
  private final SubBlockStore subBlocks_;
  private final BlockParameters parameters_;
  
  private static interface DistributionType {
  	public void updateSkeleton(DataBlockFormBlock.GeneralSubDataBlock skeletonBlock);
  }
  private static final DistributionType ZERO_OR_MORE_TYPE = new DistributionType() {
		public void updateSkeleton(DataBlockFormBlock.GeneralSubDataBlock skeletonBlock) {
			skeletonBlock.addMulti().addIsZeroOrMore();
		}
  };
  private static final DistributionType ONE_OR_MORE_TYPE = new DistributionType() {
  	public void updateSkeleton(DataBlockFormBlock.GeneralSubDataBlock skeletonBlock) {
			skeletonBlock.addMulti().addIsOneOrMore();
		}
  };
  private static final DistributionType MAYBE_TYPE = new DistributionType() {
  	public void updateSkeleton(DataBlockFormBlock.GeneralSubDataBlock skeletonBlock) {
			skeletonBlock.addIsMaybe();
		}
  };
  
  private DistributionType distrubtion_ = ZERO_OR_MORE_TYPE;
  
  
  private final DataBlockFormBlock.GeneralSubDataBlock skeletonBlock_;
  private final DesignBridgeComponents indexTracker_;
  private final DesignBridgeComponents.DesignRecordReference relatedTypeReference_;
  private final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent_;
  private final DesignTypeAccess encompassingDesignTypeAccess_;
	private final boolean	isTopLevel_;
	private boolean quickIsDependentHack_ = false;
	private final BlockParent parent_;
  private final DataBlockFormBlock.IsNotRootDataBlock myIsNotRootSpecifics_;
  private TemplateExtension endTemplateExtension_ = null;
  
  public FormSubRecord(FileRoot encompassingFile, BaseBuilder rootBuilder, String name,String parentFullName, boolean isTopLevel,DesignBridgeComponents designBridge, DesignBridgeComponents.DesignRecordReference parentType,DataBlockFormBlock parentDataSkeleton, DesignTypeAccess encompassingDesignTypeAccess, final BlockParent parent, SpecificCommonErrorOutput error) {
    this.name_ = name;
		this.isTopLevel_ = isTopLevel;
    this.indexTracker_ = designBridge;
    //This is where you would need to change things to get better type access (like local types).
    this.encompassingDesignTypeAccess_ = encompassingDesignTypeAccess;
		this.parent_ = parent;
    this.error_ = error;
    this.encompassingFile_ = encompassingFile;
    this.parameters_ = new BlockParameters(encompassingFile.getDefintionGlobalEnvironment(),parentDataSkeleton.getRootBuilder(),error);

    final String fullName = parentFullName+":"+name;
		this.relatedTypeReference_ =  designBridge.addNewRecord(name,fullName,this,this.parameters_);
    this.childRefInParent_ = parentType.createChildRef(name);
    this.skeletonBlock_ = relatedTypeReference_.createSubChild(
                                                 name, 
                                                 this.childRefInParent_,
                                                 parameters_.getParametersSkeleton(),
                                                 parentDataSkeleton
                                                );
    //In future versions there should be a more elegant way of doing this (eg passing ref to self through macro calls)
    myIsNotRootSpecifics_ = parent.updateDataBlockAsNotRoot(this.skeletonBlock_.getRelatedDataBlockParameter());

    this.subBlocks_ = new SubBlockStore(new BlockParent() {
			public DesignBlock getRootDesignBlock() {
				return parent.getRootDesignBlock();
			}
			public DataBlockFormBlock.IsNotRootDataBlock updateDataBlockAsNotRoot(DataBlockFormBlock targetSkeleton) {
				return targetSkeleton.addIsNotRoot(skeletonBlock_.getRelatedDataBlockParameter(), parent.getRootDesignBlock().getRootDataBlockSkeleton()).addParentIsNotRoot(myIsNotRootSpecifics_).getParent();
			}
    },fullName, encompassingFile.getDefintionGlobalEnvironment());
  }
  
  // =============================================================
  // Template stuff
  public void addTargetExtension(TemplateBlockFormBlock	templateSkeleton) {
  	this.skeletonBlock_.getRelatedDataBlockParameter().addTemplateExtension(templateSkeleton);
  }
  public BlockParameters getParametersDirectForTemplate() {
  	return this.parameters_;
  }
  public FormSubRecord addTemplateChildQuiet(String childName)  {
  	FormSubRecord result = this.subBlocks_.addSubBlock(this.encompassingFile_, childName,false, this.skeletonBlock_.getRelatedDataBlockParameter(), this.indexTracker_,this.relatedTypeReference_ ,this.encompassingDesignTypeAccess_,this.error_);
  	return result;
  }
  public void resolveTemplates() {
  	if(this.endTemplateExtension_!=null) {
  		this.endTemplateExtension_.resolutionPass(this,this.encompassingDesignTypeAccess_);
  	}
  	this.subBlocks_.resolveTemplates();

  }
  // 
  
  public boolean isTopLevel() { return this.isTopLevel_; }
  public DesignBlock getRootDesignBlock() {
  	return this.parent_.getRootDesignBlock();
  }
  public void basicBuild(final DesignVariableEnvironment parentEnvironment) {
  	DesignVariableEnvironment myEnvironment = new DesignVariableEnvironment() {

			public DesignVariable getVariableQuiet(String variableName, DesignVariablePath variablePath) {
				variablePath.addBlockParent(myIsNotRootSpecifics_);
				return parentEnvironment.getVariableQuiet(variableName, variablePath);
			}
  		
  	};
  	
  	DesignVariableEnvironment environmentWithParameters =  this.parameters_.basicBuildFromNonRoot(myEnvironment,this.encompassingDesignTypeAccess_ ,myIsNotRootSpecifics_);
  	this.subBlocks_.basicBuild(environmentWithParameters);
  	
  }
  public void finalPass() {
  	this.parameters_.finalBuild();
   // this.parameters_.updateParametersSkeleton(this.skeletonBlock_.getSubSettingsParameter());

  }

  public LinkTypeDataBlock addChainLink(String linkName, ChainBlockFormBlock chainBlock, TargetBaseFormBlock.ChainDataBlock.LinkRefDataBlock linkRefSkeleton) {
  	return chainBlock.addLinkType(linkName, this.skeletonBlock_.getRelatedDataBlockParameter(),linkRefSkeleton);
  }
  public LinkTypeDataBlock addChainLink(String linkName, ChainLinkSkeletonSource skeletons) {
  	return skeletons.getChainBlockSkeleton().addLinkType(linkName, this.skeletonBlock_.getRelatedDataBlockParameter(),skeletons.getLinkRefSkeleton());
  }

  // ======================================================================
  // Austen stuff
  public void end() {
  	this.distrubtion_.updateSkeleton(this.skeletonBlock_);
  }
	public BlockElementPeer.FixedPatternPeer addOptionFixedOfBlockElementForElement(
			String nameParameter, int nameLineNumber, int nameCharacterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
		return parameters_.addFixedElement(nameParameter, error);
	}
	public void addTemplateExtension(String value, int lineNumber, int characterNumber) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
		this.endTemplateExtension_ = new TemplateExtension(value,error,this.endTemplateExtension_);
	}
	public TemplatePatternPeer addOptionTemplateOfBlockElementForElement(String nameValue, int nameLine, int nameCharacter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLine, nameCharacter);
		error.generalSyntaxError("Local templates not allowed yet");
		return null;
	}
	
 
  public void addMaybeMarker() { this.distrubtion_ = MAYBE_TYPE;  }
	public void addOneOrMoreMarker() {  this.distrubtion_ = ONE_OR_MORE_TYPE; }
  public ParameterBlockPatternPeer addParameterBlockForParameters() { return this.parameters_; }

  public BlockElementPeer.BlockPatternPeer addOptionBlockOfBlockElementForElement(final String blockNameParameter, final int blockNameLineNumber, final int blockNameCharacterNumber) {
    SpecificCommonErrorOutput error = this.error_.createAdjusted(blockNameLineNumber, blockNameCharacterNumber);

    return this.subBlocks_.addSubBlock(this.encompassingFile_, blockNameParameter,false, this.skeletonBlock_.getRelatedDataBlockParameter(), this.indexTracker_,this.relatedTypeReference_ ,this.encompassingDesignTypeAccess_,error);
  }
  public void addDependent(int lineNumber, int characterNumber) {
  	this.quickIsDependentHack_ = true;
		this.skeletonBlock_.addIsDependent();
	}
	public BlockElementPeer.ChainPatternPeer addOptionChainOfBlockElementForElement(
			int keywordLineNumber, int keywordCharacterNumber, 
			String chainNameParameter
		) {
    SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
    error.invalidConstructionError("chain", chainNameParameter, "only valid at the root");
    return null;
	}
	
	public void addOptionMappingOfBlockElementForElement(int keywordLineNumber,
			int keywordCharacterNumber, String nameParameter,
			String keyNameParameter, String targetParameter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
	  error.invalidConstructionError("mapping", nameParameter, "only valid at the root");
	}
	public void addOptionKeyOfBlockElementForElement(int keywordLineNumber,
			int keywordCharacterNumber, String nameParameter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
	  error.invalidConstructionError("key", nameParameter, "only valid at the root");
	}
	//End of Austen Stuff
	// ==========================================================================
	
  // --------------------------------------------------------------------------
  public void buildImplementationParameters(ImplementationParameterTarget target) {
    this.parameters_.buildImplementationParameters(target);
  }
  // --------------------------------------------------------------------------
  public void buildImplementation(BlockImplementationTarget target) {
    if(this.subBlocks_.isEmpty()) {
      target.newTerminalChild(this.name_, this.relatedTypeReference_,this.childRefInParent_);
    } else {
    	if(target==null) {
    		Debug.println("NULLL TARGET!");
    	}
      BlockImplementationTarget childTarget = target.newInternalChild(this.name_, this.relatedTypeReference_,this.childRefInParent_);
      if(childTarget==null) {
      	Debug.println("Null child from parent:",target);
      }
      this.subBlocks_.buildImplementations(childTarget);
    }
  }
  
//  public void addAsJavaChildrenReference(JavaImplementationForm.BlockDataBlock.IsDesignLinkedDataBlock skeleton,JavaSkeletonLinkup linkup) {
//    final JavaImplementationForm.BlockDataBlock.IsDesignLinkedDataBlock implementationLink = linkup.getBlockSkeleton(this.relatedTypeReference_.getRecordIndex());
//    skeleton.getParent().addChildRefBlock(this.name_, this.childRefInParent_, implementationLink.addIsChildBlock(skeleton));
//  }
 
 
  // --------------------------------------------------------------------------
  // DesignType stuff
  public DesignType getDirectSubDesignType(String name) throws ItemNotFoundException {
    return this.subBlocks_.getDesignType(name);
  }
  public void updateParameter(ParameterSettingsFormBlock skeleton, String parameterName) {
  	ParameterSettingsFormBlock.ParameterDataBlock .RecordDataBlock recordBaseSkeleton = skeleton.addParameter(parameterName).addRecord(this.skeletonBlock_.getRelatedDataBlockParameter());
    
    recordBaseSkeleton.addIsParameter();
  }
//  public void updateJavaParameter(JavaParameterSettingsFormBlock skeleton, String parameterName) {
//  	ParameterSettingsFormBlock.ParameterDataBlock .RecordDataBlock recordBaseSkeleton = skeleton.addParameter(parameterName).addRecord(this.skeletonBlock_.getRelatedDataBlockParameter());
//    
//    recordBaseSkeleton.addIsParameter();
//	}
  public GivenParameterInstantiation createParameter(DataBlockFormBlock targetDataSkeleton, String parameterName) {
  	return createParameter(targetDataSkeleton.getParametersParameter(), parameterName);
  }
  public GivenParameterInstantiation createParameter(ParameterSettingsFormBlock targetDataSkeleton, String parameterName) {
		final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock instantiated = targetDataSkeleton.addParameter(parameterName).addRecord(this.skeletonBlock_.getRelatedDataBlockParameter());
  	final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsParameterDataBlock isParameterSkeleton = instantiated.addIsParameter();
  	return new GivenParameterInstantiation() {
			public void requestRecordFromVariableFinal(IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) throws ParsingException {
				fixedSkeleton.getArgumentsParameter().addArgumentValue().addRecordValue(isParameterSkeleton.getParent(), variablePath);
			}
  	};
  }
  public void createParameterType(ParameterSettingsFormBlock.ParameterDataBlock targetDataSkeleton) {
  	final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock instantiated = targetDataSkeleton.addRecord(this.skeletonBlock_.getRelatedDataBlockParameter());
  	final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsParameterDataBlock isParameterSkeleton = instantiated.addIsParameter();
  	Debug.println("Adding parameter!:",this);
  }
  public FixedValueAccess createFixed(DataBlockFormBlock.IsNotRootDataBlock targetDataSkeleton, String parameterName) {
  	final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock instantiated = targetDataSkeleton.getParent().getParametersParameter().addParameter(parameterName).addRecord(this.skeletonBlock_.getRelatedDataBlockParameter());
  	final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock  isFixedSkeleton = instantiated.addIsFixed(targetDataSkeleton,targetDataSkeleton.getRootBuilder().createDesignArgumentValuesChild());
  	return new FixedValueAccessImpl(isFixedSkeleton,this);
  }
  
	public String getNameForError() { return "boolean"; }
	
//  public FixedValueAccess updateAsFixedValue(ParameterSettingsTemplate skeleton, String parameterName) throws ParsingException {
//  	RecordTemplate recordBaseSkeleton = skeleton.addParameterBlock(parameterName).addRecordBlock(this.skeletonBlock_.getRelatedDataBlockOfDataBlockFixture());
//    
//    RecordTemplate.IsFixedTemplate fixedSkeleton =  recordBaseSkeleton.addIsFixedBlock();
//    return new FixedValueAccessImpl(fixedSkeleton,this);
//  }
  // --------------------------------------------------------------------------
  private final static class FixedValueAccessImpl implements FixedValueAccess {
  	private final ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock	fixedSkeleton_;
		private final FormSubRecord	parent_;

		public FixedValueAccessImpl(ParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock.IsFixedDataBlock fixedSkeleton, FormSubRecord parent) {
			this.fixedSkeleton_ = fixedSkeleton;
			this.parent_ = parent;
  	}

		public void buildParameters(ParameterSource source) {
			parent_.parameters_.requestParameterSetForFixed(this.fixedSkeleton_, source);
		}


		public void requestRecordFromVariableFinal(IsFixedDataBlock fixedSkeleton, DesignVariablePath variablePath) {
  		fixedSkeleton.getArgumentsParameter().addArgumentValue().addRecordValue(this.fixedSkeleton_.getParent(), variablePath);
			
		}
  }
  // ==========================================================================
  
  public DesignType getAsArrayType() throws ParsingException {
  	Debug.finishMeMarker();
		throw ParsingException.createGeneralSyntax("Array type not allowed here (yet)!");
	}
  public void buildImplementationParameter(ImplementationParameterTarget target, String parameterName) {
  //  Need to find the code that determines DesignTypes and transplant into MacroConstruction stuff
//	  Debug.println("Sub form:",this.name_," parameter:", parameterName);
//  	Debug.finishMeMarker();
  
  	target.newRecordParameter(parameterName, this.relatedTypeReference_);
  
  }
  //==========================================================================
//  public static interface JavaSkeletonLinkup {
//    public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock getBlockSkeleton(int index);
//    
//  }
	public void doAddChainLink(ChainImplementationTarget target, String linkName, int linkIndex) {
		
		target.addLink(linkName, linkIndex, this.relatedTypeReference_.getRecordIndex());
		
	}
	
	// ==========================================================================


	public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
		return encompassingDesignTypeAccess_.getChain(name);
	}
	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
		return encompassingDesignTypeAccess_.getMapping(name);
	}
	public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
		return encompassingDesignTypeAccess_.getKey(name);
	}
	
	

	private final static class TemplateExtension {
		private final String	templateName_;
		private final SpecificCommonErrorOutput	error_;
		private final TemplateExtension	previous_;
		private TemplateRootBlock	foundTemplate_;

		public TemplateExtension(String templateName, SpecificCommonErrorOutput error, TemplateExtension previous) {
			this.templateName_ = templateName;
			this.error_ = error;
			this.previous_ = previous;
		}
		public void resolutionPass(FormSubRecord parent, DesignTypeAccess typeAccess) {
			if(this.previous_!=null) {
				this.previous_.resolutionPass(parent, typeAccess);
			}
			try {
				this.foundTemplate_ = typeAccess.getTemplate(this.templateName_);
				Debug.println("Adding template:",this.templateName_);
				this.foundTemplate_.addToRecord(parent);
			} catch (ParsingException e) {
				e.updateError(this.error_);
			}
			
		}
	}

	




}