package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder;
import org.ffd2.skeletonx.skeleton.MappingBuilderFormHolder;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class MappingDefinitionBlock {
	private final String	name_;
	private final SpecificCommonErrorOutput	error_;
	private MappingBuilderFormHolder.MappingBuilderFormBlock	mappingBaseSkeleton_;
	private final BaseBuilder	baseBuilder_;
	private final String	keyName_;
	private final TargetType targetType_;
	public MappingDefinitionBlock(String name, String keyName, String targetName, BaseBuilder baseBuilder,SpecificCommonErrorOutput error) {
		this.name_ = name;
		this.keyName_ = keyName;
		this.baseBuilder_ = baseBuilder;
		this.error_ = error;
		this.targetType_ = getTargetType(targetName);

	}
	public String getName() {
		return "Mapping["+this.name_+"]";
	}
	public ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock.MappingDataBlock addMappingParameter(ParameterSettingsFormHolder.ParameterSettingsFormBlock.ParameterDataBlock base) {
		return base.addMapping(this.mappingBaseSkeleton_);
	}
	public JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock.MappingDataBlock addMappingParameter(JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock base) {
		return base.addMapping(this.mappingBaseSkeleton_);
	}
	public IsMacroDataBlock.MappingVariableDataBlock addMappingMacro(String name, IsMacroDataBlock baseBuilder) {
		return baseBuilder.addMappingVariable(name, this.mappingBaseSkeleton_);
	}
	public void basicBuild(DesignBlock typeAccess) {
		try {
			KeyDefinitionBlock keyDefinition = typeAccess.getKey(this.keyName_);

			this.mappingBaseSkeleton_ = keyDefinition.createMapping(this.name_, this.baseBuilder_);
		} catch(ParsingException e) {
			e.updateError(this.error_);
		}
		
	}
	public void finalBuild() {
		
	}
	public void doSetType(ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock targetViaMapping) {
		this.targetType_.doSetType(targetViaMapping);
	}
	private static final int INT_INDEX = 0;
	private static final TargetType[] ALL_TYPES = {
		new TargetType() {
			public String getName() { return "int"; }
			public void doSetType(ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock targetViaMapping) { targetViaMapping.addIntType(); }
			
		},
		new TargetType() {
			public String getName() { return "double"; }
			public void doSetType(ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock targetViaMapping) { targetViaMapping.addDoubleType(); }
			
		},
		new TargetType() {
			public String getName() { return "String"; }
			public void doSetType(ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock targetViaMapping) { targetViaMapping.addStringType(); }
			
		}
	};
	private static final TargetType getTargetType(String name) {
		for(int i = ALL_TYPES.length-1; i>=0 ;i--) {
			TargetType t = ALL_TYPES[i];
			if(t.getName().equals(name)) {
				return t;
			}
		}
		return ALL_TYPES[INT_INDEX];
	}
	private static interface TargetType {
		public String getName();
		public void doSetType(ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock targetViaMapping);
	}
	
//==========================================================================
	
}
