package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.skeletonx.compile.impl.DesignBridgeComponents.DesignRecordReference;
import org.ffd2.skeletonx.skeleton.DataBlockFormHolder.DataBlockFormBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class SubBlockStore  {
  private final AccessStoreX<FormSubRecord> subBlocks_;
	private final BlockParent	parent_;
	private final String	parentFullName_;

  public SubBlockStore(BlockParent parent, String parentFullName, GlobalEnvironment environment) {

    this.parent_ = parent;
		this.parentFullName_ = parentFullName;
		this.subBlocks_ = environment.createEntityStore("sub block");
  }
  public void basicBuild(DesignVariableEnvironment environment) {
  	for(FormSubRecord sub : this.subBlocks_) {
  		sub.basicBuild(environment);
  	}
  }
  public void resolveTemplates() {
  	for(FormSubRecord sub : this.subBlocks_) {
  		sub.resolveTemplates();
  	}
  }
  public void finalPass() {
  	for(FormSubRecord sub : this.subBlocks_) {
  		sub.finalPass();
  	}
  }
  public FormSubRecord 
        addSubBlock(
                    FileRoot encompassingFile, 
                    final String blockName,
                    boolean isTopLevel, 
                    DataBlockFormBlock dataSkeleton, 
                    DesignBridgeComponents designBridge,
                    DesignRecordReference parentType,
                    DesignTypeAccess encompassingDesignTypeAccess,
                    SpecificCommonErrorOutput error) {

    FormSubRecord block = new FormSubRecord(encompassingFile,dataSkeleton.getRootBuilder(), blockName,parentFullName_,isTopLevel, designBridge,parentType,dataSkeleton,encompassingDesignTypeAccess,this.parent_,error); 
    try {
      this.subBlocks_.put(blockName,block);
    } catch(ItemAlreadyExistsException e) {
      error.repeatedObjectError("sub block", blockName);
    }
    return block;
  }

  public boolean isEmpty() {
    return this.subBlocks_.isEmpty();
  }
  public void buildImplementations(BlockImplementationTarget target) {
    for(FormSubRecord child : this.subBlocks_) {
      child.buildImplementation(target);
    }

  }
  
//  public void addJavaChildrenReferences(BaseBuilder.JavaImplementationForm.BlockDataBlock.IsDesignLinkedDataBlock skeleton, FormSubRecord.JavaSkeletonLinkup linkup) {
//    for(FormSubRecord child : this.subBlocks_.createIterable()) {
//      child.addAsJavaChildrenReference(skeleton, linkup);
//    }
//  }
  // --------------------------------------------------------------------------
  // DesignTypeAccess stuff
  public DesignType getDesignType(String name) throws ItemNotFoundException {
    return this.subBlocks_.get("design type", name);
  }

}
