package org.ffd2.skeletonx.compile.impl;

import org.ffd2.skeletonx.austenx.peg.base.BlockElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.DesignTypePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ParameterBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.ParameterPeer;
import org.ffd2.skeletonx.austenx.peg.base.TemplateElementPeer;
import org.ffd2.skeletonx.austenx.peg.base.TemplateElementPeer.ChildPatternPeer;
import org.ffd2.skeletonx.compile.design.DesignType;
import org.ffd2.skeletonx.compile.design.DesignTypeAccess;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.DesignFormHolder.DesignFormBlock;
import org.ffd2.skeletonx.skeleton.ParameterSettingsFormHolder.ParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.TemplateBlockFormHolder.TemplateBlockFormBlock;
import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class TemplateRootBlock implements BlockElementPeer.TemplatePatternPeer {
	private final SpecificCommonErrorOutput	error_;
	private final String	templateName_;
	private final TemplateBlockFormBlock	templateSkeleton_;
	private final GlobalEnvironment	environment_;
	private final AccessStoreX<TemplateChild> children_;
	public TemplateRootBlock(String templateName, DesignFormBlock skeletonBase, SpecificCommonErrorOutput error, GlobalEnvironment environment) {
		this.templateName_ = templateName;
		this.error_ = error;
		this.environment_ = environment;
		//Need to shift this back to have a macro and set in the Design block
		this.templateSkeleton_ = skeletonBase.getRootBuilder().createTemplateBlockChild(templateName);
		skeletonBase.addTemplate(templateName, this.templateSkeleton_);
		this.children_ = environment.createEntityStore("template child");
	}
	public void resolutionPass(DesignTypeAccess typeAccess) {
		for(TemplateChild child: this.children_) {
			child.resolutionPass(typeAccess);
		}
		
	}
	public void addToRecord(FormSubRecord target) {
		target.addTargetExtension(this.templateSkeleton_);
		for(TemplateChild child: this.children_) {
			child.addChildToRecord(target);
		}
	}
	 // =========================================================================

	public void end() { }

	public TemplateElementPeer.ChildPatternPeer addOptionChildOfTemplateElementForElement(String blockNameValue, int blockNameLine, int blockNameCharacter) {
		SpecificCommonErrorOutput error = this.error_.createAdjusted(blockNameLine, blockNameCharacter);
		TemplateChild child = new TemplateChild(blockNameValue, this.templateSkeleton_,error,this.environment_);
		try {
			this.children_.put(blockNameValue,child);
		} catch (ItemAlreadyExistsException e) {
			e.updateError(error);
		}
		return child;
	}
	// =========================================================================
	private static final class TemplateChild implements TemplateElementPeer.ChildPatternPeer {
		private final GlobalEnvironment	environment_;
		private final AccessStoreX<TemplateChild> children_;
		private final TemplateParameters parameters_;
		private final SpecificCommonErrorOutput	error_;
		private final String	childName_;
		private final TemplateBlockFormBlock.SubBlockDataBlock	templateChildSkeleton_;
		private final ParameterSettingsFormBlock childParametersSkeleton_;
		public TemplateChild(String childName,TemplateBlockFormBlock parentTemplateFormSkeleton, SpecificCommonErrorOutput error, GlobalEnvironment environment) {
			this.childName_ = childName;
			BaseBuilder root = parentTemplateFormSkeleton.getRootBuilder();
			this.childParametersSkeleton_ = root.createParameterSettingsChild();
			this.templateChildSkeleton_ = parentTemplateFormSkeleton.addSubBlock(childName,root.createTemplateBlockChild(childName),this.childParametersSkeleton_);
			this.error_ = error;
			this.environment_ = environment;
			this.children_ = environment.createEntityStore("template child");
			this.parameters_ = new TemplateParameters(this.templateChildSkeleton_,this.childParametersSkeleton_,error,environment );
		}
		public void resolutionPass(DesignTypeAccess typeAccess) {
			for(TemplateChild child: this.children_) {
				child.resolutionPass(typeAccess);
			}
			this.parameters_.resolutionPass(typeAccess);
			//this.parameters_.basicBuildFromRoot(DesignVariableEnvironment.NULL_ENVIRONMENT, typeAccess, templateChildSkeleton_);

		}
		public void addChildToRecord(FormSubRecord target) {
			//TODO
			// adTemplateChildQuiet needs work (need to mark as template)
			// need parameters
			Debug.finishMeMarker();
			FormSubRecord templateChild = target.addTemplateChildQuiet(this.childName_);
			templateChild.end(); //Quick hack to ensure that the cardinality stuff is set up - should call addIsMaybe(), addIsOneOrMore(), beforehand if needed
			parameters_.updateChild(templateChild);
			templateChild.addTargetExtension(templateChildSkeleton_.getChildParameter());
			for(TemplateChild child: this.children_) {
				child.addChildToRecord(templateChild);
			}
		}

		// --------------

		public void end() { }

		public ParameterBlockPatternPeer addParameterBlockForParameters() {
			Debug.println("PARAMETERS!");
			return this.parameters_;
		}

		public ChildPatternPeer addOptionChildOfTemplateElementForElement(String blockNameValue, int blockNameLine, int blockNameCharacter) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(blockNameLine, blockNameCharacter);
			TemplateChild child = new TemplateChild(blockNameValue,this.templateChildSkeleton_.getChildParameter(),error,this.environment_);
			try {
				this.children_.put(blockNameValue,child);
			} catch (ItemAlreadyExistsException e) {
				e.updateError(error);
			}
			return child;
		}
		
	} //End of class TemplateChild
	private static  interface Parameter {
		public void resolutionPass( DesignTypeAccess typeAccess);
		public void updateChildParameters(BlockParameters childParameters);
	}
	public final static class TemplateParameters implements ParameterBlockPatternPeer {
		private final SpecificCommonErrorOutput	error_;
		private final AccessStoreX<Parameter> parameters_;
		private final SimpleVector<Parameter> orderedParameters_ = new SimpleVector<TemplateRootBlock.Parameter>();
		private final GlobalEnvironment	environment_;
		private final TemplateBlockFormBlock.SubBlockDataBlock		templateChildSkeleton_;
		private final ParameterSettingsFormBlock childParameters_;
		public TemplateParameters(TemplateBlockFormBlock.SubBlockDataBlock	templateChildSkeleton, ParameterSettingsFormBlock childParameters, SpecificCommonErrorOutput error, GlobalEnvironment environment) {
			this.templateChildSkeleton_ = templateChildSkeleton;
			this.childParameters_ = childParameters;
			this.error_ = error;
			this.environment_ = environment;
			this.parameters_ = environment.createEntityStore("parameter");
		}
		public void resolutionPass(DesignTypeAccess typeAccess) {
			for(Parameter p : this.orderedParameters_) {
				p.resolutionPass(typeAccess);
			}
		} 
		public void end() { }
		private final void addParameter(String parameterName, Parameter p, SpecificCommonErrorOutput error) {
			try {
				this.parameters_.put(parameterName,p);
			} catch (ItemAlreadyExistsException e) {
				e.updateError(error);
			}
			this.orderedParameters_.addElement(p);
		}
		public void updateChild(FormSubRecord templateChild ) {
			BlockParameters childParameters = templateChild.getParametersDirectForTemplate();
			for(Parameter p : this.orderedParameters_) {
				p.updateChildParameters(childParameters);
			}
		}
		public ParameterPeer.BlockPatternPeer addOptionBlockOfParameterForParameter(String nameValue, int nameLine, int nameCharacter) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(nameLine, nameLine);
			BlockParameter bp = 
					new BlockParameter(
							nameValue,
							this.childParameters_.addParameter(nameValue),
							error, this.environment_);
			addParameter(nameValue,bp,error);
			return bp;
		}

		public void addOptionChainOfParameterForParameter(int keywordLine, int keywordCharacter, String chainNameValue, String variableNameValue) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLine, keywordCharacter);
			// TODO Auto-generated method stub
			
		}

		public void addOptionMappingOfParameterForParameter(int keywordLine, int keywordCharacter, String mappingNameValue, String variableNameValue) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLine, keywordCharacter);
// TODO Auto-generated method stub
			
		}

		public void addOptionKeyOfParameterForParameter(int keywordLine, int keywordCharacter, String keyNameValue, String variableNameValue) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLine, keywordCharacter);
			// TODO Auto-generated method stub
			
		}
	}
	private final static class BlockParameter implements Parameter, ParameterPeer.BlockPatternPeer  {
		private final SpecificCommonErrorOutput error_;
    private final DesignTypeBlock type_;
		private DesignType	resolvedType_;
		private final String	parameterName_;
		private final ParameterSettingsFormBlock.ParameterDataBlock 	parameterSkeleton_; 

		public BlockParameter(String parameterName,ParameterSettingsFormBlock.ParameterDataBlock 	parameterSkeleton,SpecificCommonErrorOutput error, GlobalEnvironment environment) {
			this.parameterName_ = parameterName;
			this.parameterSkeleton_ = parameterSkeleton;
			this.error_=error;
			Debug.println("BLOCK PARAMETER FOR:",parameterName);
			this.type_ = new DesignTypeBlock(environment, error);
		}
		public void resolutionPass( DesignTypeAccess typeAccess) {
    	this.resolvedType_ = type_.buildResolvedQuiet(typeAccess);
			Debug.println("Resolution pass:",parameterName_," to ",resolvedType_);
			if(this.resolvedType_!=null) {
				//BasicDesignTypes.BOOLEAN_TYPE.createParameterType(this.parameterSkeleton_);
				this.resolvedType_.createParameterType(this.parameterSkeleton_);
			}
		}
		public void end() { 	}
		public DesignTypePatternPeer addDesignTypeForType() { return this.type_; }

		public void addIsArray(int lineNumber, int characterNumber) {
			SpecificCommonErrorOutput error = this.error_.createAdjusted(lineNumber, characterNumber);
			this.type_.setIsArray(error);
			
		}
		public void updateChildParameters(BlockParameters childParameters) {
			childParameters.directAddBlock(this.parameterName_,this.resolvedType_,this.error_);
		}
	} //end of class BlockParameter
}
