package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.*;
import org.ffd2.skeletonx.austenx.peg.base.TargetCallComponentPeer.LinkPatternPeer;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.CodeBlockEnvironment.ArgumentsLinker;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;

public class ArgumentBucket implements TargetCallArgumentsPatternPeer,TargetCallComponentPeer.Indirect {
  private final SpecificCommonErrorOutput error_;

  private final SimpleVector<CallArgument> arguments_ = new SimpleVector<CallArgument>();
  public ArgumentBucket( SpecificCommonErrorOutput error) {
    this.error_ = error;
    
  }
	public TargetCallComponentPeer.BasicPatternPeer createBasic() {
		BasicCallArgument basic = new BasicCallArgument( this.error_);
    this.arguments_.addElement(basic);
    return basic;
	}
	public void createMark(String markNameParameter, int markNameLineNumber, int markNameCharacterNumber) {
		// TODO Auto-generated method stub
		SpecificCommonErrorOutput error = this.error_.createAdjusted(markNameLineNumber, markNameCharacterNumber);
		MarkCallArgument mark = new MarkCallArgument(markNameParameter, error);
		this.arguments_.addElement(mark);
	}
	public TargetCallComponentPeer.LinkPatternPeer createLink(int keywordLineNumber,
			int keywordCharacterNumber) {
		LinkCallArgument link = new LinkCallArgument( this.error_);
    this.arguments_.addElement(link);
    return link;
	}
	public void addOptionMarkOfTargetCallComponentForArgument(
			String markNameParameter, int markNameLineNumber,
			int markNameCharacterNumber) {
		createMark(markNameParameter, markNameLineNumber, markNameCharacterNumber);
	}
  public TargetCallComponentPeer.BasicPatternPeer addOptionBasicOfTargetCallComponentForArgument() { return createBasic(); }

  public TargetCallComponentPeer.LinkPatternPeer addOptionLinkOfTargetCallComponentForArgument(int keywordLineNumber, int keywordCharacterNumber) {
    return createLink(keywordLineNumber, keywordCharacterNumber);
  }
 
  public void end() { }
  public void finish(CodeBlockEnvironment environment,ArgumentSetDetailsFormBlock codeFixtureSetArguments) {
    for(CallArgument argument : this.arguments_) {
      argument.finish(environment, codeFixtureSetArguments);
    }
 
  }
  public boolean basicBuild(CodeBlockEnvironment environment) {
    boolean succeeded = true;
  	for(CallArgument argument : this.arguments_) {
  		succeeded &= argument.basicBuild(environment);
    }
  	return succeeded;
 
  }
  private static interface CallArgument {
    public void finish(CodeBlockEnvironment environment,ArgumentSetDetailsFormBlock codeFixtureSetArguments) ;
    public boolean basicBuild(CodeBlockEnvironment environment) ;
  }
  // -------------------------
  private static final class BasicCallArgument implements CallArgument,TargetCallComponentPeer.BasicPatternPeer {
    private final ExpressionBlock expressionBase_;
    public BasicCallArgument( SpecificCommonErrorOutput error) {
      this.expressionBase_ = new ExpressionBlock( error);
    }
    public void end() { }

    public JavaExpressionPeer.Indirect getJavaExpressionForArgument() {
      return expressionBase_;
    }
    public void finish(CodeBlockEnvironment environment, ArgumentSetDetailsFormBlock codeFixtureSetArguments) {
    	ExpressionDetailsFormBlock expressionDetails = codeFixtureSetArguments.getRootBuilder().createExpressionDetailsChild();
      ArgumentSetDetailsFormBlock.ItemDataBlock.ArgumentDataBlock block = codeFixtureSetArguments.addItem().addArgument(expressionDetails);
      this.expressionBase_.finish(environment,expressionDetails);
    }
    public boolean basicBuild(CodeBlockEnvironment environment) {

      return this.expressionBase_.basicBuild(environment);
    }
  }
  // -----------------------
  private static final class LinkedBuild {
  	private final LinkedBuild	previous_;
		private final CodeBlockEnvironment	linkedEnvironment_;
		private final LinkChainBlock	linkChain_;
		private final ArgumentBucket	linkedArguments_;

		public LinkedBuild(LinkChainBlock linkChain,CodeBlockEnvironment linkedEnvironment,ArgumentBucket linkedArguments, LinkedBuild previous) {
			this.linkChain_ = linkChain;
			this.linkedEnvironment_ = linkedEnvironment;
			this.linkedArguments_ = linkedArguments;
			this.previous_ = previous;
  		
  	}
		 public void finish(CodeBlockEnvironment baseEnvironment, ArgumentSetDetailsFormBlock codeFixtureSetArguments) {
			 if(previous_!=null) {
				 previous_.finish(baseEnvironment, codeFixtureSetArguments);
			 }
			 CodeBlockEnvironment.ArgumentsLinker toTargetEnvironmentLinker = baseEnvironment.allocateLinkedArgumentSetter();
			 codeFixtureSetArguments.addItem().addLinkBlock(toTargetEnvironmentLinker.getLinkedArgumentsSkeleton().getVariableStoreParameter());
			 //Debug.println("Linked environment:",linkedEnvironment_);
			 
			 ArgumentSetDetailsFormBlock linkCode  = 	codeFixtureSetArguments.getRootBuilder().createArgumentSetDetailsChild();
			 linkedEnvironment_.getBaseLayer().addItemSkeleton().addLinkedArgumentsSet(
      		toTargetEnvironmentLinker.getLinkedArgumentsSkeleton().getVariableStoreParameter(),
      		Utils.buildGeneralRecordRefParameter(this.linkedEnvironment_.getBaseLayer(),baseEnvironment.getBaseLayer()),
      		linkCode
      	);   	
	      linkedArguments_.finish(this.linkedEnvironment_,linkCode);
     }
  }
  private static final class LinkCallArgument implements CallArgument, TargetCallComponentPeer.LinkPatternPeer {
    private LinkChainBlock lastLinkChain_;
    private final SpecificCommonErrorOutput error_;
    private final SimpleVector<DeferredTargetCallComponent> deferredArguments_ = new SimpleVector<DeferredTargetCallComponent>();
    private LinkedBuild lastLinkBuild_ = null;
    public LinkCallArgument(SpecificCommonErrorOutput error) {
      this.error_ = error;
      
    }
    public ArgumentBucket createLinkedArguments() {
    	ArgumentBucket result = new ArgumentBucket( this.error_);
    	
    	for(DeferredTargetCallComponent da : this.deferredArguments_) {
    		da.buildTargetCallComponent(result);
    	}
    	return result;
    }

    public boolean basicBuild(CodeBlockEnvironment environment) {
    	LinkChainBlock currentLinkChain = this.lastLinkChain_;
    	boolean okay = true;
    	while(currentLinkChain!=null) {
    		CodeBlockEnvironment linkedEnvironment;
	      if((linkedEnvironment= environment.getLinkEndQuiet(currentLinkChain))!=null) {
	      	ArgumentBucket linkedArguments = createLinkedArguments();
	      	boolean result = linkedArguments.basicBuild(linkedEnvironment);
	      	//Debug.println("Link arg:",result);
	      	if(result) {
	      		this.lastLinkBuild_ = new LinkedBuild(currentLinkChain,linkedEnvironment,linkedArguments,this.lastLinkBuild_);
	      	}
	      	okay = okay&&result;
	      } else {
	      	okay=false;
	      }
	      currentLinkChain = currentLinkChain.getPrevious();
    	}
    	return okay;
    }
    public void finish(CodeBlockEnvironment baseEnvironment, ArgumentSetDetailsFormBlock codeFixtureSetArguments) {
    	if(this.lastLinkBuild_!=null) {
    		this.lastLinkBuild_.finish(baseEnvironment, codeFixtureSetArguments);
    	}
    }


//    public TargetCallArgumentsPatternPeer addTargetCallArgumentsForArguments() {
//      //By now we assume that the link chain block has been set up 
//      return this.linkedArguments_;
//    }

    public void end() { /* called twice */ }

//    public TargetCallComponentPeer.Indirect getTargetCallComponentForArgument() {
//       return this.linkedArguments_; //For single parameter stuff
//    }
//		public LinkChainPatternPeer addLinkChainForChain() {
//			return this.linkChain_;
//		}
		public LinkChainPatternPeer addLinkChainForSingleChain() {
			return  this.lastLinkChain_ = new LinkChainBlock(this.error_,this.lastLinkChain_);

		}
		
		public void setTargetCallComponentForArgument(DeferredTargetCallComponent value) {
			this.deferredArguments_.addElement(value);
		}

  }
  //===========================================================================
  // -----------------------
  private static final class MarkCallArgument extends AbstractMarkIRecordVariable implements CallArgument, IRecordVariable {
    private final SpecificCommonErrorOutput error_;
		private final String	markVariableName_;
		private ArgumentsLinker	allocatedArgumentSetter_;
    
    public MarkCallArgument(String markVariableName, SpecificCommonErrorOutput error) {
    	super("argvalues mark");
      this.markVariableName_ = markVariableName;
			this.error_ = error;
      
    }
    @Override public boolean isMatchesVariableName(String queryVariableName) { return this.markVariableName_.equals(queryVariableName); }
		@Override public String getRoughVariableName() { return this.markVariableName_; }
    public void getAsReference(IRecordTypeTarget target) {
	  	//Not a reference - yet?
	  	target.setNotReference();

	  }
   
    public boolean basicBuild(CodeBlockEnvironment environment) {
    	try {
    		environment.addIRecordVariable(this.markVariableName_, this);
    	} catch(ItemAlreadyExistsException e) {
    		this.error_.repeatedObjectError("record variable", markVariableName_);
    		return false;
    	}
    	this.allocatedArgumentSetter_ = environment.allocateLinkedArgumentSetter();
    	
      return true;
    }
    public void finish(CodeBlockEnvironment environment, ArgumentSetDetailsFormBlock codeFixtureSetArguments) {
     codeFixtureSetArguments.addItem().addLinkBlock(this.allocatedArgumentSetter_.getLinkedArgumentsSkeleton().getVariableStoreParameter());
    }
		
		
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath,IVariableType type) throws ParsingException {
			if(type==IVariableType.ARGUMENT_VALUES_REFERENCE) {
				macroCallBlock.addArgumentValuesReference(this.allocatedArgumentSetter_.getLinkedArgumentsSkeleton().getVariableStoreParameter(), variablePath);
			} else {
				throw ParsingException.createInvalidType("argument reference", type.getName());
			}
		}
		public SLayer getAsFoundataionNodeReferenceVariable(JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not record reference");
		}
		public void addAsMacroRecordMappingParameter( MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not mapping");
		}
		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, KeyDefinitionBlock type)
				throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable not key");
			
		}
		public void addAsGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath) throws ParsingException {
			macroCallBlock.addCallParameter().addExpressionSourceFromArgumentsMark(allocatedArgumentSetter_.getLinkedArgumentsSkeleton().getVariableStoreParameter(), variablePath);

		}

  }

}
