package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

public class ArrayedTargetType implements TargetType,DirectTargetElementAccess {
	private final int	dimensions_;
	private final TargetType	base_;

	public ArrayedTargetType(TargetType base,int dimensions) {
		this.base_ = base;
		this.dimensions_ = dimensions;
		
	}
	public DirectTargetElementAccess getDirectElementAccessQuiet() {
		return this;
	}

	public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
		throw ParsingException.createGeneralSyntax("arrays can't provide generics");
	}

	public TargetType getArrayed(int dimensions) throws ParsingException {
		Debug.println("Odd");
		return new ArrayedTargetType(this.base_, this.dimensions_+dimensions);
	}
	public boolean isSubTypeOf(TargetType parentType) {
		//TODO: maybe could do something a bit better - but not a priority
		return parentType==this;
	}
	public void buildCodeSetType(TypeSetterDetailsFormBlock target, JavaVariablePath disposableTargetPath) {
		this.base_.buildCodeSetType(target, disposableTargetPath);
		target.addArrayDimensions(this.dimensions_);
	}

	public String getOutputName() {
		StringBuffer result = new StringBuffer();
		result.append(this.base_.getOutputName());
		for(int i = dimensions_ ; i> 0 ; i--) {
			result.append("[]");
		}
		return result.toString();
	}
	public TargetVariable getTargetVariableQuiet(String name,JavaVariablePath pathToDeclaration) {
		//This is where the stuff for ".length" should go
		if(name.equals("length")) {
			return new LengthVariable();
		} else {
			return null;
		}
	}
	public TargetMethod getTargetMethodQuiet(String name,JavaVariablePath pathToDeclaration) {
		return null;
	}
	// ==========================
	private final static class LengthVariable implements TargetVariable {

		public void createCallExpressionFinal(BasicCallChainFormHolder.BasicCallChainFormBlock target, JavaVariablePath basePathDisposable) {
			target.addElement().addExternalVariable("length");
		}

		public TargetTypeReference getTargetTypeReferenceQuietFinal() {
			return new TargetTypeReference( StandardTargetTypes.INT_TARGET_TYPE);
			
		}

		public void createMacroCallParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath pathToVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("array length cannot provide macro parameters");
		}
		
	}


}
