/**
 * 
 */
package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.compile.impl.ChainImplementationTarget;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;

public final class ChainImplementationConnection implements ChainImplementationTarget {
	final LinkupTracker linkupTracker_ ;
	final JavaImplementationFormBlock.ChainDataBlock chainImplementationSkeleton_;
	private final int chainID_;
	final String chainName_;
	private final SimpleVector<LinkConnection> links_ = new SimpleVector<LinkConnection>();
	final FileEnvironment environment_;
	final SpecificCommonErrorOutput baseError_;

	public ChainImplementationConnection(int chainID,  String chainName,LinkupTracker linkupTracker,JavaImplementationFormBlock.ChainDataBlock chainImplementationSkeleton, FileEnvironment environment, SpecificCommonErrorOutput baseError) {
		this.chainID_ = chainID;
		this.chainName_ = chainName;
		this.linkupTracker_ = linkupTracker;
		this.chainImplementationSkeleton_ = chainImplementationSkeleton;
		this.environment_ = environment;
		this.baseError_ = baseError;
		this.linkupTracker_.setChainImplementation(this.chainID_, this);
	}
	public void finalPass() {
		for(LinkConnection link : this.links_) {
			link.finalPass();
		}
		
	}
	public void addLink(String linkName, int linkIndex, int blockIndex) {
		LinkConnection link = new LinkConnection(this,linkName,blockIndex);
		this.links_.addElement(link);
	}
	public JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock setupParameter(JavaParameterSettingsFormBlock.ParameterDataBlock parameterSkeleton) {
		JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock result = parameterSkeleton.addChainRecord(this.chainImplementationSkeleton_);
		return result;
	}
	private static final class LinkConnection {
  	private final ChainImplementationConnection parent_;
		private final String linkName_;
		private final int blockIndex_;
		public LinkConnection(ChainImplementationConnection parent,String linkName, int blockIndex) {
			this.parent_ = parent;
			this.linkName_ = linkName;
			this.blockIndex_ = blockIndex;
  		
  	}
  	public void finalPass() {
  		//IsDesignLinkedDataBlock typeSkeleton = parent_.linkupTracker_.getBlockSkeleton(blockIndex_);
			try {
				FoundationNode record = this.parent_.linkupTracker_.getImplementationContainer(this.blockIndex_);
				BaseTrackers.JavaVariablePath pathToVariablesStore = new BaseTrackers.JavaVariablePath();
				JavaConcreteChainLink concreteLink = record.getRelatedLayerForLayerComponents().getChainLinkFinal(this.parent_.chainName_, linkName_, pathToVariablesStore);
				JavaImplementationFormBlock.ChainDataBlock.LinkRefDataBlock linkRefBlock = record.doAddLinkRefBlock(this.parent_.chainImplementationSkeleton_,linkName_);
				concreteLink.updateChainLink(linkRefBlock);
			} catch(ParsingException e) {
				e.updateError(this.parent_.baseError_);
			}
		}
  }
}