package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;

import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BClassAccessFormHolder.BClassAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;


public class ClassImplementation implements TargetType{
  private final SLayer definitionLayer_;
	private BClassAccessFormBlock classAccess_;
	private final ClassAccessSource classAccessSource_;
  //private final String name_;
	private final String name_;
	private final TargetTypeListBlock	extendsList_;
	private final TargetTypeListBlock	implementsList_;
 
  
  public ClassImplementation(final String name, ClassAccessSource classAccessSource,SLayer definitionLayer,TargetTypeListBlock extendsList,TargetTypeListBlock implementsList) {
  	this.classAccessSource_ = classAccessSource;
  	this.name_ = name;
    this.definitionLayer_ =definitionLayer;
		this.extendsList_ = extendsList;
		this.implementsList_ = implementsList;
  }
  public String toString() {
  	return "Class["+name_+"]";
  }
	public boolean isSubTypeOf(TargetType parentType) {
		return parentType==this || ( extendsList_!=null&&extendsList_.isSubTypeOf(parentType)) || ( implementsList_!=null&&implementsList_.isSubTypeOf(parentType));
	}
	public SLayer getDefiningLayer() { return this.definitionLayer_; }
	
	public TargetType getArrayed(int dimensions) throws ParsingException {
		return new ArrayedTargetType(this,dimensions);
	}

  public static interface ClassAccessSource {
  	public BClassAccessFormBlock getClassAccess();
  }
  public String getOutputName() {
  	return name_;
  }
  
  public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
		throw ParsingException.createGeneralSyntax("Internal types do not currently support generics");
	}
	public DirectTargetElementAccess getDirectElementAccessQuiet() {
		return null; //Internal thingies can't handle that.
	}
  // ===================================================================================
  
  public final BClassAccessFormBlock getClassAccess() {
  	if(this.classAccess_ == null) { 
  		this.classAccess_ = this.classAccessSource_.getClassAccess();
  	} 
  	return this.classAccess_;  	
  }
 

  
  public void buildCodeSetType(TypeSetterDetailsFormBlock target, JavaVariablePath disposableTargetPath) {

  	target.addFromClass(this.getClassAccess(), disposableTargetPath);
  }
  
  public SLayer getTypeDefinitionRawLayer() {
    return this.definitionLayer_;
  }

  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock 
      createInstanceVariable(SLayer targetLayer, String name, TargetVariable variable, JavaVariablePath definedPathToClassDefiningLayer) 
       {
    return targetLayer.getBlockSkeleton().createInstanceVariable(name, this.getClassAccess(), definedPathToClassDefiningLayer);
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock 
    createInstanceMethodFinal(SLayer targetLayer, String name,  JavaVariablePath pathToClassDefiningLayer) 
    {
    return targetLayer.getBlockSkeleton().createInstanceMethodFinal(name, this.getClassAccess(),pathToClassDefiningLayer);
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock 
  					createConstructorFinal(SLayer targetLayer,  JavaVariablePath pathToClassDefiningLayer)  {
  	return targetLayer.getBlockSkeleton().createConstructorFinal(this.getClassAccess(),pathToClassDefiningLayer);
  }
  public BaseTrackers.JavaVariablePath getPathToClass(SLayer recordAccess) {
    BaseTrackers.JavaVariablePath pathToClassDef = new BaseTrackers.JavaVariablePath();
    getPathToClass(recordAccess, pathToClassDef);
    return pathToClassDef;
  }
  public void getPathToClass(SLayer recordAccess,BaseTrackers.JavaVariablePath pathToClassDef) {
    recordAccess.buildPathToUpperLayer(this.definitionLayer_, pathToClassDef);
  }
  //  ==== Declaring instance variable stuff ====
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock 
          doDeclareInstanceVariable(SLayer recordAccess, String variableName,BaseTrackers.JavaVariablePath pathToThisClass) {
    return recordAccess.getBlockSkeleton().doDeclareInstanceVariable(variableName, this.getClassAccess(),pathToThisClass);
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock 
    doDeclareInstanceVariable(SLayer recordAccess, String variableName) {
    return doDeclareInstanceVariable(recordAccess, variableName,getPathToClass(recordAccess));
  }
//  public JavaImplementationForm.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock 
//    doDeclareInstanceVariable(JavaImplementationForm.BlockDataBlock targetSkeleton, String variableName,BaseTrackers.JavaVariablePath pathToThisClass) {
//      return targetSkeleton.addItemBlock().addInstanceVariableBlock(variableName, this.getClassAccess(),this.definitionLayerAccess_.nextIDIndex(), pathToThisClass);
//  }

  //==== Declaring instance method stuff ====
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock 
          doDeclareInstanceMethod(SLayer recordAccess, String methodName,BaseTrackers.JavaVariablePath pathToThisClass) {
    return recordAccess.getBlockSkeleton().doDeclareInstanceMethod(methodName, this.getClassAccess(), pathToThisClass);
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock 
    doDeclareInstanceMethod(SLayer recordAccess, String methodName) {
    return doDeclareInstanceMethod(recordAccess, methodName,getPathToClass(recordAccess));
  }

	
  
  

}
