package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;

public class CodeBlockEnvironment implements  DataBlockEnvironment,LocalCodeTargetVariableAccess {
  private final RecordClassAccess baseRecord_;

  private final ScopeContainer scopeContainer_;

  /**
   * Constructor
   * @param enclosingType
   * @param baseRecord
   * @param parentRecord
   */
  private CodeBlockEnvironment(RecordClassAccess baseRecord, ScopeContainer scopeContainer) {
    this.baseRecord_ = baseRecord; 
    this.scopeContainer_ = scopeContainer;
  }
  public CodeBlockEnvironment createSubBlockEnvironment() {
    return new CodeBlockEnvironment(this.baseRecord_,this.scopeContainer_.getChild());
  }
  public SLayer getBaseLayer() { return this.baseRecord_.getBaseLayer(); }
  
//  public RecordAccess getIncludingRecord() {
//    return this.baseRecord_;
//  }
//  public ImplementationTargetType getEnclosingType() {
//    return enclosingType_;
//  }
  public IRecordVariable getResolvedVariableQuiet(String variableName, BaseTrackers.JavaVariablePath path) throws ParsingException {
    //TODO: Maybe stuff with widen search?
  	return baseRecord_.getBaseLayer().getBuilderVariableQuiet(variableName,path,true);
  }
  public JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock allocateLinkCodeBlock() {
  	return this.baseRecord_.getBaseLayer().allocateLinkedCode();
  }
  public  JavaImplementationFormBlock.BlockDataBlock.LinkedParametersStoreDataBlock allocateLinkedParameters() {
  	return this.baseRecord_.getBaseLayer().allocateLinkedParameters();
  }

  public RecordClassAccess getBaseRecordClassAccess() { 
  	return this.baseRecord_;
  }
  public ArgumentsLinker allocateLinkedArgumentSetter() { 
    return new ArgumentsLinker( this.baseRecord_.getBaseLayer().allocateLinkedArgumentSetter(),this);
  }
  public TargetType getTargetTypeQuiet(String name, BaseTrackers.JavaVariablePath pathToDeclaration) {
  	return this.baseRecord_.getBaseLayer().getTargetTypeQuiet(name, pathToDeclaration,true);
  }
  private final boolean isVariableWithSameNameExistInLocalScope(String name) {
    return this.baseRecord_.getBaseLayer().isLocalTargetVariableQuiet( this.scopeContainer_, name,true);
  }
	public MacroConstruction getMacroConstructionFinal(String name, BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		return this.baseRecord_.getMacroConstructionFinal(name,pathToContainingLayer);
	}
	public void addIRecordVariable(String name, IRecordVariable variable )throws ItemAlreadyExistsException  {
		this.baseRecord_.getBaseLayer().addBuilderVariable( name, variable);
	}
	public void addLocalVariable(String name, TargetVariable variable) throws ItemAlreadyExistsException {
    if(isVariableWithSameNameExistInLocalScope(name)) {
      throw new ItemAlreadyExistsException();
    }
    this.baseRecord_.getBaseLayer().addLocalVariable(this.scopeContainer_, name, variable);
    
  }
  public TargetVariable getLocalVariableQuiet(String name, BaseTrackers.JavaVariablePath chainToVariable){
    return this.baseRecord_.getBaseLayer().getLocalVariableQuiet(this.scopeContainer_, name, chainToVariable,true);
  }
  /**
   * Called by children, back calling for local variables...
   * @param initiatorRecord
   * @param name
   * @param chainToVariable
   * @return
   */
//  private TargetVariable getLocalVariableBackwardsQuiet(RecordClassAccess initiatorRecord, String name, BaseTrackers.JavaVariablePath chainToVariable)  {
//    LocalVariableLinkup linkup = this.localVariables_.get( name);
//    if(linkup!=null) {
//      if(this.baseRecord_==initiatorRecord) {
//        return linkup.getVariableDirect(); //Defined within same record, no chain
//      } else {
//        return linkup.getTargetVariableQuiet(initiatorRecord.getBaseRecord(),chainToVariable); //Defined different record, need chain
//      }
//    } else if(parentRecord_!=null) {
//      return parentRecord_.getLocalVariableBackwardsQuiet(initiatorRecord,name,chainToVariable);
//    } else {
//      return null;
//    }
//  }
  
  public CodeBlockEnvironment getLinkEndQuiet(LinkChainBlock chainBlock) {
  	RecordClassAccess newBaseRecord = this.baseRecord_.getLinkedQuiet(chainBlock);
  	if(newBaseRecord!=null) {
  		return new CodeBlockEnvironment(newBaseRecord,this.scopeContainer_);
  	} else {
  		return null;
  	}
  }
  public static final CodeBlockEnvironment createRootEnvironment(RecordClassAccess baseRecord) {
  	return new CodeBlockEnvironment( baseRecord,  new ScopeContainer());
  }
//  public TargetMethod getTargetMethod(String name,BaseTrackers.JavaVariablePath variablePath) throws ParsingException {
//   return this.baseRecord_.getClassMethod( name,variablePath);
//  }
  
//  public TargetVariable getTargetVariable(String name,BaseTrackers.JavaVariablePath variablePath) throws ParsingException {
//    TargetVariable local = getLocalVariableQuiet(name,variablePath);
//    if(local!=null) {
//      return local;
//    } else {
//      return this.baseRecord_.getClassVariable( name,variablePath);
//    }
//  }
  public TargetMethod getTargetMethodQuiet( String name, JavaVariablePath pathToDeclaration) {
    return this.baseRecord_.getTargetMethodQuiet( name,pathToDeclaration);
  }
 
  public TargetVariable getTargetVariableQuiet( boolean allowLocal, String name, JavaVariablePath pathToDeclaration) {
    if(allowLocal) {
    	BaseTrackers.JavaVariablePath tempStore = new BaseTrackers.JavaVariablePath();
	    TargetVariable local = getLocalVariableQuiet(name,tempStore);
	    if(local!=null) {
	    	tempStore.transferTo(pathToDeclaration);
	    	
	      return local;
	    }
    } 
    return this.baseRecord_.getTargetVariableQuiet(allowLocal,  name,pathToDeclaration);
    
  }
  public TargetMethod getTargetMethodQuiet(TargetType locationType, String name, JavaVariablePath pathToDeclaration) {
    return this.getBaseLayer().getTargetMethodQuiet(locationType, name,pathToDeclaration,true);
  }

  public TargetVariable getTargetVariableQuiet( boolean allowLocal,TargetType locationType,  String name, JavaVariablePath pathToDeclaration) {
    if(this.baseRecord_.isMatchingLocation(locationType)) {
      return this.getTargetVariableQuiet(allowLocal, name, pathToDeclaration);
    } else {
      return this.getBaseLayer().getTargetVariableQuiet(allowLocal, locationType, name,pathToDeclaration,true);
    }
  }
  public TargetVariable getTargetVariableQuiet( TargetType locationType,  String name, JavaVariablePath pathToDeclaration) {
    return getTargetVariableQuiet(true,name,pathToDeclaration);
  }
  
  // ==========================================================================
  public static final class ArgumentsLinker {
    private final CodeBlockEnvironment parent_;
    private final  JavaImplementationFormBlock.BlockDataBlock.LinkedArgumentsStoreDataBlock linkedArgumentsSkeleton_;

    public ArgumentsLinker(JavaImplementationFormBlock.BlockDataBlock.LinkedArgumentsStoreDataBlock linkedArgumentsSkeleton, CodeBlockEnvironment parent) {
      this.linkedArgumentsSkeleton_ = linkedArgumentsSkeleton;
      this.parent_ = parent;
      
    }
    
    public JavaImplementationFormBlock.BlockDataBlock.LinkedArgumentsStoreDataBlock getLinkedArgumentsSkeleton() {
      return this.linkedArgumentsSkeleton_;
    }
  }
  // ==========================================================================
//  private final static class LocalVariableLinkup {
//    private final CodeBlockContainer container_;
//    private final String variableName_;
//    private final TargetVariable variable_;
//
//    public LocalVariableLinkup(CodeBlockContainer container, String variableName, TargetVariable variable) {
//      this.container_ = container;
//      this.variableName_ = variableName;
//      this.variable_ = variable;
//    }
//    public TargetVariable getTargetVariableQuiet(RecordAccess record, BaseTrackers.JavaVariablePath chainToVariable) {
//      return record.getLocalVariableQuiet(this.container_, this.variableName_, chainToVariable);
//    }
//    public TargetVariable getVariableDirect() {
//      return this.variable_;
//    }
//  }
	public MacroCallInstance createMacroCallFinal(
			MacroConstruction macro,
			MacroCallRequirementsAccess requirementsAccess
		) {
		return this.baseRecord_.createMacroCallFinal(macro.getName(), macro, requirementsAccess);
	}
  public void addContextRef(JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock macroCallBlock) {
  	this.baseRecord_.addContextRef(macroCallBlock);
  }
  public void addGeneralRef(JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock macroCallBlock) {
  	this.baseRecord_.addGeneralRef(macroCallBlock);
  }
//
//	public ImplementationTargetType getEnclosingTargetType() {
//		return baseRecord_.getContainingClass();
//	}
  // ==========================================================================
	public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable) {
		return this.baseRecord_.getBaseLayer().getBuilderVariableQuiet(name, pathToVariable,true);
	}
 
}
