package org.ffd2.skeletonx.compile.java;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.general.StringUtils;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GeneralSymbolPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCallChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaExpressionPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetCallArgumentsPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferencePatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.TargetTypeReferenceWithoutArrayPatternPeer;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.ArgumentSetDetailsFormHolder.ArgumentSetDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.BExpressionAccessFormHolder;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock.ComplexDataBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;

import org.ffd2.skeletonx.skeleton.BaseTrackers;

public class ExpressionBlock implements JavaExpressionPeer.Indirect {

  private final SpecificCommonErrorOutput error_;
  private Handler handler_;
  
  private static final Handler NULL_HANDLER = new Handler() {
		public boolean basicBuild(CodeBlockEnvironment environment) { return true; }
		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			expressionSkeleton.addSetNull();	
		}
  };
  private static final Handler THIS_HANDLER = new Handler() {
		public boolean basicBuild(CodeBlockEnvironment environment) { return true; }
		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			expressionSkeleton.addSetThis();	
		}
  };

  public ExpressionBlock( SpecificCommonErrorOutput error) {
    this.error_ = error;    
  }

  public boolean basicBuild(CodeBlockEnvironment environment) {

    if(this.handler_!=null) {
      return handler_.basicBuild(environment);
    } else {
      throw new RuntimeException("Assertion error : STRANGE no handler!");
    }
  }
  public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
    handler_.finish(environment, expressionSkeleton);
  }
  public JavaExpressionPeer.GivenArrayPatternPeer createGivenArray(int keywordLineNumber, int keywordCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
  	GivenArrayHandler handler = new GivenArrayHandler(error);
  	this.handler_ = handler;
		return handler;
	}
	public void createMark(String markName, int markNameLine, int markNameCharacter) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(markNameLine, markNameCharacter);
  	MarkHandler handler = new MarkHandler(markName, error);
  	this.handler_ = handler;
	}
  public JavaExpressionPeer.ArrayConstructorCallPatternPeer createArrayConstructorCall(int keywordLineNumber, int keywordCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
  	ArrayConstructorHandler constructor = new ArrayConstructorHandler(error);
  	this.handler_ = constructor;
  	return constructor;
  }

  public JavaExpressionPeer.AssignmentPatternPeer createAssignment(int keywordLineNumber, int keywordCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
  	AssignmentHandler handler = new AssignmentHandler(error);
  	this.handler_ = handler;
  	return handler;
  }
  public BinaryHandler createBinaryBase() {
  	BinaryHandler handler = new BinaryHandler(this.error_);
  	this.handler_ = handler;
  	return handler;
  }
  public JavaExpressionPeer.BinaryPatternPeer createBinary() {
  	return createBinaryBase();
  }

  public JavaExpressionPeer.BracketedPatternPeer createBracketed(int keywordLineNumber, int keywordCharacterNumber) {
    return new JavaExpressionPeer.BracketedPatternPeer() {
			public void end() {}
			public JavaExpressionPeer.Indirect getJavaExpressionForValue() {
				return ExpressionBlock.this;
			}    	
    };
  }

  public JavaExpressionPeer.BuilderVariableRefPatternPeer createBuilderVariableRef() {
  	BuilderVariableHandler vh = new BuilderVariableHandler(this.error_);
  	this.handler_ = vh;
  	return vh;
  }
  public JavaExpressionPeer.MappingVariableRefPatternPeer createMappingVariableRef() {
  	MappingVariableHandler vh = new MappingVariableHandler(this.error_);
  	this.handler_ = vh;
  	return vh;
	}
  
  
  public JavaExpressionPeer.CallPatternPeer createCall() {
    CallHandler call = new CallHandler(this);
    this.handler_ = call;
    return call;
  }

  public JavaExpressionPeer.ConstructorCallPatternPeer createConstructorCall(int keywordLineNumber, int keywordCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(keywordLineNumber, keywordCharacterNumber);
  	ConstructorCall call = new ConstructorCall(error);
    this.handler_ = call;
    return call;
  }

  public JavaExpressionPeer.GeneralBinaryPatternPeer createGeneralBinary(int symbolLineNumber, int symbolCharacterNumber) {
  	SpecificCommonErrorOutput error = this.error_.createAdjusted(symbolLineNumber, symbolCharacterNumber);
  	GeneralBinaryHandler handler = new GeneralBinaryHandler(error);
  	this.handler_ = handler;
  	return handler;
  }


  public void createNull(int keywordLineNumber, int keywordCharacterNumber) {
  	this.handler_ = NULL_HANDLER;
  }
  public void createThis(int keywordLineNumber, int keywordCharacterNumber) {
    this.handler_ = THIS_HANDLER;
  }

  public JavaExpressionPeer.PostUnaryPatternPeer createPostUnary() {

  	UnaryHandler handler = new UnaryHandler(true,this.error_);
  	this.handler_ = handler;
  	return handler;
  }

  public JavaExpressionPeer.PreUnaryPatternPeer createPreUnary() {
  	UnaryHandler handler = new UnaryHandler(false,this.error_);
  	this.handler_ = handler;
  	return handler;
  }
  

  public JavaExpressionPeer.PrimitivePatternPeer createPrimitive() {
    return new JavaExpressionPeer.PrimitivePatternPeer() {
      public void addBooleanValue(final boolean value, int lineNumber, int characterNumber) {
        handler_ = new SimpleHandler() {
          public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton) {
            expressionSkeleton.addPrimitiveBoolean(value);
          }
        };
      }

      public void addCharValue(final String value, int lineNumber, int characterNumber) {
        handler_ = new SimpleHandler() {
          public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton) {
            expressionSkeleton.addPrimitiveChar(StringUtils.interpretString(value).charAt(0));
          }
        };
      }

      public void addInteger(final int value, int lineNumber, int characterNumber) {
        handler_ = new SimpleHandler() {
          public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton) {
            expressionSkeleton.addPrimitiveInt(value);
          }
        };
      }

      public void addString(final String value, int lineNumber, int characterNumber) {
        handler_ = new SimpleHandler() {
          public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton) {
            expressionSkeleton.addPrimitiveString(value);
          }
         
        }; 
      }
      public void end() {}
    };
  }

  // ==========================================================================
  private final static class MappingVariableHandler implements Handler, JavaExpressionPeer.MappingVariableRefPatternPeer {
  	private final BuilderVariableChainBlock mappingChain_;
  	private final BuilderVariableChainBlock keyChain_;
  	private final BaseTrackers.JavaVariablePath pathToMappingVariableDeclartion_ = new BaseTrackers.JavaVariablePath();
  	private IRecordVariable mappingVariable_;
  	private final BaseTrackers.JavaVariablePath pathToKeyVariableDeclartion_ = new BaseTrackers.JavaVariablePath();
  	private IRecordVariable keyVariable_;
		private MappingReference	mappingReference_;
		private KeyMarkerReference	keyReference_;
  	public MappingVariableHandler(SpecificCommonErrorOutput baseError) {
			this.mappingChain_ = new BuilderVariableChainBlock(baseError);
			this.keyChain_ = new BuilderVariableChainBlock(baseError);

  	}

		public void end() {  }

		public BuilderVariableChainPatternPeer addBuilderVariableChainForMappingChain() { return mappingChain_; }
		public BuilderVariableChainPatternPeer addBuilderVariableChainForSourceVariable() { return keyChain_; }

		public boolean basicBuild(CodeBlockEnvironment environment) {
			this.mappingVariable_ = mappingChain_.resolveRecordVariableQuiet(environment.getBaseLayer(), this.pathToMappingVariableDeclartion_,true);
			this.keyVariable_ = keyChain_.resolveRecordVariableQuiet(environment.getBaseLayer(), this.pathToKeyVariableDeclartion_,true);
			try {
				if(this.mappingVariable_!=null) {
					this.mappingReference_ = IRecordTypeTarget.generateTarget(mappingVariable_).getAsMappingReference(); }
			} catch (ParsingException e) {
				e.updateError(this.mappingChain_.getEndError());
			}
			try {
				if(this.keyVariable_!=null) { this.keyReference_ = IRecordTypeTarget.generateTarget(keyVariable_).getAsKeyReference(); }
			} catch (ParsingException e) {
				e.updateError(this.keyChain_.getEndError());
			}
	
			return this.mappingReference_!=null&&this.keyReference_!=null;
			
		}

		public void finish(CodeBlockEnvironment environment, 	ExpressionDetailsFormBlock expressionSkeleton) {
			this.mappingReference_.doAddViaMapping(expressionSkeleton,  this.pathToMappingVariableDeclartion_, this.keyReference_, this.pathToKeyVariableDeclartion_);

		}
  }
  private final static class BuilderVariableHandler implements Handler, JavaExpressionPeer.BuilderVariableRefPatternPeer {
  	private final BuilderVariableChainBlock variableChain_;
  	private final BaseTrackers.JavaVariablePath pathToVariableDeclartion_ = new BaseTrackers.JavaVariablePath();
  	private IRecordVariable variable_;
		public BuilderVariableHandler(SpecificCommonErrorOutput baseError) {
			this.variableChain_ = new BuilderVariableChainBlock(baseError);
  	}
		public boolean basicBuild(CodeBlockEnvironment environment) {
			this.variable_ = variableChain_.resolveRecordVariableQuiet(environment.getBaseLayer(), this.pathToVariableDeclartion_,true);
			return this.variable_!=null;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			try {
				this.variable_.updateExpressionFinal(environment, expressionSkeleton, this.pathToVariableDeclartion_);
			} catch (ParsingException e) {
				e.updateError(this.variableChain_.getEndError());
			}
			
		}
		
		// ==== Peer stuff ============

		public BuilderVariableChainPatternPeer addBuilderVariableChainForChain() {
			return this.variableChain_;
		}

		public void end() { }
  } // end of BuilderVariableHandler
 
  // =================================================================================
  private static final class ConstructorCall implements Handler, JavaExpressionPeer.ConstructorCallPatternPeer {
  	private TargetTypeBlock type_;
  	private final ArgumentBucket arguments_;
		private final SpecificCommonErrorOutput baseError_;
  	public ConstructorCall(SpecificCommonErrorOutput baseError) {
  		this.baseError_ = baseError;
  		this.arguments_ = new ArgumentBucket(baseError);
  	}
		public boolean basicBuild(CodeBlockEnvironment environment) {
			boolean succeeded = this.arguments_.basicBuild(environment);
			type_.basicBuild(environment.getBaseLayer().getAsDataBlockEnvironment());
			return succeeded;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			TargetTypeWithPath builtType = this.type_.getResolvedTypeWithPathQuietFinal();
			ArgumentSetDetailsFormBlock argumentDetails = expressionSkeleton.getRootBuilder().createArgumentSetDetailsChild();
			TypeSetterDetailsFormBlock typeDetails = expressionSkeleton.getRootBuilder().createTypeSetterDetailsChild();

			ExpressionDetailsFormBlock.ConstructorCallDataBlock constructorCallSkeleton = expressionSkeleton.addConstructorCall(argumentDetails,typeDetails);
			if(builtType!=null) {
				builtType.buildCodeSetType(typeDetails);
			}
			this.arguments_.finish(environment, argumentDetails);
		}
		// ---------- Peer stuff
		public void end() {			
		}
		
		public TargetCallArgumentsPatternPeer addTargetCallArgumentsForArguments() {
			return this.arguments_;
		}

		public TargetTypeReferenceWithoutArrayPatternPeer addTargetTypeReferenceWithoutArrayForTypeName(
				String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput error = this.baseError_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.type_ = new TargetTypeBlock(typeNameParameter,error);
		}
		
  }
  // =================================================================================

  private static final class UnaryHandler implements Handler, JavaExpressionPeer.PreUnaryPatternPeer, JavaExpressionPeer.PostUnaryPatternPeer {
  	private final boolean isPostAndNotPre_;
  	private final ExpressionBlock baseExpression_;
		private final GeneralSymbolBlock symbol_;
		public UnaryHandler(boolean isPostAndNotPre,  SpecificCommonErrorOutput baseError) {
			this.isPostAndNotPre_ = isPostAndNotPre;
			this.symbol_ = new GeneralSymbolBlock();
			this.baseExpression_ = new ExpressionBlock(baseError);
  	}
		public boolean basicBuild(CodeBlockEnvironment environment) {
			return this.baseExpression_.basicBuild(environment);
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			ExpressionDetailsFormBlock baseSkeleton = expressionSkeleton.getRootBuilder().createExpressionDetailsChild();
			if(this.isPostAndNotPre_) {
				expressionSkeleton.addPostUnary(this.symbol_.getSymbol(),baseSkeleton);
			} else {
				 expressionSkeleton.addPreUnary(this.symbol_.getSymbol(),baseSkeleton);	
			}
			this.baseExpression_.finish(environment, baseSkeleton);
		}
		// ===== Peer Stuff
		public void end() {}

		public JavaExpressionPeer.Indirect getJavaExpressionForValue() {
			return this.baseExpression_;
		}

		public GeneralSymbolPatternPeer addGeneralSymbolForSymbol() {
			return this.symbol_;
		}
  	
  }


  // ==========================================================================
  private static interface Handler {
    public boolean basicBuild(CodeBlockEnvironment environment);
    public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton);
  }
  private static abstract class SimpleHandler implements Handler {
    public final boolean basicBuild(CodeBlockEnvironment environment){ return true; }
  }
  // --------------------------------------------------------------------------
  private static final class AssignmentHandler implements Handler, JavaExpressionPeer.AssignmentPatternPeer {

    private final XCodeCallChainBlock callChain_;
    private final ExpressionBlock valueExpression_;
		public AssignmentHandler( SpecificCommonErrorOutput assignmentError) {
			this.callChain_ = new XCodeCallChainBlock(assignmentError);
    	this.valueExpression_ = new ExpressionBlock(assignmentError);
    }
		public boolean basicBuild(CodeBlockEnvironment environment) {
			return this.valueExpression_.basicBuild(environment);
		}

		public void finish(CodeBlockEnvironment environment,
				ExpressionDetailsFormBlock expressionSkeleton) {
			BaseBuilder root = expressionSkeleton.getRootBuilder();

			ExpressionDetailsFormBlock.BinaryDataBlock binaryBlock = expressionSkeleton.addBinary("=",
					root.createExpressionDetailsChild(),root.createExpressionDetailsChild());			
			this.valueExpression_.finish(environment, binaryBlock.getRightDetailsParameter());
      this.callChain_.resolveInFinish( binaryBlock.getLeftDetailsParameter(), environment);
		}

		public JavaCallChainPatternPeer addJavaCallChainForVariable() { return this.callChain_; }

		public void end() {}

		public JavaExpressionPeer.Indirect getJavaExpressionForValue() { return this.valueExpression_; }
  	
  }
//--------------------------------------------------------------------------
  private static final class BinaryHandler implements Handler, JavaExpressionPeer.BinaryPatternPeer {

    private final ExpressionBlock rightExpression_;
    private final ExpressionBlock leftExpression_;
		private final GeneralSymbolBlock symbol_;
		public BinaryHandler( SpecificCommonErrorOutput binaryError) {
			this.symbol_ = new GeneralSymbolBlock();
    	this.rightExpression_ = new ExpressionBlock(binaryError);
    	this.leftExpression_ = new ExpressionBlock(binaryError);
    }
		public boolean basicBuild(CodeBlockEnvironment environment) {
			boolean leftSucceed = this.leftExpression_.basicBuild(environment);
			boolean rightSucceed = this.rightExpression_.basicBuild(environment);
			return leftSucceed&&rightSucceed;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			BaseBuilder root = expressionSkeleton.getRootBuilder();
			ExpressionDetailsFormBlock.BinaryDataBlock binaryBlock = expressionSkeleton.addBinary(
					this.symbol_.getSymbol(), root.createExpressionDetailsChild(),root.createExpressionDetailsChild());
			
			this.rightExpression_.finish(environment, binaryBlock.getRightDetailsParameter());
			this.leftExpression_.finish(environment, binaryBlock.getLeftDetailsParameter());
  	}

		public JavaExpressionPeer.Indirect getJavaExpressionForLeft() { return leftExpression_; }
		public JavaExpressionPeer.Indirect getJavaExpressionForRight() { return rightExpression_; }
	
		public void end() {}

		public GeneralSymbolPatternPeer addGeneralSymbolForSymbol() {
			return this.symbol_;
		}		
  }
//--------------------------------------------------------------------------
  private static final class GeneralBinaryHandler implements Handler,JavaExpressionPeer.GeneralBinaryPatternPeer{

    private final ExpressionBlock rightExpression_;
    private final ExpressionBlock leftExpression_;
		private final ExpressionBlock symbol_;
		public GeneralBinaryHandler(  SpecificCommonErrorOutput binaryError) {
    	this.rightExpression_ = new ExpressionBlock(binaryError);
    	this.leftExpression_ = new ExpressionBlock(binaryError);
    	this.symbol_ = new ExpressionBlock(binaryError);
		}
		public boolean basicBuild(CodeBlockEnvironment environment) {
			boolean leftSucceed = this.leftExpression_.basicBuild(environment);
			boolean rightSucceed = this.rightExpression_.basicBuild(environment);
			boolean symbolSucceed = this.symbol_.basicBuild(environment);
			return leftSucceed&&rightSucceed&&symbolSucceed;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			BaseBuilder root = expressionSkeleton.getRootBuilder();
			ExpressionDetailsFormBlock.GeneralBinaryDataBlock binaryBlock = expressionSkeleton.addGeneralBinary(
					root.createExpressionDetailsChild(),root.createExpressionDetailsChild(),root.createExpressionDetailsChild()
			);
			this.symbol_.finish(environment, binaryBlock.getSymbolDetailsParameter());

			this.rightExpression_.finish(environment, binaryBlock.getRightDetailsParameter());
			this.leftExpression_.finish(environment, binaryBlock.getLeftDetailsParameter());
  	}

		public JavaExpressionPeer.Indirect getJavaExpressionForLeft() { return leftExpression_; }
		public JavaExpressionPeer.Indirect getJavaExpressionForRight() { return rightExpression_; }
	
		public void end() {}
		public JavaExpressionPeer.Indirect getJavaExpressionForSymbol() {
			return this.symbol_;
		}		
  }

  // --------------------------------------------------------------------------
  private final static class MarkHandler extends AbstractMarkIRecordVariable implements Handler, IRecordVariable {
  	private final String variableName_;
		private final SpecificCommonErrorOutput error_;
		private BExpressionAccessFormHolder.BExpressionAccessFormBlock expressionStoreAccessSkeleton_;
		private JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ExpressionStoreDataBlock expressionStoreOperationSkeleton_;
		public MarkHandler(String variableName, SpecificCommonErrorOutput error) {
			super("expression mark");
			this.variableName_ = variableName;
			this.error_ = error;
  		
  	}
		//Handler bits ---------------------------------------------------------
		public boolean basicBuild(CodeBlockEnvironment environment) {
			JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock skeletonBase = 
				environment.getBaseLayer().getBlockSkeleton().getSkeletonBase();
			this.expressionStoreAccessSkeleton_ = skeletonBase.getRootBuilder().createBExpressionAccessChild();
			try {
				
				this.expressionStoreOperationSkeleton_ = skeletonBase.addExpressionStore(this.variableName_, this.expressionStoreAccessSkeleton_);
				environment.addIRecordVariable(this.variableName_, this);

				return true;
			} catch(ParsingException e) {
				e.updateError(this.error_);

				return false;
			}
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			expressionSkeleton.addExpressionRef(this.expressionStoreAccessSkeleton_, new BaseTrackers.JavaVariablePath());
			
		} 
		// IRecordVariable bits --------

		public boolean isMatchesVariableName(String queryVariableName) { return this.variableName_.equals(queryVariableName);  }
		public String getRoughVariableName() { return this.variableName_; }
		
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			if(type==IVariableType.ExpressionReference) {
				macroCallBlock.addCallParameter().addExpressionSource(this.expressionStoreAccessSkeleton_, variablePath);
			} else {
				throw ParsingException.createGeneralSyntax("expression mark cannot provide macro parameter for "+type.getName());
			}
		}
		public void addAsGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath) throws ParsingException {
			macroCallBlock.addCallParameter().addExpressionSource(this.expressionStoreAccessSkeleton_, variablePath);
		}
		public void addAsMacroRecordMappingParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression mark cannot provide macro parameter for mapping");
			
		}
		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, KeyDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression mark cannot provide macro parameter for key");
			
		}

		public void getAsReference(IRecordTypeTarget target) {
			//TODO: ?
			throw Debug.finishMeMarker();
			
		}
		public SLayer getAsFoundataionNodeReferenceVariable(JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("expression mark cannot provide node reference");
		}
		
  } // end of class MarkHandler
  // --------------------------------------------------------------------------
  private final static class GivenArrayHandler implements Handler, JavaExpressionPeer.GivenArrayPatternPeer {
  	private final SpecificCommonErrorOutput error_;
  	private TargetTypeBlock typeBlock_;
  	private final ArgumentBucket values_;
  	
		public GivenArrayHandler(SpecificCommonErrorOutput error) {
			this.error_ = error;
			this.values_ = new ArgumentBucket(error);
  	}

		public boolean basicBuild(CodeBlockEnvironment environment) {
			typeBlock_.basicBuild(environment.getBaseLayer().getAsDataBlockEnvironment());
			boolean succeeded = this.values_.basicBuild(environment);
			return succeeded;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			BaseBuilder rootBuilder = expressionSkeleton.getRootBuilder();
			TypeSetterDetailsFormBlock arrayTypeSkeleton = rootBuilder.createTypeSetterDetailsChild();
			ArgumentSetDetailsFormBlock valuesSkeleton = rootBuilder.createArgumentSetDetailsChild();
			ExpressionDetailsFormBlock.GivenArrayDataBlock givenArraySkeleton = 
				expressionSkeleton.addGivenArray(arrayTypeSkeleton,valuesSkeleton);
			this.values_.finish(environment, valuesSkeleton);
			TargetTypeWithPath builtType = this.typeBlock_.getResolvedTypeWithPathQuietFinal();

			builtType.buildCodeSetType(arrayTypeSkeleton);
		}

		public TargetCallArgumentsPatternPeer addTargetCallArgumentsForValues() {
			return values_;
		}

		public TargetTypeReferenceWithoutArrayPatternPeer addTargetTypeReferenceWithoutArrayForTypeName(
				String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput typeError = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter,typeError);
		}

		public void end() { }
  } //end of GivenArrayHandler
  // -----------------------------------------------------------------
  private final static class ArrayConstructorHandler implements Handler, JavaExpressionPeer.ArrayConstructorCallPatternPeer {
  	private final SpecificCommonErrorOutput error_;
  	private final SimpleVector<ExpressionBlock> dimensionExpressions_ = new SimpleVector<ExpressionBlock>();
  	private TargetTypeBlock typeBlock_;
  	
		public ArrayConstructorHandler(SpecificCommonErrorOutput error) {
			this.error_ = error;
  	}

		public boolean basicBuild(CodeBlockEnvironment environment) {
			boolean successful = true;
			for(ExpressionBlock d : this.dimensionExpressions_) {
				successful&=d.basicBuild(environment);
			}
			this.typeBlock_.basicBuild(environment.getBaseLayer().getAsDataBlockEnvironment());
			return successful;
		}

		public void finish(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton) {
			BaseBuilder base = expressionSkeleton.getRootBuilder();
			TypeSetterDetailsFormBlock typeDetails = base.createTypeSetterDetailsChild();
			ExpressionDetailsFormBlock.ArrayConstructorDataBlock constructorCallSkeleton = expressionSkeleton.addArrayConstructor(typeDetails);
			TargetTypeWithPath builtType = this.typeBlock_.getResolvedTypeWithPathQuietFinal();
			builtType.buildCodeSetType(typeDetails);
			for(ExpressionBlock d : this.dimensionExpressions_) {
				ExpressionDetailsFormBlock dimensionsExpressionDetails = base.createExpressionDetailsChild();
				constructorCallSkeleton.addDimension().addWithSize(dimensionsExpressionDetails);
				d.finish(environment,dimensionsExpressionDetails);
			}
			
		}
		// --- Peer stuff
		public TargetTypeReferenceWithoutArrayPatternPeer addTargetTypeReferenceWithoutArrayForTypeName(
				String typeNameParameter, int typeNameLineNumber, int typeNameCharacterNumber) {
			SpecificCommonErrorOutput typeError = this.error_.createAdjusted(typeNameLineNumber, typeNameCharacterNumber);
			return this.typeBlock_ = new TargetTypeBlock(typeNameParameter,typeError);
		}

		public void end() {}

		public JavaExpressionPeer.Indirect getJavaExpressionForDimensionExpression() {
			ExpressionBlock d = new ExpressionBlock(this.error_);
			this.dimensionExpressions_.addElement(d);
			return d;
		}
		//===End of Peer stuff
		
  } // End of class ArrayConstructorHandler
  
  // --------------------------------------------------------------------------
  private final static class CallHandler implements Handler, JavaExpressionPeer.CallPatternPeer {
    private final ExpressionBlock parent_;
    private final XCodeCallChainBlock callChain_;
   // private XCodeCallChainBlock.Built built_;
    public CallHandler(ExpressionBlock parent) {
      this.parent_ = parent;     
      this.callChain_ = new XCodeCallChainBlock(parent_.error_);
    }

    public boolean basicBuild(CodeBlockEnvironment environment) {
      return true;
    }
    public void finish(CodeBlockEnvironment environment,ExpressionDetailsFormBlock expressionSkeleton) {
      this.callChain_.resolveInFinish(expressionSkeleton, environment);
    }
    public JavaCallChainPatternPeer addJavaCallChainForValue() {
      return this.callChain_;
    }

    public void end() {
      
    }

  
  }



	

	
}
