package org.ffd2.skeletonx.compile.java;

import java.util.HashMap;

import org.ffd2.skeletonx.austenx.peg.base.GeneralMacroCallPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.JavaCallChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroCallParameterBlockPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.MacroCallParameterPeer.TargetVariablePatternPeer;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class GeneralMacroCall implements GeneralMacroCallPatternPeer{
	private final String	macroCallName_;
	private final SpecificCommonErrorOutput	error_;
  private final MacroCallParameters callValues_;

	public GeneralMacroCall(String macroCallName, SpecificCommonErrorOutput error ){
		this.macroCallName_ = macroCallName;

		this.error_ = error; 
		this.callValues_ = new MacroCallParameters(this);

	}
	//Do in class
	public void finishBuildInClass(RecordClassAccess classRecordAccess) {
		doFinishBuildGeneral(new CodeBlockContext(CodeBlockEnvironment.createRootEnvironment(classRecordAccess)), classRecordAccess.getBaseLayer());
	}
	public void finishBuildOutsideClass(SLayer layerAccess,String currentPackageName) {
		doFinishBuildGeneral(new OutsideClassContext(layerAccess,currentPackageName), layerAccess);	
	}
	
	private final LinkedCodeBlockStoreDataBlock  doFinishBuildGeneral(CallContext callContext, SLayer environment) {
		final LinkedCodeBlockStoreDataBlock linkedCode = environment.allocateLinkedCode();
		try {
			BaseTrackers.JavaVariablePath pathToContainingLayer = new BaseTrackers.JavaVariablePath();
			MacroConstruction macro = environment.getMacroConstructionFinal(this.macroCallName_,pathToContainingLayer);
			//Need to use path in creation of macro call (eg, reference to parent layer)
			//First, create linked in code
			
			//Create call, and give it our reference to the linked in code, when asked.
			MacroCallInstance macroCall = macro.createMacroCall(macro.getName(),environment,new MacroCallAccess(linkedCode,this.callValues_,callContext,this.error_));
			
			//TODO: Needs work for macro call, and macro call variable name 
			Debug.finishMeMarker();

		} catch (ItemNotFoundException e) {
			error_.objectNotFoundError("macro", this.macroCallName_);
		}
		return linkedCode;
	}
	
	//do in code
	public void finishBuild(final CodeBlockEnvironment environment, 	final BaseBuilder.CodeBlockMacroBlock baseCodeSkeleton) {
		LinkedCodeBlockStoreDataBlock linkedCode = doFinishBuildGeneral(new CodeBlockContext(environment), environment.getBaseLayer());
//	final LinkedCodeBlockStoreDataBlock linkedCode = environment.getBaseRecord().allocateLinkedCode();
		baseCodeSkeleton.addStatement().addLinkedCode(linkedCode.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());

//		try {
//			MacroConstruction macro = environment.getMacroConstructionFinal(this.macroCallName_);
//			//First, create linked in code
//			
//			//In the code block, add in the insertion of the linked code
//			baseCodeSkeleton.addLinkedCodeBlock(linkedCode, new BaseTrackers.JavaVariablePath());
//			
//			//Create call, and give it our reference to the linked in code, when asked.
//			MacroCallInstance macroCall = environment.createMacroCallFinal(macro,new MacroCallAccess(linkedCode,this.callValues_,new CodeBlockContext(environment),this.error_));
//			
//			
//			Debug.println("found macro");
//			Debug.finishMeMarker();
//		} catch (ItemNotFoundException e) {
//			error_.objectNotFoundError("macro", this.macroCallName_);
//		}
	}

	public MacroCallParameterBlockPatternPeer addMacroCallParameterBlockForParameters() {
		return this.callValues_;
	}

	public void end() {}
	public void addVariableName(String value, int lineNumber,
			int characterNumber) {
		// TODO Auto-generated method stub
		Debug.finishMeMarker();
	}
	// ==========================================================================
	private static interface CallContext {
		public void accessContextReference(MacroCallDataBlock macroCallBlock) throws ParsingException;
		public void accessGeneralReference(MacroCallDataBlock macroCallBlock) throws ParsingException;
		public JavaVariablePath buildPathToSLayer(SLayer targetLayer);
		public void doAddTargetVarRequirement(CallParameterValue value, TargetType varType,MacroCallDataBlock macroCallBlock);
  	public void doAddRecordVarRequirement(CallParameterValue value, FoundationNode recordType,MacroCallDataBlock macroCallBlock);
  	public void doAddCodeReferenceRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddArgValuesMarkRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddArgTypesMarkRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddStringRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType,CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddIntRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock);
  	public void doAddMappingRequirement(CallParameterValue value,MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock);
  	public void doAddKeyRequirement(CallParameterValue value,KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock);

	}
	
	// --------------------------------------------------------------------------
	private static final class CodeBlockContext implements CallContext {
		private final CodeBlockEnvironment	environment_;

		public CodeBlockContext(final CodeBlockEnvironment environment) {
			this.environment_ = environment;
			
		}
		public JavaVariablePath buildPathToSLayer(SLayer targetLayer) {
			JavaVariablePath path = new JavaVariablePath();
			environment_.getBaseLayer().buildPathToUpperLayer(targetLayer, path );
			return path;
		}
		public void accessContextReference(MacroCallDataBlock macroCallBlock) {
			environment_.addContextRef(macroCallBlock);
		}
		public void accessGeneralReference(MacroCallDataBlock macroCallBlock) {
			environment_.addGeneralRef(macroCallBlock);
		}
		public void doAddTargetVarRequirement(CallParameterValue value, TargetType varType,MacroCallDataBlock macroCallBlock) {
			value.doAddTargetVarRequirement(varType, macroCallBlock, this.environment_);
		}
  	public void doAddRecordVarRequirement(CallParameterValue value, FoundationNode recordType,MacroCallDataBlock macroCallBlock) {
  		value.doAddRecordVarRequirement(recordType, macroCallBlock, this.environment_.getBaseLayer());
  	}
  	public void doAddMappingRequirement(CallParameterValue value,MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock) {
  		value.doAddMappingRequirement(type, macroCallBlock, this.environment_.getBaseLayer());
  	}
  	public void doAddKeyRequirement(CallParameterValue value,KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock) {
  		value.doAddKeyRequirement(type, macroCallBlock, this.environment_.getBaseLayer());

  	}
  	public void doAddCodeReferenceRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddCodeReferenceRequirement(macroCallBlock, this.environment_.getBaseLayer());
  	}
  	public void doAddArgValuesMarkRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddArgValuesMarkRequirement(macroCallBlock, this.environment_.getBaseLayer());
  	}
		public void doAddArgTypesMarkRequirement(CallParameterValue value, 	MacroCallDataBlock macroCallBlock) {
  		value.doAddArgTypesMarkRequirement(macroCallBlock, this.environment_.getBaseLayer());
			
		}
		public void doAddStringRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddStringRequirement(macroCallBlock, this.environment_.getBaseLayer());
		}
		public void doAddIntRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddIntRequirement(macroCallBlock, this.environment_.getBaseLayer());

		}
		public void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType, CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		Debug.println("Doing adding general:",value);
			value.doAddGeneralExpressionTargetRequirement( macroCallBlock, this.environment_.getBaseLayer());			
		}
	}
	// =========================================================================
//--------------------------------------------------------------------------
	private static final class OutsideClassContext implements CallContext {
		private final SLayer	environment_;
		private final String currentPackageName_;
		public OutsideClassContext(final SLayer environment, String currentPackageName) {
			this.environment_ = environment;
			this.currentPackageName_ = currentPackageName;
			
		}
		public void accessContextReference(MacroCallDataBlock macroCallBlock)  throws ParsingException {
			throw ParsingException.createGeneralSyntax("macro requires surrounding class");
		}
		public void accessGeneralReference(MacroCallDataBlock macroCallBlock) throws ParsingException {

			macroCallBlock.addInRoot(currentPackageName_);
		}
		public void doAddTargetVarRequirement(CallParameterValue value, TargetType varType,MacroCallDataBlock macroCallBlock) {
			value.getError().generalSyntaxError("target variables not available outside of a class");
		}
  	public void doAddRecordVarRequirement(CallParameterValue value, FoundationNode recordType,MacroCallDataBlock macroCallBlock) {
//  		Debug.println("*********================Macro record value!");
  		value.doAddRecordVarRequirement(recordType, macroCallBlock, this.environment_);
  	}
  	public JavaVariablePath buildPathToSLayer(SLayer targetLayer) {
			JavaVariablePath path = new JavaVariablePath();
			environment_.buildPathToUpperLayer(targetLayer, path );
			return path;
		}
  	public void doAddCodeReferenceRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddCodeReferenceRequirement(macroCallBlock, this.environment_);
  	}
  	public void doAddMappingRequirement(CallParameterValue value,MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock) {
  		value.doAddMappingRequirement(type, macroCallBlock, this.environment_);
  	}
  	public void doAddArgValuesMarkRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddArgValuesMarkRequirement(macroCallBlock, this.environment_);
  	}
		public void doAddArgTypesMarkRequirement(CallParameterValue value, 	MacroCallDataBlock macroCallBlock) {
  		value.doAddArgTypesMarkRequirement(macroCallBlock, this.environment_);
		}
		public void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType,CallParameterValue value, MacroCallDataBlock macroCallBlock) {
  		value.doAddGeneralExpressionTargetRequirement(macroCallBlock, this.environment_);			
		}
		public void doAddStringRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
			value.doAddStringRequirement(macroCallBlock, this.environment_);
		}
		public void doAddIntRequirement(CallParameterValue value, MacroCallDataBlock macroCallBlock) {
			value.doAddIntRequirement(macroCallBlock, this.environment_);
		}
		public void doAddKeyRequirement(CallParameterValue value,
				KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock) {
			value.doAddKeyRequirement(type,macroCallBlock, this.environment_);
			
		}

	}
	// =========================================================================
	private final static class MacroCallAccess implements MacroCallRequirementsAccess {
		private final LinkedCodeBlockStoreDataBlock linkedCode_;
		private final CallContext callContext_;
		private final MacroCallParameters callVaues_;
		private final SpecificCommonErrorOutput callError_;

		public MacroCallAccess(final LinkedCodeBlockStoreDataBlock linkedCode, final MacroCallParameters callVaues,final CallContext callContext, SpecificCommonErrorOutput callError) {
			this.linkedCode_ = linkedCode;
			this.callVaues_ = callVaues;
			this.callContext_  = callContext;
			
			//FIX THIS SO THAT LINKED CODE IS LINKED SOMEWHERE
			this.callError_ = callError;
		}
		public JavaVariablePath buildPathToSLayer(SLayer targetLayer) {
			return this.callContext_.buildPathToSLayer(targetLayer);
		}
  	public void accessCodeReference(MacroCallDataBlock macroCallBlock) {
			macroCallBlock.addInCode(this.linkedCode_.getVariableStoreParameter(), new BaseTrackers.JavaVariablePath());
		}

		public void accessContextReference(MacroCallDataBlock macroCallBlock) {
			try {
				this.callContext_.accessContextReference(macroCallBlock);
			} catch (ParsingException e) {
				e.updateError(this.callError_);
			}
		}
		public void accessGeneralReference(MacroCallDataBlock macroCallBlock) {
			try {
				this.callContext_.accessGeneralReference(macroCallBlock);
			} catch (ParsingException e) {
				e.updateError(this.callError_);
			}
		}

		public void addTargetVarRequirement(String name, TargetType varType,
				MacroCallDataBlock macroCallBlock) {
			
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddTargetVarRequirement(value, varType,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addRecordVarRequirement(String name, FoundationNode varType, MacroCallDataBlock macroCallBlock){
  		try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddRecordVarRequirement(value,varType,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addCodeReferenceRequirement(String name, MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddCodeReferenceRequirement(value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addArgValuesMarkRequirement(String name,MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddArgValuesMarkRequirement(value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addArgTypesMarkRequirement(String name,MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddArgTypesMarkRequirement(value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
			
		}
		public void addStringRequirement(String name, MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);

				callContext_.doAddStringRequirement(value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addIntRequirement(String name, MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddIntRequirement(value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}	
		public void addKeyRequirement(String name, KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddKeyRequirement(value,type,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}

		public void addMappingRequirement(String name, MappingDefinitionBlock type,
				MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				callContext_.doAddMappingRequirement(value,type,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		public void addGeneralExpressionTargetRequirement(String name,TargetType requiredVarType, MacroCallDataBlock macroCallBlock) {
			try {
				CallParameterValue value = this.callVaues_.getParameterValue(name);
				Debug.println("Call context:",callContext_);
				Debug.println("Value:",value);
				callContext_.doAddGeneralExpressionTargetRequirement( requiredVarType,value,macroCallBlock);
			} catch(ItemNotFoundException e) {
				e.updateError(this.callError_);
			}
		}
		
		
		
	}
	// ==========================================================================
	private final static class MacroCallParameters implements MacroCallParameterBlockPatternPeer{
  	private final GeneralMacroCall parent_;
  	private final HashMap<String,CallParameterValue> definedParameters_ = new HashMap<String,CallParameterValue>();
		public MacroCallParameters(GeneralMacroCall parent) {
			this.parent_ = parent;	
  	}
		private final void addParameter(String name, CallParameterValue parameter, SpecificCommonErrorOutput error) {
			if(this.definedParameters_.containsKey(name)) {
				error.repeatedObjectError("parameter value", name);
			} else {
				this.definedParameters_.put(name, parameter);
			}
		}
		public CallParameterValue getParameterValue(String name) throws ItemNotFoundException {
			CallParameterValue result = this.definedParameters_.get(name);
			if(result==null) {
				throw ItemNotFoundException.createObjectNotFound("parameter value", name);
			} else {
				return result;
			}
		}
		
		public void end() { 	}
		
		public TargetVariablePatternPeer addOptionTargetVariableOfMacroCallParameterForParameter(
				String nameParameter, int nameLineNumber, int nameCharacterNumber ) {
			SpecificCommonErrorOutput error = parent_.error_.createAdjusted(nameLineNumber, nameCharacterNumber);
			TargetVariableCallParameter p = new TargetVariableCallParameter(error);
			addParameter(nameParameter, p,error);
			return p;
		}
		public void addOptionStringBasedOfMacroCallParameterForParameter(String nameValue, int nameLine, int nameCharacter, final String valueValue) {
			final SpecificCommonErrorOutput error = parent_.error_.createAdjusted(nameLine, nameCharacter);
			CallParameterValue p = new AbstractPrimitiveCallParameter("String", error) {
				public void doAddStringRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
					macroCallBlock.addCallParameter().addStringDirect(valueValue);
				}
				public void doAddIntRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
					error.invalidTypeError("String", "integer value");			
				}
			};
			addParameter(nameValue, p,error);			
		}
		public void addOptionIntBasedOfMacroCallParameterForParameter(String nameValue, int nameLine, int nameCharacter, final int valueValue) {
			final SpecificCommonErrorOutput error = parent_.error_.createAdjusted(nameLine, nameCharacter);
			CallParameterValue p = new AbstractPrimitiveCallParameter("String", error) {
				public void doAddStringRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
					macroCallBlock.addCallParameter().addStringDirect(String.valueOf(valueValue));
				}
				public void doAddIntRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
					macroCallBlock.addCallParameter().addIntDirect(valueValue);
				}
			};
			addParameter(nameValue, p,error);			
		}
		public void addOptionThisBasedOfMacroCallParameterForParameter(String name, int nameLine, int nameCharacter) {
			final SpecificCommonErrorOutput error = parent_.error_.createAdjusted(nameLine, nameCharacter);
		 ThisReferenceCallParameter p = new ThisReferenceCallParameter(error);
			addParameter(name, p,error);

		}
  }
//--------------------------------------------------------------------------
  private static interface CallParameterValue {
  	public void doAddTargetVarRequirement(TargetType varType,MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);

  	public SpecificCommonErrorOutput getError();
  	
  	public void doAddRecordVarRequirement(FoundationNode recordType,MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddMappingRequirement(MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddKeyRequirement(KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment);

  	public void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddArgTypesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);

  	public void doAddStringRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddIntRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);
  	public void doAddGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, SLayer environment);
//  	public void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType, final MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment);

  //	public void doAddTargetVarRequirementOutsideClass(TargetType varType,MacroCallDataBlock macroCallBlock, SLayer environment);

  }
  // ==========================================================================
  private final static class ThisReferenceCallParameter implements CallParameterValue {
  	private final SpecificCommonErrorOutput error_;

		public ThisReferenceCallParameter(SpecificCommonErrorOutput error) {
			this.error_ = error;
  		
  	}
		public void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
			error_.invalidTypeError("$this","general expression target");	
		}
		public final void doAddGeneralExpressionTargetRequirement(final MacroCallDataBlock macroCallBlock, SLayer environment){
			error_.invalidTypeError("$this","general expression target");	
		}
		public void doAddTargetVarRequirement(TargetType varType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
			error_.invalidTypeError("$this", "target variable");						
		}

		public SpecificCommonErrorOutput getError() { return this.error_; }

		public void doAddRecordVarRequirement(FoundationNode recordType, MacroCallDataBlock macroCallBlock, SLayer environment) {
			macroCallBlock.addCallParameter().addParentBlockRef(environment.buildPathToFoundationNode(recordType));
		}
		public final void doAddMappingRequirement(MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "mapping");			
		}

		public final void doAddKeyRequirement(KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "key");			
		}

		public final void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "code ref");					
		}

		public final void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "arg values mark");			
		}

		public final void doAddArgTypesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "arg types mark");			
		}


		public void doAddStringRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "string value");			
		}

		public void doAddIntRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError("$this", "int value");			
			
		}

  	
  }
  // ==========================================================================
  private final static class TargetVariableCallParameter implements TargetVariablePatternPeer, CallParameterValue {
  	private final SpecificCommonErrorOutput error_;

		private final XCodeCallChainBlock chainBlock_;
		public TargetVariableCallParameter( SpecificCommonErrorOutput error) {

			this.error_ = error;
			this.chainBlock_ = new XCodeCallChainBlock(error);
		}
		public SpecificCommonErrorOutput getError() { return error_; }
		public JavaCallChainPatternPeer addJavaCallChainForCallChain() {
			return this.chainBlock_;
		}

		public void end() { }
		//The varType is for type checking.
		public void doAddTargetVarRequirement(TargetType varType,
				MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {

			this.chainBlock_.doAddTargetVarRequirement(varType, macroCallBlock, environment);
		}
		public void doAddStringRequirement( final MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement(new RecordVariableAction() {
				public void doAction(IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker) throws ParsingException {
					resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock, pathToVariableChainTracker, IVariableType.StringSource);
				}
				
			}, "string access", environment);
		}
//  	public final void doAddGeneralExpressionTargetRequirement(TargetType requiredVarType, final MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment){
//			Debug.println("Do add general on:",this.chainBlock_);
//  		this.chainBlock_.doAddTargetVarRequirement(requiredVarType, macroCallBlock, environment);
//  	}
  	public final void doAddGeneralExpressionTargetRequirement(final MacroCallDataBlock macroCallBlock, SLayer environment){

   		this.chainBlock_.doAddRequirement(new RecordVariableAction() {
				public void doAction(IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker) throws ParsingException {
					Debug.println("Variable:",resolvedRecordVariable);
					resolvedRecordVariable.addAsGeneralExpressionTargetRequirement(macroCallBlock, pathToVariableChainTracker);
				}
			}, "general expression target", environment);	
		}
		public void doAddIntRequirement( final MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement(new RecordVariableAction() {
				public void doAction(IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker) throws ParsingException {
					resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock, pathToVariableChainTracker, IVariableType.IntSource);
				}
				
			}, "int access", environment);
		}
		
  	public void doAddMappingRequirement(final MappingDefinitionBlock type,final MacroCallDataBlock macroCallBlock, SLayer environment) {
  		this.chainBlock_.doAddRequirement(new RecordVariableAction() {
				public void doAction(IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker) throws ParsingException {
					resolvedRecordVariable.addAsMacroRecordMappingParameter(macroCallBlock, pathToVariableChainTracker, type);
				}
				
			}, "mapping", environment);
  	}

		public void doAddRecordVarRequirement(FoundationNode requiredRecordType,
				MacroCallDataBlock macroCallBlock, SLayer environment) {
//			Debug.println("Adding record vare:",this.chainBlock_);
			this.chainBlock_.doAddRecordVarRequirement(requiredRecordType, macroCallBlock, environment);
		}
		public void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement( new RecordVariableActions.GetCodeReferenceAction(macroCallBlock), "code reference", environment);
		}
		public void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement( new RecordVariableActions.GetArgValuesMarkAction(macroCallBlock), "argument values mark", environment);
		}
	
		public void doAddArgTypesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement( new RecordVariableActions.GetArgTypesMarkAction(macroCallBlock), "argument types mark", environment);
		}
		public void doAddKeyRequirement(final KeyDefinitionBlock type,
			final 	MacroCallDataBlock macroCallBlock, SLayer environment) {
			this.chainBlock_.doAddRequirement(new RecordVariableAction() {
				public void doAction(IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker) throws ParsingException {
					resolvedRecordVariable.addAsMacroRecordKeyParameter(macroCallBlock, pathToVariableChainTracker, type);
				}
				
			}, "key", environment);
		}
	
		
  }
  private static abstract class AbstractPrimitiveCallParameter implements CallParameterValue{
  	private final String	typeName_;
		private final SpecificCommonErrorOutput	error_;

		public AbstractPrimitiveCallParameter(String typeName, SpecificCommonErrorOutput error) {
			this.typeName_ = typeName;
			this.error_ = error;
  		
  	}
  	public final void doAddGeneralExpressionTargetRequirement(MacroCallDataBlock macroCallBlock, SLayer environment){
  		error_.invalidTypeError(this.typeName_, "general target expression");			
		}
  	public final void doAddGeneralExpressionTargetRequirement(  	TargetType requiredVarType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment){
  		error_.invalidTypeError(this.typeName_, "general target expression");			
		}
		public final void doAddTargetVarRequirement(TargetType varType, MacroCallDataBlock macroCallBlock, CodeBlockEnvironment environment) {
			error_.invalidTypeError(this.typeName_, "target variable");			
		}
		public final SpecificCommonErrorOutput getError() { return this.error_; }
		public final void doAddRecordVarRequirement(FoundationNode recordType, MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "record variable");
		}
		public final void doAddMappingRequirement(MappingDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "mapping");			
		}

		public final void doAddKeyRequirement(KeyDefinitionBlock type, MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "key");			
		}

		public final void doAddCodeReferenceRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "code ref");					
		}

		public final void doAddArgValuesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "arg values mark");			
		}

		public final void doAddArgTypesMarkRequirement(MacroCallDataBlock macroCallBlock, SLayer environment) {
			error_.invalidTypeError(this.typeName_, "arg types mark");			
		}

		
	};
}
