package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.GlobalClassDefinitionPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.GlobalElementPeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.GlobalClassSourceFormHolder.GlobalClassSourceFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class GlobalsClass implements GlobalClassDefinitionPatternPeer, RecordItem,GlobalClassReference{
	private final GlobalsBlock elements_;
	private final SpecificCommonErrorOutput	error_;
	private final String	className_;
	private JavaImplementationFormBlock.BlockDataBlock.GlobalClassVariableDataBlock	variableSkeleton_;
	public GlobalsClass(String className, SpecificCommonErrorOutput error) {
		this.className_ = className;
		this.error_ = error;
		this.elements_ = new GlobalsBlock(error);
		
	}
	
	
	// ==========================================================================
	// Austen 
	public void end() { }

	public GlobalElementPeer.Indirect getGlobalElementForElements() {
		return this.elements_;
	}
	// ==========================================================================
	// RecordItem stuff
	public boolean basicBuild(SLayer currentRecord) {
		try {
			this.variableSkeleton_ = currentRecord.addGlobalClassReference(className_, this);
		} catch(ItemAlreadyExistsException e) {
			error_.repeatedObjectError("class global reference", className_);
		}
		return this.elements_.basicBuild(currentRecord, this);
	}
	public void finishBuild() {
		this.elements_.finishBuild(this);
	}
	// ==========================================================================
	// GlobalClassReference stuff
	public boolean isSubTypeOf(TargetType parentType) {
		//TODO: I'm not sure
		Debug.finishMeMarker(); 
		return parentType==this;
	}


 public void buildCodeSetType(TypeSetterDetailsFormBlock target,JavaVariablePath disposableTargetPath) {
  	target.addFromClass(this.variableSkeleton_.getVariableStoreParameter(), disposableTargetPath);
  }
  public String getOutputName() { return className_; }
	
	public DirectTargetElementAccess getDirectElementAccessQuiet() {
		//No direct access - internal class
		return null;
	}
	public TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
		throw ParsingException.createGeneralSyntax("Internal types cannot take generic types");
	}
	public TargetType getArrayed(int dimensions) throws ParsingException {
		return new ArrayedTargetType(this,dimensions);
	}
	
	public void updateSetGlobalFinal(GlobalClassSourceFormBlock source, JavaVariablePath pathToTarget) {
		 source.addSetGlobal(this.variableSkeleton_, pathToTarget);
	}
	
}
