package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.austenx.peg.base.LinkChainPatternPeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.SpecificCommonErrorOutput;


public class LinkChainBlock implements LinkChainPatternPeer { 
  private final SpecificCommonErrorOutput baseError_;

  private final SimpleVector<Element> linkElements_ = new SimpleVector<Element>();
  private final LinkChainBlock previous_;
  public LinkChainBlock( SpecificCommonErrorOutput baseError) {
  	this(baseError,null);
  }
  public LinkChainBlock( SpecificCommonErrorOutput baseError,LinkChainBlock previous) {
    this.baseError_ = baseError;
    this.previous_ = previous;
  }
  public LinkChainBlock getPrevious() { return this.previous_; }
  
  public String toString() { return this.linkElements_.toString(); }
  public void addVariableName(final String name, final  int lineNumber, int characterNumber) {
    final SpecificCommonErrorOutput requestErrorPoint = this.baseError_.createAdjusted(lineNumber, characterNumber);
		addNewElement(new Element() {
			public String toString() { return "$"+name; }

			public SLayer getNextLayerQuiet(SLayer currentLayer) {
				try {
					SLayer result = currentLayer.getVariableChild(name,lineNumber);
					
					return result;
				} catch (ParsingException e) {
					e.updateError(requestErrorPoint);
					return null;
				}
			}
		});
	}
	public void addNodeName(final String name, final int lineNumber, int characterNumber) {
    final SpecificCommonErrorOutput requestErrorPoint = this.baseError_.createAdjusted(lineNumber, characterNumber);
		addNewElement(new Element() {
			public String toString() { return name; }
			public SLayer getNextLayerQuiet(SLayer currentLayer) {
				try {
					return currentLayer.getNodeChild(name,lineNumber);
				} catch(ParsingException e) {
					e.updateError(requestErrorPoint);
					return null;
				}
			}
		});
	}
	public void addIsNotNodeName(String value, int lineNumber, int characterNumber) {
		Debug.finishMeMarker();
	}
  
  public void end() {
  }
  // ==========================================================================
  
  private static interface Element {
  	public SLayer getNextLayerQuiet(SLayer currentLayer);
  }
  private final void addNewElement(Element element) {
  	this.linkElements_.addElement(element);
  }
  
  /**
   * Just used to check if this chain block has actually be used in parsing (used by some code where I'm 
   * nervous about the order of parsing - that it might change in the future)
   * @return
   */
//  public boolean isParsed() {
//    return this.parsed_;
//  }

 
  public SLayer getLinkChainEndContainerQuiet(final SLayer base) {
    SLayer result = base;
    for(Element e : this.linkElements_) {
    	if(result==null) {
    		return result;
    	}
    	result = e.getNextLayerQuiet(result);
    }
    
  	return result;
  }


	
	
	
  
  
//  public BaseTrackers.JavaVariablePath getExtendedLinkVariablePath(BaseTrackers.JavaVariablePath base) {
//    BaseTrackers.JavaVariablePath result = base.getCopy();
//    for(ImplementationRecord l : this.newLinks_) {
//      
//    }
//    return result;
//  }

}
