package org.ffd2.skeletonx.compile.java;


import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.general.StringUtils;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.austenx.peg.base.BuilderVariableChainPatternPeer;
import org.ffd2.skeletonx.austenx.peg.base.NamePeer;
import org.ffd2.skeletonx.austenx.peg.base.NameValuePeer;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock;

public class NameBlock implements NamePeer.BasicPatternPeer {
  private final SimpleVector<Element> elements_ = new SimpleVector<Element>();
  private final SpecificCommonErrorOutput baseError_;
  public NameBlock(SpecificCommonErrorOutput baseError) {
    this.baseError_ = baseError;
  }
  public void updateInstanceNameSkeleton(String defaultName,  NameDetailsFormBlock target,SLayer baseEnvironment) {
  	this.updateNameSkeleton(defaultName, INSTANCE_NAME, target, baseEnvironment);
  }
  public void updateTypeNameSkeleton(String defaultName,  NameDetailsFormBlock target,SLayer baseEnvironment) {
  	this.updateNameSkeleton(defaultName, TYPE_NAME, target, baseEnvironment);
  }
  
  private void updateNameSkeleton(String defaultName, NameStyleHandler styleHandler, NameDetailsFormBlock target,SLayer baseEnvironment) {
//    if(defaultName.equals("void")) {
//    	Thread.dumpStack();
//    }
  	if(this.elements_.isEmpty()) {
    	target.addSimple(defaultName);
    } else {
    	NameDetailsFormBlock.ComplexDataBlock complex = target.addComplex();
	  	boolean isFirst = true;
	  	//Debug.println("Elements:",this.elements_);
	  	for(Element e : this.elements_) {
	      isFirst = e.updateNameSkeleton(complex,styleHandler,isFirst,baseEnvironment);
	    }
    	
    }
  }
  
  private static interface NameStyleHandler {
  	public void addStraight(NameDetailsFormBlock.ComplexDataBlock target, String value, boolean isFirst);
  	public void addViaVariable(
  			NameDetailsFormBlock.ComplexDataBlock target, 
  			IRecordVariable value, BaseTrackers.JavaVariablePath variablePath, boolean isFirst) throws ParsingException;
  	
  }
  private static final NameStyleHandler INSTANCE_NAME =new NameStyleHandler() {
  	public void addStraight(NameDetailsFormBlock.ComplexDataBlock target, String value, boolean isFirst) {
  		if(isFirst) {
        target.addNameElement().addString(StringUtils.decapitalise(value));

  		} else {
        target.addNameElement().addString(StringUtils.getCapatilised(value));
  		}
  	}
  	public void addViaVariable(NameDetailsFormBlock.ComplexDataBlock target, IRecordVariable value, BaseTrackers.JavaVariablePath variablePath, boolean isFirst) throws ParsingException {
  		if(isFirst) {
    		value.updateNameBlock(target,variablePath, IRecordVariable.DECAPITALISED);

  		} else {
    		value.updateNameBlock(target,variablePath, IRecordVariable.CAPITALISED);
  		}
  	}

  };
  private static final NameStyleHandler TYPE_NAME =new NameStyleHandler() {
  	public void addStraight(NameDetailsFormBlock.ComplexDataBlock target, String value, boolean isFirst) {
      target.addNameElement().addString(StringUtils.getCapatilised(value));
  	}
  	public void addViaVariable(NameDetailsFormBlock.ComplexDataBlock target, IRecordVariable value, BaseTrackers.JavaVariablePath variablePath, boolean isFirst) throws ParsingException {
  		value.updateNameBlock(target,variablePath, IRecordVariable.CAPITALISED);
  	}
  };
  
  // ==========================================================================
  
  public NameValuePeer.LinkPatternPeer 
  					addOptionLinkOfNameValueForComponent(  int keywordLineNumber,  int keywordCharacterNumber) {
    // TODO Auto-generated method stub
    Debug.finishMeMarker();

    return null;
  }

  public void addOptionStraightOfNameValueForComponent(final String valueParameter,
                                                       int valueLineNumber,
                                                       int valueCharacterNumber) {
    this.elements_.addElement(new Element() {
      public boolean updateNameSkeleton(NameDetailsFormBlock.ComplexDataBlock target,NameStyleHandler styleHandler, boolean isFirst,SLayer baseEnvironment) {
        styleHandler.addStraight(target, valueParameter, isFirst);
        return false;
      }
      public String toString() { return '"'+valueParameter+'"'; }
    });

  }

  public NameValuePeer.VariablePatternPeer addOptionVariableOfNameValueForComponent() {
    VariableOption variable = new VariableOption(this.baseError_);
    this.elements_.addElement(variable);
    return variable;
  }

  public void end() {
  }
  // ==========================================================================
  private static interface Element {
  	//Could add simple, one element stuff...
  	/**
  	 * @return true if still first variable,else false
  	 */
  	public boolean updateNameSkeleton(NameDetailsFormBlock.ComplexDataBlock target,NameStyleHandler styleHandler, boolean isFirst,SLayer baseEnvironment);
  }
  private final static class VariableOption implements Element, NameValuePeer.VariablePatternPeer {
    private final BuilderVariableChainBlock variableChain_;
    public VariableOption(SpecificCommonErrorOutput baseError) {
      this.variableChain_= new BuilderVariableChainBlock(baseError);
    }
    public BuilderVariableChainPatternPeer addBuilderVariableChainForRef() {
      return variableChain_;
    }
    public boolean updateNameSkeleton(NameDetailsFormBlock.ComplexDataBlock target,NameStyleHandler styleHandler, boolean isFirst,SLayer baseEnvironment) {
      BaseTrackers.JavaVariablePath variablePath = new BaseTrackers.JavaVariablePath();
      IRecordVariable resolved = variableChain_.resolveRecordVariableQuiet(baseEnvironment,variablePath,true);
    	 if(resolved!=null) {
      	try {
      		styleHandler.addViaVariable(target, resolved, variablePath, isFirst);
      	} catch(ParsingException e) {
      		e.updateError(variableChain_.getEndError());
      	}
      }
    	return false;
    }
    public void end() {  }
    
  }

}
