package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.compile.java.layer.FoundationNode;
import org.ffd2.skeletonx.compile.java.layer.SLayer;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

public class RecordVariableActions {
	// ==========================================================================
	public static final class GetRecordAction implements RecordVariableAction {
		private final FoundationNode	requiredRecordType_;
		private final MacroCallDataBlock	macroCallBlock_;
		public GetRecordAction(FoundationNode requiredRecordType,MacroCallDataBlock macroCallBlock) {
			this.requiredRecordType_ = requiredRecordType;
			this.macroCallBlock_ = macroCallBlock;
			
		}
		public void doAction( IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker ) throws ParsingException {
			SLayer variableFoundationNode = resolvedRecordVariable.getAsFoundataionNodeReferenceVariable(null);//pathToVariableChainTracker);
			if(variableFoundationNode!=this.requiredRecordType_.getRelatedLayerForLayerComponents()) {
				throw ParsingException.createInvalidType(variableFoundationNode.getFullName()+" record", this.requiredRecordType_.getFullName());
			} else {
				resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock_, pathToVariableChainTracker, IVariableType.RecordReference);
			}
		}
	} // End of GetRecordAction
	// ==========================================================================
	public static final class GetCodeReferenceAction implements RecordVariableAction {
		private final MacroCallDataBlock	macroCallBlock_;
		public GetCodeReferenceAction(MacroCallDataBlock macroCallBlock) {
			this.macroCallBlock_ = macroCallBlock;
		}
		public void doAction( IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker ) throws ParsingException {

			resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock_, pathToVariableChainTracker, IVariableType.CodeMarkReference);
		}
	} // End of GetRecordAction
//==========================================================================
	public static final class GetArgValuesMarkAction implements RecordVariableAction {
		private final MacroCallDataBlock	macroCallBlock_;
		public GetArgValuesMarkAction(MacroCallDataBlock macroCallBlock) {
			this.macroCallBlock_ = macroCallBlock;
		}
		public void doAction( IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker ) throws ParsingException {

			resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock_, pathToVariableChainTracker, IVariableType.ARGUMENT_VALUES_REFERENCE);
		}
	} // End of GetRecordAction
//==========================================================================
	public static final class GetArgTypesMarkAction implements RecordVariableAction {
		private final MacroCallDataBlock	macroCallBlock_;
		public GetArgTypesMarkAction(MacroCallDataBlock macroCallBlock) {
			this.macroCallBlock_ = macroCallBlock;
		}
		public void doAction( IRecordVariable resolvedRecordVariable, JavaVariablePath pathToVariableChainTracker ) throws ParsingException {

			resolvedRecordVariable.addAsMacroRecordParameter(macroCallBlock_, pathToVariableChainTracker, IVariableType.ParametersReference);
		}
	} // End of GetRecordAction
}
