package org.ffd2.skeletonx.compile.java;

import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TypeSetterDetailsFormHolder.TypeSetterDetailsFormBlock;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;

public class StandardTargetTypes {
  public static final TargetType VOID_TARGET_TYPE = new AbstractPrimitive("void") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock target, JavaVariablePath disposableTargetPath) {
      target.addVoid();      
    }  
    public boolean isSubTypeOf(TargetType parentType) {
    	return  parentType==this;
		}
  };
  public static final TargetType LONG_TARGET_TYPE = new AbstractPrimitive("long") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addLong();
    } 
    public boolean isSubTypeOf(TargetType parentType) {
    	return  parentType==this;
		}
  };
  public static final TargetType INT_TARGET_TYPE = new AbstractPrimitive("int") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addInt();
    } 
    public boolean isSubTypeOf(TargetType parentType) {
    	return parentType==DOUBLE_TARGET_TYPE|| parentType==this;
		}
  };
  public static final TargetType BYTE_TARGET_TYPE = new AbstractPrimitive("byte") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addByte();
    } 
    public boolean isSubTypeOf(TargetType parentType) {
    	return parentType==DOUBLE_TARGET_TYPE||parentType==INT_TARGET_TYPE|| parentType==this;
		}
  };
  public static final TargetType BOOLEAN_TARGET_TYPE = new AbstractPrimitive("boolean") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addBoolean();
    }  
    public boolean isSubTypeOf(TargetType parentType) {
    	return parentType==this;
		}
  };
  public static final TargetType CHAR_TARGET_TYPE = new AbstractPrimitive("char") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addChar();
    }  
    public boolean isSubTypeOf(TargetType parentType) {
    	return parentType==INT_TARGET_TYPE|| parentType==this;
		}
  };
  public static final TargetType DOUBLE_TARGET_TYPE = new AbstractPrimitive("double") {
    public void buildCodeSetType(TypeSetterDetailsFormBlock  target, JavaVariablePath disposableTargetPath) {
      target.addDouble();
    } 
    public boolean isSubTypeOf(TargetType parentType) {
    	return parentType==this;
		}
  };
  
  private abstract static class AbstractPrimitive implements TargetType {
    private final String typeName_;

    public AbstractPrimitive(String typeName) {
      this.typeName_ = typeName; 
    }
    public final TargetType getArrayed(int dimensions) throws ParsingException {
  		return new ArrayedTargetType(this,dimensions);
  	}
  	public DirectTargetElementAccess getDirectElementAccessQuiet() {
  		return null;
  	}
  	public final TargetType specifyGenericTypes(TargetTypeWithPath[] types) throws ParsingException {
  		throw ParsingException.createGeneralSyntax(typeName_+" does not take generic types");
  	}

    public String getOutputName() { return typeName_; }

    public VariableAccess getTargetVariableQuiet(boolean allowLocal, TargetType targetType, String name, BaseTrackers.JavaVariablePath pathToDeclaration){
      return null;
    }

    public MethodAccess getTargetMethodQuiet(TargetType targetType, String name, BaseTrackers.JavaVariablePath pathToDeclaration) {
      return null;
    }
  }

  
  /**
   * Get a standard target type by name
   * @param name
   * @return null, if no known target type
   */
  public static final TargetType getStandardTypeQuiet(String name) {
    if(name.equals("void")) {
      return VOID_TARGET_TYPE;
    } else if(name.equals("double")) {
      return DOUBLE_TARGET_TYPE;
    } else if(name.equals("int")) {
      return INT_TARGET_TYPE;
    } else if(name.equals("char")) {
    	return CHAR_TARGET_TYPE;
    } else if(name.equals("long")) {
    	return LONG_TARGET_TYPE;
    } else if(name.equals("byte")) {
    	return BYTE_TARGET_TYPE;
    } else if(name.equals("boolean")) {
      return BOOLEAN_TARGET_TYPE;
    } else {
      return null;
    }
  }
  /**
   * Get a standard target type by name
   * @param name
   * @return null, if no known target type
   */
  public static final TargetType getStandardType(String name) throws ItemNotFoundException {
    TargetType result = getStandardTypeQuiet(name);
    if(result!=null) {
      return result;
    } else {
      throw new ItemNotFoundException(name, "target type", name);
    }
  }
}
