package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;

//This is the LayerType for general Foundation Layers (excluding the RootLayer)
/**
 * When accessing things (like method/variables) a foundation layer only needs to check along the layers, as the associated layer is the layer of Foundation node
 */
public class FoundationLayerType implements LayerType {
	
	private final SLayer childLayer_;
	private final SLayer parentLayer_;
	private final FoundationNode	childThatIsMeNode_;
	private final FoundationNode	parentNode_;
	private final FoundationNodeChildReference	childNodeReferenceInParent_;
	private final String	nodeName_;
	private final ImplementationBlockSkeletonManager	blockSkeleton_;
	
	public FoundationLayerType(String nodeName, SLayer parentLayer, FoundationNode parentNode, FoundationNode childNode, ImplementationBlockSkeletonManager blockSkeleton,FoundationNodeChildReference childNodeReferenceInParent, FileEnvironment environment) {
		this.nodeName_ = nodeName;
		this.parentLayer_ = parentLayer;
		this.blockSkeleton_ = blockSkeleton;
		this.childLayer_ = new SLayer(this,SLayer.NORMAL_CHILD_MAKER,blockSkeleton,parentLayer.getRelatedTracker(), environment);
		this.parentNode_ = parentNode;
		this.childThatIsMeNode_ = childNode;
		this.childNodeReferenceInParent_ = childNodeReferenceInParent;
		parentLayer.forceFoundationNodeChild(nodeName,childNode,childLayer_,childNodeReferenceInParent);

	}
	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
		return childThatIsMeNode_;
	}
	public boolean buildHorizontalPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
	  return this.childLayer_==target;
	}
	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
		Thread.dumpStack();
		return false;
	}

	public IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
		this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getLayerStrictBuilderVariableQuiet(name, pathToVariable);
	}
	public String getFullName() { return parentLayer_.getFullName()+":"+nodeName_; }

	public SLayer getAncestorNodeSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		if(this.nodeName_.equals(name)) {
			return this.childLayer_;
		} else {
			this.childNodeReferenceInParent_.addBackLink(pathToParentLayer);
			return this.parentLayer_.getAncestorNodeSLayer(name, pathToParentLayer);
		}
	}
	public SLayer getAncestorVariableSLayerBackwards(String name, JavaVariablePath pathToParentLayer) throws ParsingException {
		throw ParsingException.createObjectNotFound("Variable SLayer", name);
	}
	public String toString() { return "Foundation["+this.nodeName_+"]"; }
	public SLayer getChildLayer() {
		return childLayer_;
	}
	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		this.childNodeReferenceInParent_.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, targetVariablePathToFoundationChildrenChainTracker);
	}
	public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
  	return parentLayer_.getChainDefinition(name);
  }
	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
  	return parentLayer_.getMappingDefinition(name);
  }
	public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
  	return parentLayer_.getKeyDefinition(name);
  }
	public FoundationNode getRootFoundationNodeBasic() { return parentLayer_.getRootFoundationNodeBasic(); }

	public GlobalClassReference getGlobalClassReferenceQuiet(String className, 	JavaVariablePath pathToVariable, boolean isFirstLayer) {
		this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getGlobalClassReferenceQuiet(className, pathToVariable, false);
	}

	public GlobalMethodReference getGlobalMethodReferenceQuiet(GlobalClassReference baseClass, String methodName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getGlobalMethodReferenceQuiet(baseClass,methodName, pathToVariable, false);
	}
	public void checkVariableChildOkay(IRecordVariable referencingVariable) {}
	public AncestorConnection getAncestorFoundationNodeQuiet(String foundationNodeName, BaseTrackers.JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer) {
		//Get the FoundationNode to do the work (as Layers==Nodes in this case)
		return this.childThatIsMeNode_.getAncestorConnectionQuiet(foundationNodeName, pathToAncestorNodeParent,true);
	}

//	public void basicBuildNodeChildren(LinkupTracker nodeBuilder, ImplementationBlockSkeletonManager parentBlockSkeleton, AccessStoreX<NodeLayerChild> children) {
//		for(NodeLayerChild c : children) {
//			//Remember that this is called by the childLayer - these "children" are the grandchildren of the parent stuff in this class
//			c.basicBuildFromFoundationLayerParent(nodeBuilder, this.childLayer_, parentBlockSkeleton, this.childNode_);
//		}
//	}



//	public void addPathElementsToFoundationNodePeer(JavaVariablePath pathStore) {
//		//Nothing to do, as we *ARE* the FoundationNodeLayer....
//	}

	public void accessInternalType(TargetType outputClass, JavaVariablePath chainToVariableStore) {
		throw new RuntimeException("THIS SHOULD NOT HAPPEN!");
	}
	public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		//We don't use the foundations ancestor check, because there maybe layer variables defined that won't be picked up that way)
		IRecordVariable foundationVariable = this.childThatIsMeNode_.getBuilderParameterQuiet(name, pathToVariable, false);
		if(foundationVariable!=null) {
			return foundationVariable;
		}
		
		this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getBuilderVariableQuiet(name, pathToVariable, false);
	}
	
  public TargetType getTargetTypeQuiet( String typeName, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
  	//Try getting the type off the related Foundation Node (for Globals and such like). 
//  	Object pathMark = pathToVariable.mark();
//  	TargetType foundationType = this.childNode_.getFoundationNodeTiedTargetTypeQuiet(typeName, pathToVariable,false);
//  	if(foundationType!=null) {
//  		return foundationType;
//  	}
//  	pathToVariable.unmark(pathMark);
  	
  	this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getTargetTypeQuiet(typeName, pathToVariable,false);
  }
  public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope,  String variableName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
  	this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getLocalVariableQuiet(relatedScope,variableName,  pathToVariable,false);
  }
	
	public boolean buildPathToType(TargetType type, JavaVariablePath pathStore, boolean isFirstLayer) {
		this.childNodeReferenceInParent_.addBackLink(pathStore);
		return parentLayer_.buildPathToType(type, pathStore,false);
	}
	public TargetVariable getTargetVariableQuiet(
				boolean allowLocalVariables, TargetType targetType, String variableName, JavaVariablePath chainToVariable, boolean isFirstLayer
			) {
		this.childNodeReferenceInParent_.addBackLink(chainToVariable);
		return parentLayer_.getTargetVariableQuiet(allowLocalVariables, targetType, variableName, chainToVariable,false);
	}
	@Override
	public TargetMethod getTargetMethodQuiet(TargetType outputClass, String typeName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		//Try getting the method off the related Foundation Node (for Globals and such like). 
//  	Object pathMark = pathToVariable.mark();
//  	TargetMethod foundationType = this.childNode_.getFoundationNodeTiedTargetMethodQuiet(outputClass, typeName, pathToVariable,false);
//  	if(foundationType!=null) {
//  		return foundationType;
//  	}
//  	pathToVariable.unmark(pathMark);
		
		this.childNodeReferenceInParent_.addBackLink(pathToVariable);
		return parentLayer_.getTargetMethodQuiet(outputClass, typeName, pathToVariable,false);
	}
	public MacroConstruction getMacroConstructionFinal(String name,BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
	//Debug.println("Get macro called:",name);
		this.childNodeReferenceInParent_.addBackLink(pathToContainingLayer);
		return parentLayer_.getMacroConstructionFinal(name,pathToContainingLayer);
	}

	public void buildPathToUpperLayerBack(SLayer ancestor, JavaVariablePath path) {
		this.childNodeReferenceInParent_.addBackLink(path);
		this.parentLayer_.buildPathToUpperLayer(ancestor, path);
	}
	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path) {
		if(target!=this.childThatIsMeNode_) {
			this.childNodeReferenceInParent_.addBackLink(path);
			return this.parentLayer_.buildPathToFoundationNode(target, path);
			//else found
		} else {
			return true;
		}

	}
	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		if(target==this.childThatIsMeNode_.getGlobalSLayer()) {
			return true;
		} else {
			this.childNodeReferenceInParent_.addBackLink(path);
			return this.parentLayer_.buildPathToPeerLayer(target, path);
			//else found
		}
	}
	public boolean isLocalVariableQuiet(ScopeContainer relatedScope, String variableName, boolean isFirstLayer) {
		return parentLayer_.isLocalTargetVariableQuiet(relatedScope, variableName,false);
	}
	

	

}
