package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.BlockImplementationTarget;
import org.ffd2.skeletonx.compile.impl.DesignBridgeComponents.DesignRecordReference;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.DataBlockEnvironment;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.JavaImplementationBlock;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.VariableStore;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleStringBuffer;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

public class FoundationNode  implements BlockImplementationTarget{
	private final LinkupTracker relatedTracker_;
	private final AccessStoreX<FoundationNodeChildReference> children_;
//  private final AccessStoreX<GlobalClassReference> globalClassReferences_;
//  private final AccessStoreX<MethodReferenceHolder> globalMethodReferences_;
	private final FileEnvironment	environment_;
	private final NodeType type_;
	
	private final DesignRecordReference reference_;
	 
	private final JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock baseSkeleton_;
	private final VariableStore parameterVariables_;
  //Internal constructor
  private FoundationNode( 
  		DesignRecordReference reference,
  		LinkupTracker relatedTracker, FileEnvironment environment,
  		InteractingBitsBuilder bitsBuilder
  	) {
//  		String nodeName, DesignRecordReference reference,TargetBaseForm.BlockDataBlock.ChildRefDataBlock childRefInParent,
//  		JavaImplementationForm.BlockDataBlock parentBlockSkeleton,
//  		NodeTypeBuilder typeBuilder) {
  	this.reference_ = reference;
  	this.parameterVariables_ = new VariableStore(environment);
  	this.relatedTracker_ = relatedTracker; this.environment_ = environment;
  	this.children_ = this.environment_.createEntityStore("foundation node");
//	  this.globalClassReferences_ = this.environment_.createEntityStore("global class");
//	  this.globalMethodReferences_ = this.environment_.createEntityStore("global method");
	  this.baseSkeleton_ = relatedTracker.instantiateDesignNode(reference, this);

    this.type_ = bitsBuilder.constructType(this,this.baseSkeleton_,environment);
    	//new InternalType(nodeName, reference,childRefInParent,parentBlockSkeleton,parentNode.relatedTracker_,this,parentNode);
  }
  public DataBlockEnvironment getAsBuilderVariableAccess(){
  	return this.type_.getRelatedLayer().getAsDataBlockEnvironment();
  }
  public JavaImplementationFormBlock.BlockDataBlock getBaseBlockSkeleton(){ 
  	return this.baseSkeleton_.getParent();
  }
	public  JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock.RootTypeDataBlock addSearchBlockForThisNodeAsTarget(ImplementationBlockSkeletonManager targetBuilder, int searchLineNumber) {
		return targetBuilder.addSearchBlock(this.baseSkeleton_.getParent(),searchLineNumber).addRootType(this.baseSkeleton_);
	}
	public IRecordVariable getDirectField(String fieldName) throws ItemNotFoundException {
		return this.parameterVariables_.getBuilderVariable(fieldName);
	}
  /**
   * Create a Record parameter, with the type matching this FoundationNode
   * @param baseParameterSkeleton
   * @return
   */
	public JavaParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock 
							createRecordParameter(JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton) {
		return baseParameterSkeleton.addRecord(this.baseSkeleton_);
	}
//  public static final class DirectChildLink {
//  	
//  }
//  public DirectChildLink getDirectChildQuiet(String name ) {
//  	NodeChildReference childRef = this.children_.getQuiet(name);
//  	
//  }
  public String getFullName() {
  	SimpleStringBuffer buffer = new SimpleStringBuffer();
  	backAppendName(buffer);
  	return buffer.toString();
  }
  public JavaParameterSettingsFormBlock.ParameterDataBlock.GeneralRecordDataBlock buildGeneralRecordRefParameter(JavaParameterSettingsFormBlock.ParameterDataBlock base) {
  	return base.addGeneralRecord(baseSkeleton_.getParent());
  }
//  public ParameterSettingsFormBlock.ParameterDataBlock.GeneralRecordDataBlock buildGeneralRecordRefParameter(ParameterSettingsFormBlock.ParameterDataBlock base) {
//  	return base.addGeneralRecord(baseSkeleton_.getParent());
//  }
  void backAppendName(SimpleStringBuffer buffer) {
  	this.type_.backAppendName(buffer);

  }
  public JavaImplementationFormBlock.ChainDataBlock.LinkRefDataBlock doAddLinkRefBlock(JavaImplementationFormBlock.ChainDataBlock chainImplementationSkeleton, String linkName) {
  	return chainImplementationSkeleton.addLinkRef(linkName, this.baseSkeleton_);
  }
  //TODO: one needs to go...
  public SLayer getGlobalSLayer() {
  	return this.type_.getRelatedLayer();
  }
	public SLayer getRelatedLayerForLayerComponents() {
		return this.type_.getRelatedLayer();
	}
	public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathStore, boolean isFirstLayer) {
		return this.type_.getRelatedLayer().buildPathToType(type, pathStore, isFirstLayer);
	}
//Internal constructor
//  private FoundationNode( 
//  		DesignRecordReference reference,
//  		LinkupTracker relatedTracker, FileEnvironment environment,
//  		JavaImplementationBlock parentBlock) {
//  	this.reference_ = reference;
//  	this.relatedTracker_ = relatedTracker; this.environment_ = environment; this.children_ = environment.createEntityStore();
//	  this.globalClassReferences_ = environment.createEntityStore();
//	  this.globalMethodReferences_ = environment.createEntityStore();
//	  this.builderVariables_ = this.environment_.createEntityStore();
//
//    this.type_ = new RootType(reference,relatedTracker,this,parentBlock);
//  }
  // ==========================================================================
  // Constructor public access sstuff
  public static final RootFoundationNodeAccess createInvisibleRoot(final DesignRecordReference reference, final LinkupTracker relatedTracker, FileEnvironment environment, final JavaImplementationBlock parentBlock, final SpecificCommonErrorOutput baseError) {

    RootFoundationNodeAccess result = new RootFoundationNodeAccess(reference, relatedTracker, parentBlock);
    //The strange following code is all part of a subtle plan to intricately link objects...
    FoundationNode resultingNode = new FoundationNode(reference,relatedTracker,environment,result);
  	if(resultingNode!=result.getBuiltNode()) {
  		throw new RuntimeException("Assertion error: something went wrong with construction");
  	}
    return result;
  }
  // --------------------------------------------------------------------------
  //This is fool of very strange back and forward referencing stuff 
  public final static class RootFoundationNodeAccess implements InteractingBitsBuilder {
		private final DesignRecordReference	reference_;
		private final LinkupTracker	relatedTracker_;
		private final JavaImplementationBlock	parentBlock_;
		private  RootType rootType_;
		private FoundationNode	relatedNode_;
		private SLayer relatedLayer_;
		public RootFoundationNodeAccess(final DesignRecordReference reference, final LinkupTracker relatedTracker,final JavaImplementationBlock parentBlock) {
			this.reference_ = reference;
			this.relatedTracker_ = relatedTracker;
			this.parentBlock_ = parentBlock;
  		
  	}
		public NodeType constructType(FoundationNode relatedNode, IsDesignLinkedDataBlock baseSkeleton, FileEnvironment environment) {
			this.relatedNode_ = relatedNode;
			ImplementationBlockSkeletonManager blockSkeleton = new ImplementationBlockSkeletonManager(baseSkeleton.getParent(), relatedTracker_, environment);
			this.relatedLayer_ = new RootLayerType(relatedNode,blockSkeleton,reference_.getDesignRecordBackLink(),parentBlock_,relatedTracker_,environment).getRootLayer();
	
			return this.rootType_ = new RootType(this.reference_, relatedLayer_, relatedTracker_, baseSkeleton, parentBlock_);
		}
		public FoundationNode getBuiltNode() {
			return this.relatedNode_;
		}
		public SLayer getRootLayer() {
			return this.relatedLayer_;
		}
		/**
		 * Builds both the FoundationNode tree, and the Layer stack/tree.
		 * Needed to assign foundation paramaters - call after construction of Foundation tree
		 * (and before doing any allocation of globals)
		 */
  	public void buildRemainderIncludingParameters() {
  		this.relatedNode_.basicBuild();
  	}

  }
  // ==========================================================================
  // Stuff for GlobalReferences
  // --------------------------------------------------------------------------
  // Global Class Reference
  
//  public GlobalClassReference getGlobalClassReference(String className, BaseTrackers.JavaVariablePath pathToVariable) throws ItemNotFoundException {
//		return this.globalClassReferences_.get(className);
//	}
//  public GlobalClassReference getGlobalClassReferenceQuiet(String className, BaseTrackers.JavaVariablePath pathToVariable)  {
//		return this.globalClassReferences_.getQuiet(className);
//	}
//	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath classPath) {
//		MethodReferenceHolder base = this.globalMethodReferences_.getQuiet(methodName);
//		if(base!=null) {
//			return base.getMethodQuiet(baseClass);
//		} else {
//			//Add in check ancestors stuff her
//			return null;
//		}
//	}
	/**
	 * MethodAccess base = this.getTargetMethodQuiet(baseClass, methodName, classPath);
		if(base!=null) {
			return base.getMethod().getAsGlobalQuiet();
		} else {
			return null;
		}
	 * @param name
	 * @param reference
	 * @return
	 * @throws ItemAlreadyExistsException
	 */
//	public JavaImplementationForm.BlockDataBlock.GlobalClassVariableDataBlock addGlobalClassReference(String name, GlobalClassReference reference) throws ItemAlreadyExistsException {
//		JavaImplementationForm.BlockDataBlock.GlobalClassVariableDataBlock skeleton = 
//			type_.getSkeletonManager().addGlobalClassVariableBlock(name);
//
//		this.globalClassReferences_.put(name, reference);
////		this.basicContainer_.addType(reference, name);
//		// We do this later when attaching to a layer...
//		return skeleton;
//	}
//	public JavaImplementationForm.BlockDataBlock.GlobalMethodVariableDataBlock addGlobalMethodReference(String methodName,GlobalClassReference baseClass, GlobalMethodReference method ) throws ItemAlreadyExistsException {
//		JavaImplementationForm.BlockDataBlock.GlobalMethodVariableDataBlock skeleton = this.type_.getSkeletonManager().addGlobalMethodVariableBlock(methodName) ;
//		MethodReferenceHolder holder = this.globalMethodReferences_.getQuiet(methodName);
//    
//    if(holder!=null&&holder.isContains(baseClass)) {
//      throw new ItemAlreadyExistsException();
//    } else {
//      holder = new MethodReferenceHolder(baseClass, method, holder);
//      this.globalMethodReferences_.force(methodName, holder); //Could optimise
//      Debug.futureFeatureMarker("Could optimise!");
//      return skeleton;
//    }
//  }
//  
  /**
   * Auto calls addClassMethod()...
   * @param containingTargetClass
   * @param methodName
   * @param method
   * @throws ItemAlreadyExistsException
   */
//  public void addClassMethodReference(GlobalClassReference containingTargetClass, String methodName, GlobalMethodReference method) throws ItemAlreadyExistsException {
//  	//I think more could be done here
//  	addClassMethod(containingTargetClass, methodName, method);
//
//  }


  // ==========================================================================
  // General Public stuff
//	public JavaImplementationrelatedLayerLink_Form.BlockDataBlock.IsDesignLinkedDataBlock getDesignLinkedSkeleton() {
//		return type_.getDesignLinkedSkeleton();
//	}
	public ImplementationBlockSkeletonManager getSkeletonManager() {
		return type_. getSkeletonManager() ;
	}
	
	//After construction, for building the parameters
	public void basicBuild() {
		this.reference_.buildImplementationParameters(this.type_.getSkeletonManager().getAsParameterTarget(this.parameterVariables_));
	  for(FoundationNodeChildReference child : this.children_) {
	  	child.getChildNode().basicBuild();
	  }
	}

  // --------------------------------------------------------------------------
  public FoundationNode getChildQuiet(String childName) { 
  	FoundationNodeChildReference childReference = this.children_.getQuiet(childName);
  	if(childReference!=null) {
  		return childReference.getChildNode(); 
  	} else {
  		return null;
  	}
  }
  
  public FoundationNode getChild(String childName) throws ItemNotFoundException { return this.children_.get(childName).getChildNode(); }
  /**
   * Just redirects to the "type" - thus - if a FNode with a parent layer, just redirects to parent
   * @param name
   * @param path
   * @param checkAncestors if true looks back up tree (building variable) for connection
   * @return
   */
  public AncestorConnection getAncestorConnectionQuiet(String name, BaseTrackers.JavaVariablePath path, boolean checkAncestors) {
  	FoundationNodeChildReference child = this.children_.getQuiet(name);
  	if(child!=null) {
  		return new AncestorConnection(this,child);
  	} else if(checkAncestors){
  		return type_.getAncestorBackQuiet(name, path);
  	} else {
  		return null;
  	}
  }
  // ==========================================================================
  final static class InternalTypeBuilder implements InteractingBitsBuilder{
  	private final String	childName_;
		private final FoundationNode	parent_;
		private final DesignRecordReference	reference_;
		private final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock	childRefInParent_;
		private FoundationNodeChildReference resultType_ = null;
		private final SLayer	parentLayer_;
		
		public InternalTypeBuilder(
				final String childName, final DesignRecordReference reference,final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent,
				final SLayer parentLayer,
				final FoundationNode parent
			) {
			this.childName_ = childName;
			this.reference_ = reference;
			this.childRefInParent_ = childRefInParent;
			this.parentLayer_ = parentLayer;
			this.parent_ = parent;
  		
  	}
		public NodeType constructType(FoundationNode relatedNode, JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock baseSkeleton, FileEnvironment environment) {			
			FoundationNodeChildReference resultType = new FoundationNodeChildReference(
				parentLayer_,
				childName_,reference_,childRefInParent_,baseSkeleton,
				parent_.type_.getDesignLinkedSkeleton().getParent(),parent_.relatedTracker_,parent_,relatedNode,environment
			);
			this.resultType_ = resultType;
			return resultType;
		}
		public FoundationNodeChildReference getResultingType() {
			return this.resultType_;
		}
  } //End of class InternalTYPE
  // ==========================================================================
	private final FoundationNode doBasic(
  		final String childName, final DesignRecordReference reference,final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent
  	) {
  	final InternalTypeBuilder typeBuilder = this.type_.createChildTypeBuilder(childName, reference, childRefInParent, this);
  	FoundationNode child = 
  		new FoundationNode(
  				reference,this.relatedTracker_, this.environment_,
  				typeBuilder
  		);
  	FoundationNodeChildReference resultingType = typeBuilder.getResultingType();
  	this.children_.force(childName, resultingType);

    return child;
  }
 
  public BlockImplementationTarget newInternalChild(String childName, DesignRecordReference reference, TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent) {
  	FoundationNode child = doBasic(childName,reference,childRefInParent);
    return child;
  }
  public void newTerminalChild(String childName, DesignRecordReference reference, TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent) {
  	doBasic(childName,reference,childRefInParent);
  }   
 
  //TODO - do I need this
  @Deprecated
	public boolean buildPathToTypeDep(TargetType type, JavaVariablePath pathStore, boolean checkAncetors) { 
//		if(this.globalClassReferences_.isContainsValue(type)) {
//			return true; //Caller is assumed to be "pointing" at this node
//		} if(checkAncetors) {
//			return this.type_.backBuildPathToType(type, pathStore);
//		} else {
			return false;
//		}
	}

  /**
   * Get general parameter (including from parents)
   * @param name
   * @param pathToVariable
   * @return
   */
  public IRecordVariable getBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable,boolean checkAncestors) {
  	IRecordVariable local = this.parameterVariables_.getBuilderVariableQuiet(name);
  	if(local!=null) {
  		return local;
  	} else if(checkAncestors){
  		return this.type_.getBuilderParameterQuiet(name, pathToVariable);
  	} else {
  		return null;
  	}
  }

  
 
  // ==========================================================================
  // NODE TYPE STUFF (ROOT/INTERNAL)
  public static interface NodeType {
		public InternalTypeBuilder createChildTypeBuilder(String childName, final DesignRecordReference reference,final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent, FoundationNode peerNode);
		
  	public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock getDesignLinkedSkeleton();
//  	public TargetType getFoundationNodeTiedTargetTypeBackwardsQuiet(String name, JavaVariablePath pathToDeclaration);
//  	public TargetMethod getFoundationNodeTiedTargetMethodQuiet( TargetType baseClass, String methodName, JavaVariablePath classPath);
  	
  	public IRecordVariable getBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable);
    public AncestorConnection getAncestorBackQuiet(String name, BaseTrackers.JavaVariablePath path);
		public SLayer getRelatedLayer();
		
    //Append parent names of parent (with '.' if needed)
    public void backAppendName(SimpleStringBuffer buffer);
  	public boolean backBuildPathToType(TargetType type, JavaVariablePath pathStore); 

  	public ImplementationBlockSkeletonManager getSkeletonManager();
  }
  private static interface InteractingBitsBuilder {
  	public NodeType constructType(FoundationNode relatedNode,JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock baseSkeleton, FileEnvironment environment);
  }
  
  // --------------------------------------------------------------------------
  // CLASS ROOT TYPE
  private static final class RootType implements NodeType{
  	private final JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock	designLinkedBlockSkeleton_;
		private final JavaImplementationBlock	parentBlock_;
		private final ImplementationBlockSkeletonManager skeletonManager_;
		private final SLayer	relatedLayer_;
		
		public RootType(DesignRecordReference reference, SLayer relatedLayer, LinkupTracker relatedTracker, JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock	designLinkedBlockSkeleton,  JavaImplementationBlock parentBlock) {
  		this.relatedLayer_ = relatedLayer;
			this.parentBlock_ = parentBlock;
			this.designLinkedBlockSkeleton_ = designLinkedBlockSkeleton;
			this.skeletonManager_ = new ImplementationBlockSkeletonManager(designLinkedBlockSkeleton_.getParent(),relatedTracker, parentBlock.getFileEnvironment());

      //Hack stuff - maybe the linkup tracker bit should do the work?
      // Actually, maybe DesignRecordReference should...
  		this.designLinkedBlockSkeleton_.getParent().addIsRoot();
  	  this.designLinkedBlockSkeleton_.addIsTopLevel();
  	}
		
		public SLayer getRelatedLayer() { 
			return this.relatedLayer_;
		}
		public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock getRawSkeleton() {
			return this.designLinkedBlockSkeleton_;
		}
  	public boolean backBuildPathToType(TargetType type, JavaVariablePath pathStore) { return false; }
  	
		public InternalTypeBuilder createChildTypeBuilder(String childName, final DesignRecordReference reference,final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent, FoundationNode peerNode) {
  		return new InternalTypeBuilder(childName, reference,childRefInParent,this.relatedLayer_,peerNode);
  	}
    public AncestorConnection getAncestorBackQuiet(String name, BaseTrackers.JavaVariablePath path) {
    	return null; //Can't find
    }
    public void backAppendName(SimpleStringBuffer buffer) {
    	//Nothing to do
    }
		
    public IRecordVariable getBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
    	//Future feature
    	// Could have access to variables at the root of the implementation (including perhaps specific implementation variables)
    	// Would just need a reference in the Block to the base Implementation 
    	Debug.futureFeatureMarker("add root implementation variables");
    	return null;
    }

  	public TargetType getFoundationNodeTiedTargetTypeBackwardsQuiet(String name, JavaVariablePath pathToDeclaration) {
  		return parentBlock_.getTargetTypeQuiet(name, pathToDeclaration);
  	}
  	public TargetMethod getFoundationNodeTiedTargetMethodQuiet( TargetType targetType, String methodName, JavaVariablePath pathToDeclaration) {
  		return parentBlock_.getTargetMethodQuiet(targetType, methodName, pathToDeclaration);

  	}
  	public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock getDesignLinkedSkeleton() {
  		return this.designLinkedBlockSkeleton_;
  	}
  	public ImplementationBlockSkeletonManager getSkeletonManager() {
  		return this.skeletonManager_;
  	}
  } //end of class RootType
//  // ==========================================================================
//  private final static class MethodReferenceHolder {
//    private final GlobalMethodReference method_;
//    private MethodReferenceHolder next_;
//    private final GlobalClassReference targetType_;
//
//    public MethodReferenceHolder(GlobalClassReference targetType, GlobalMethodReference method, MethodReferenceHolder next) {
//      this.targetType_ = targetType;
//      this.method_ = method;
//      this.next_ = next;
//    }
//    public boolean isContains(TargetType queryTargetType) {
//      if(this.targetType_ == queryTargetType) {
//        return true;
//      } else if(next_==null) {
//        return false;
//      } else {
//        return next_.isContains(queryTargetType);
//      }
//    }
//    public GlobalMethodReference getMethodQuiet(TargetType queryTargetType) {
//      if(this.targetType_ == queryTargetType) {
//        return this.method_;
//      } else if(next_==null) {
//        return null;
//      } else {
//        return next_.getMethodQuiet(queryTargetType);
//      }
//    }
//  }
  
  /**
   * Get any specific Target Type stuff stored in the FoundationNodes (eg, Globals)
   * @param name name of the type
   * @param pathToDeclaration
   * @param checkAncestors whether to check ancestors, or just this node
   * @return
   */
//	public TargetType getFoundationNodeTiedTargetTypeQuiet(String name, JavaVariablePath pathToDeclaration, boolean checkAncestors) {
//		GlobalClassReference localClass = this.globalClassReferences_.getQuiet(name);
//		if(localClass!=null) {
//			return localClass;
//		} else if(checkAncestors){
//			return this.type_.getFoundationNodeTiedTargetTypeBackwardsQuiet(name,  pathToDeclaration);
//		} else {
//			return null;
//		}
//	}
//	public TargetMethod getFoundationNodeTiedTargetMethodQuiet( TargetType targetType, String methodName, JavaVariablePath pathToDeclaration, boolean checkAncestors) {
//		MethodReferenceHolder base = this.globalMethodReferences_.getQuiet(methodName);
//		if(base!=null) {
//			return base.getMethodQuiet(targetType);
//		} else if(checkAncestors) {
//			//Add in check ancestors stuff her
//			return this.type_.getFoundationNodeTiedTargetMethodQuiet(targetType, methodName,  pathToDeclaration);
//		}else {
//			return null;
//		}
//	}
	
	

 
	// =========================================================================
	// This should be done in the basicBuild() phase, when all Foundation Nodes have been prior created
	

	// ==========================================================================

	

}
