package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.DesignBridgeComponents.DesignRecordReference;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode.InternalTypeBuilder;
import org.ffd2.skeletonx.compile.java.layer.FoundationNode.NodeType;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock.NormalDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TargetBaseFormHolder.TargetBaseFormBlock;

import org.ffd2.solar.general.SimpleStringBuffer;

/**
   * Class functions as a NodeType for the related child
   */
public final class FoundationNodeChildReference implements NodeType {
	private final DesignRecordReference reference_; //Reference of the child
	private final	TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent_;
	private final JavaImplementationFormBlock.BlockDataBlock.ChildRefDataBlock implementationChildRefInParentSkeleton_;
	private final JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock.IsChildDataBlock	designLinkedBlockSkeleton_;
	private final ImplementationBlockSkeletonManager skeletonManager_;
	private final FoundationNode	parentNode_; 
	private final String	nodeName_;
	private final SLayer	relatedLayer_; //The related layer of the child
	private final FoundationNode	childNode_;
	public FoundationNodeChildReference( 
				SLayer parentLayer, 
				String nodeName, DesignRecordReference reference, 
				TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent,
				JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock	myDesignLinkedBlockSkeleton, 
				JavaImplementationFormBlock.BlockDataBlock parentBlockSkeleton, 
				LinkupTracker relatedTracker, FoundationNode parentNode, FoundationNode childNode,
				FileEnvironment environment
		) {
		
		this.nodeName_ = nodeName;
		this.reference_ = reference;
		this.childRefInParent_ = childRefInParent;
		this.parentNode_ = parentNode;
		this.childNode_ = childNode;	  
		this.designLinkedBlockSkeleton_ = myDesignLinkedBlockSkeleton.addIsChild(parentBlockSkeleton);

		this.skeletonManager_ = new ImplementationBlockSkeletonManager(designLinkedBlockSkeleton_.getParent().getParent(),relatedTracker,environment);

		FoundationLayerType layerType = new FoundationLayerType(nodeName, parentLayer,parentNode,childNode, this.skeletonManager_, this,environment);
	  this.implementationChildRefInParentSkeleton_ = parentBlockSkeleton.addChildRef(nodeName, childRefInParent, this.designLinkedBlockSkeleton_);
		this.relatedLayer_ = layerType.getChildLayer();

	}
	/**
	 * Called to set the LocalChild type, when regarding this node as the peer
	 * @param localChildBaseSkeleton
	 * @param targetVariablePathToFoundationChildrenChainTracker
	 * @return
	 */
	public NormalDataBlock setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		return localChildBaseSkeleton.addNormal(this.implementationChildRefInParentSkeleton_, targetVariablePathToFoundationChildrenChainTracker);
	}
	public void doAddAncestorTypeSearch(JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock target, BaseTrackers.JavaVariablePath pathToParentLayer) {
		target.addAncestorTypeFoundation(this.implementationChildRefInParentSkeleton_, pathToParentLayer);
	}
	public SLayer getChildLayer() {
		return this.relatedLayer_;
	}
	public InternalTypeBuilder createChildTypeBuilder(String childName, final DesignRecordReference reference,final TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock childRefInParent, FoundationNode peerNode) {
		return new InternalTypeBuilder(childName, reference,childRefInParent,this.relatedLayer_,peerNode);
	}
	
	public boolean backBuildPathToType(TargetType type, JavaVariablePath pathStore) { 
		addBackLink(pathStore);
		return parentNode_.buildPathToType(type, pathStore,true);
	}

	public void backAppendName(SimpleStringBuffer buffer) {
   	this.parentNode_.backAppendName(buffer);
   	//This goes after the parent call, as we are doing this backwards (so parent appears first)
   	buffer.append('.');
   	buffer.append(this.nodeName_);
   }
	public ImplementationBlockSkeletonManager getNodeSkeleton() {
		return this.skeletonManager_;
	}
	public FoundationNode getChildNode() {
		return this.childNode_;
	}
	public FoundationNode getParentNode() {
		return this.parentNode_;
	}
	public TargetBaseFormBlock.BlockDataBlock.ChildRefDataBlock getChildRefInParentSkeleton() {
		return this.childRefInParent_;
	}
	public JavaImplementationFormBlock.BlockDataBlock.ChildRefDataBlock getImplementationChildRefInParentSkeleton() {
		return this.implementationChildRefInParentSkeleton_;
	}
	public final void addBackLink(BaseTrackers.JavaVariablePath path) {
		path.addBlockParent(designLinkedBlockSkeleton_);
	}
//  	public AncestorConnection getAncestorQuiet(String name, BaseTrackers.JavaVariablePath path) {
//  		addBackLink(path);
//  		return parentNode_.getAncestorQuiet(name, path); 
//    }
	public ImplementationBlockSkeletonManager getSkeletonManager() {
		return this.skeletonManager_;
	}
	//We put true for the "checkAncestors" parameter, as the only reason these methods would get called would be in a "check ancestors" scenario
//	public TargetType getFoundationNodeTiedTargetTypeBackwardsQuiet(String name, JavaVariablePath pathToDeclaration) {
//		pathToDeclaration.addBlockParent(designLinkedBlockSkeleton_);
//    return parentNode_.getFoundationNodeTiedTargetTypeQuiet(name, pathToDeclaration,true);   
//	}
//	public TargetMethod getFoundationNodeTiedTargetMethodQuiet( TargetType baseClass, String methodName, JavaVariablePath pathToDeclaration) {
//		pathToDeclaration.addBlockParent(designLinkedBlockSkeleton_);
//    return parentNode_.getFoundationNodeTiedTargetMethodQuiet(baseClass, methodName, pathToDeclaration,true);   
//	}
	public IRecordVariable getBuilderParameterQuiet(String name, JavaVariablePath pathToDeclaration) {
		pathToDeclaration.addBlockParent(designLinkedBlockSkeleton_);
    return parentNode_.getBuilderParameterQuiet(name,pathToDeclaration,true);   
	}
	public JavaImplementationFormBlock.BlockDataBlock.IsDesignLinkedDataBlock getDesignLinkedSkeleton() {
		return this.designLinkedBlockSkeleton_.getParent();
	}
	//This will change with the new SkeletonX, as the main skeleton can then be built a priori...
  //		baseContainer_.setBlockSkeleton(this.designLinkedBlockSkeleton_.getParent());
	
	/**
	 * 
	 */
  public IRecordVariable getBuilderParameterQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean checkAncestors) {
  	if(checkAncestors) {
  
	  	//Add link to parent, and then check parent. 
			addBackLink(pathToVariable);
	  	return parentNode_.getBuilderParameterQuiet(name, pathToVariable,false);
  	} else {
  		return null;
  	}		 
	
  }
  
  public AncestorConnection getAncestorBackQuiet(String name, JavaVariablePath path) { 
  	//Finished
		return parentNode_.getAncestorConnectionQuiet(name, path,true); //We assume that if this method is being called all ancestors ought to be checked if needed. 
	}
	public SLayer getRelatedLayer() {
		return this.relatedLayer_;
	}
	
}