package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.DesignBridgeComponents.DesignRecordReference;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.ChainImplementationConnection;
import org.ffd2.skeletonx.compile.java.CodeBlockEnvironment;
import org.ffd2.skeletonx.compile.java.SLayerRecordReference;
import org.ffd2.skeletonx.compile.java.IRecordTypeTarget;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.IRecordVariable.NameBlockDetails;
import org.ffd2.skeletonx.compile.java.IVariableType;
import org.ffd2.skeletonx.compile.java.ImplementationParameterTarget;
import org.ffd2.skeletonx.compile.java.KeyMarkerReference;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.MacroCallInstance;
import org.ffd2.skeletonx.compile.java.MacroCallRequirementsAccess;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.MappingReference;
import org.ffd2.skeletonx.compile.java.VariableStore;
import org.ffd2.skeletonx.skeleton.BaseBuilder;

import org.ffd2.skeletonx.skeleton.ArgumentSetStoreFormHolder.ArgumentSetStoreFormBlock;
import org.ffd2.skeletonx.skeleton.BClassAccessFormHolder.BClassAccessFormBlock;
import org.ffd2.skeletonx.skeleton.BasicCallChainFormHolder.BasicCallChainFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.ExpressionDetailsFormHolder.ExpressionDetailsFormBlock.ViaMappingDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.GeneralExpressionDefinitionDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.GeneralTargetVariableDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedArgumentsSetDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedTypeListSetDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MacroCallDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedArgumentsStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedForBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedParametersStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedTypeListDataBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock;
import org.ffd2.skeletonx.skeleton.JavaParameterSettingsFormHolder.JavaParameterSettingsFormBlock.ParameterDataBlock.GeneralRecordDataBlock;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock;
import org.ffd2.skeletonx.skeleton.NameDetailsFormHolder.NameDetailsFormBlock.ComplexDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.TypeListAccessFormHolder.TypeListAccessFormBlock;

import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

/**
 * This is just a link to a base skeleton, with a single set of counters, so
 * multiple things can access the skeleton without worrying about name conflicts
 * - in the next version of Skeleton this will be redundant (as it automatically
 * creates unique names)
 * 
 * @author Matthew Goode
 * 
 */
public class ImplementationBlockSkeletonManager {

	private final static PeerConnection NULL_PEER_CONNECTION = new PeerConnection() {

		public void addPathToPeer(JavaVariablePath path) {
			//Nothing do.
			Debug.println("Should this happen?");
		}

	};
	private final JavaImplementationFormBlock.BlockDataBlock skeletonJavaBlockBase_;
	// private final AccessStoreX<BuilderVariable> builderVariables_;
	private final FileEnvironment environment_;
	private final LinkupTracker linkupTracker_;

	private int macroCallCount_ = 0;
	private int generalIDIndex_ = 0;
	private int numberOfSearches_ = 0;
	private int numberOfLinkedCodeBlocks_ = 0;
	private int numberOfGeneralExpressionBlocks_ = 0;

	private int numberOfLinkedForBlocks_ = 0;

	private int linkedArgumentsCount_ = 0;
	private int variableIndexCount_ = 0;
	private PeerConnection peerConnection_ = NULL_PEER_CONNECTION;

	public ImplementationBlockSkeletonManager(JavaImplementationFormBlock.BlockDataBlock skeletonBase, LinkupTracker linkupTracker, FileEnvironment environment) {
		this.skeletonJavaBlockBase_ = skeletonBase;
		this.linkupTracker_ = linkupTracker;
		this.environment_ = environment;

		// this.builderVariables_ =
		// this.environment_.createEntityStore("builder variable");
	}

	public void setPeerConnection(PeerConnection connection) {
		this.peerConnection_ = connection;

	}
	public PeerConnection getPeerConnection() {
		return this.peerConnection_;
	}
	private final BaseBuilder getRootBuilder() {
		return this.skeletonJavaBlockBase_.getRootBuilder();
	}

	public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock addSearchBlock(JavaImplementationFormBlock.BlockDataBlock searchType, int searchLineNumber) {
		return this.skeletonJavaBlockBase_.addSearchRecord(this.numberOfSearches_++, searchLineNumber, searchType);
	}

	public JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock addSearchBlock(ImplementationBlockSkeletonManager searchType, int searchLineNumber) {
		return this.skeletonJavaBlockBase_.addSearchRecord(this.numberOfSearches_++, searchLineNumber, searchType.skeletonJavaBlockBase_);
	}

	public LinkedCodeBlockStoreDataBlock allocateLinkedCode() {
		return this.skeletonJavaBlockBase_.addLinkedCodeBlockStore(this.numberOfLinkedCodeBlocks_++, this.skeletonJavaBlockBase_.getRootBuilder().createBCodeBlockAccessChild());
	}

	public GeneralExpressionDefinitionDataBlock allocateExpressionDefinition() {
		return this.skeletonJavaBlockBase_.addGeneralExpressionDefinition(
				this.numberOfGeneralExpressionBlocks_++, this.skeletonJavaBlockBase_.getRootBuilder().createExpressionDetailsChild());

	}

	// Use wisely
	public JavaImplementationFormBlock.BlockDataBlock getSkeletonBase() {
		return this.skeletonJavaBlockBase_;
	}

	public LinkedForBlockStoreDataBlock allocateForBlock() {
		return this.skeletonJavaBlockBase_.addLinkedForBlockStore(this.numberOfLinkedForBlocks_++, this.skeletonJavaBlockBase_.getRootBuilder().createBForBlockAccessChild());

	}

	public LinkedArgumentsStoreDataBlock allocateLinkedArgumentSetter() {
		LinkedArgumentsStoreDataBlock block = this.skeletonJavaBlockBase_.addLinkedArgumentsStore(
				this.linkedArgumentsCount_++, this.skeletonJavaBlockBase_.getRootBuilder().createArgumentSetStoreChild());
		return block;
	}

	public LinkedTypeListDataBlock allocateLinkedTypeList() {
		LinkedTypeListDataBlock block = this.skeletonJavaBlockBase_.addLinkedTypeList(
				this.linkedArgumentsCount_++, this.skeletonJavaBlockBase_.getRootBuilder().createTypeListAccessChild());
		return block;
	}

	public LinkedParametersStoreDataBlock allocateLinkedParameters() {
		return this.skeletonJavaBlockBase_.addLinkedParametersStore(this.linkedArgumentsCount_++, this.skeletonJavaBlockBase_.getRootBuilder().createBParametersAccessChild());
	}

	public LinkedArgumentsSetDataBlock createArgumentLinkage(ArgumentSetStoreFormBlock targetArgumentSet, BaseTrackers.JavaVariablePath pathToTargetRecord) {

		LinkedArgumentsSetDataBlock block = this.skeletonJavaBlockBase_.addItem().addLinkedArgumentsSet(
				targetArgumentSet, pathToTargetRecord, this.skeletonJavaBlockBase_.getRootBuilder().createArgumentSetDetailsChild());
		return block;
	}

//	public LinkedTypeListSetDataBlock createTypeListLinkage(TypeListAccessFormBlock targetArgumentStore, BaseTrackers.JavaVariablePath pathToTargetRecord) {
//		// TODO: Probably should make the type list a parameter to this method...
//		LinkedTypeListSetDataBlock block = this.skeletonJavaBlockBase_.addItem().addLinkedTypeListSet(
//				targetArgumentStore, pathToTargetRecord, targetArgumentStore.getRootBuilder().createTypeListDetailsChild());
//		return block;
//	}

	public GeneralTargetVariableDataBlock addGeneralTargetVariableCodeVariable(String variableName) {
		return this.addGeneralTargetVariableCodeVariableBase(variableName, this.skeletonJavaBlockBase_);
	}

	private GeneralTargetVariableDataBlock addGeneralTargetVariableCodeVariableBase(String variableName, JavaImplementationFormBlock.BlockDataBlock baseBlock) {
		GeneralTargetVariableDataBlock result = baseBlock.addItem().addGeneralTargetVariable(
				variableName, this.variableIndexCount_++, baseBlock.getRootBuilder().createCodeVariableChild());
		return result;
	}

	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ParameterVariableStoreDataBlock addParameterTargetVariableStore(String variableName) {
		return addParameterTargetVariableStoreBase(variableName, this.skeletonJavaBlockBase_);
	}

	private JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ParameterVariableStoreDataBlock
			addParameterTargetVariableStoreBase(String variableName, JavaImplementationFormBlock.BlockDataBlock baseBlock) {
		return baseBlock.addItem().addParameterVariableStore(
				variableName, this.variableIndexCount_++, baseBlock.getRootBuilder().createCodeVariableChild(), baseBlock.getRootBuilder().createNameDetailsChild());
	}

	// ==========================================================================
	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock
						createInstanceVariable(String name, final BClassAccessFormBlock classAccessParameter, JavaVariablePath definedPathToClassDefiningLayer
						) {
		return addItemSkeleton().addInstanceVariable(
				name, classAccessParameter, nextGeneralIDIndex(), definedPathToClassDefiningLayer, classAccessParameter.getParent().createCodeVariableChild(),
				classAccessParameter.getParent().createNameDetailsChild(), classAccessParameter.getParent().createTypeSetterDetailsChild());
	}

	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock
						createInstanceMethodFinal(
								String name, final BClassAccessFormBlock classAccessParameter, JavaVariablePath pathToClassDefiningLayer
						) {
		final BaseBuilder rootBuilder = this.skeletonJavaBlockBase_.getRootBuilder();
		return addItemSkeleton().addMethod(
				name, classAccessParameter, nextGeneralIDIndex(), pathToClassDefiningLayer, rootBuilder.createCodeMethodChild(), rootBuilder.createTypeSetterDetailsChild());
	}

	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.ConstructorDataBlock
						createConstructorFinal(final BClassAccessFormBlock classAccessParameter, JavaVariablePath pathToClassDefiningLayer) {
		final BaseBuilder rootBuilder = this.skeletonJavaBlockBase_.getRootBuilder();
		return addItemSkeleton().addConstructor(
				classAccessParameter, nextGeneralIDIndex(), pathToClassDefiningLayer, rootBuilder.createBCodeBlockAccessChild(), rootBuilder.createJavaParameterSettingsChild());
	}

	// ==== Declaring instance variable stuff ====
	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock
					doDeclareInstanceVariable(String variableName, final BClassAccessFormBlock classAccessParameter, BaseTrackers.JavaVariablePath pathToThisClass) {
		final BaseBuilder rootBuilder = this.skeletonJavaBlockBase_.getRootBuilder();
		return addItemSkeleton().addInstanceVariable(
				variableName, classAccessParameter, nextGeneralIDIndex(), pathToThisClass, rootBuilder.createCodeVariableChild(), rootBuilder.createNameDetailsChild(),
				rootBuilder.createTypeSetterDetailsChild());
	}

	// public
	// JavaImplementationForm.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock
	// doDeclareInstanceVariable(SLayer recordAccess, String variableName) {
	// return doDeclareInstanceVariable(recordAccess,
	// variableName,getPathToClass(recordAccess));
	// }
	// public
	// JavaImplementationForm.BlockDataBlock.ItemDataBlock.InstanceVariableDataBlock
	// doDeclareInstanceVariable(JavaImplementationForm.BlockDataBlock
	// targetSkeleton, String variableName,BaseTrackers.JavaVariablePath
	// pathToThisClass) {
	// return targetSkeleton.addItemBlock().addInstanceVariableBlock(variableName,
	// this.getClassAccess(),this.definitionLayerAccess_.nextIDIndex(),
	// pathToThisClass);
	// }

	// ==== Declaring instance method stuff ====
	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.MethodDataBlock
					doDeclareInstanceMethod(String methodName, final BClassAccessFormBlock classAccessParameter, BaseTrackers.JavaVariablePath pathToThisClass) {
		final BaseBuilder rootBuilder = this.skeletonJavaBlockBase_.getRootBuilder();
		return addItemSkeleton().addMethod(
				methodName, classAccessParameter, nextGeneralIDIndex(), pathToThisClass, rootBuilder.createCodeMethodChild(), rootBuilder.createTypeSetterDetailsChild());
	}

	// ==========================================================================

	/**
	 * Create a link to a child layer, include creating a new Skeleton block, and
	 * setting everything up in the parent block.
	 * 
	 * @param childName
	 * @param childLayer
	 * @param baseBuilder
	 * @param ancestor
	 * @param pathToAncestorParent
	 * @return
	 */
	public LocalChild createLocalChild(String childName, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, LinkupTracker baseBuilder, AncestorConnection ancestor) {
		// Create new child layer block
		JavaImplementationFormBlock.BlockDataBlock newLayerSkeletonBase = baseBuilder.allocateSpareNode(childName, parentLayer.getFullName() + ":" + childName);
		BaseTrackers.JavaVariablePath pathToAncestorParent = parentLayer.buildPathToFoundationNode(ancestor.getParentNode());
		// Set as local - this means setting the foundation node linked to the
		// layer, and the actual layer that is the parent of this child
		JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock localChildSkeleton = newLayerSkeletonBase.addIsLocal(
				this.skeletonJavaBlockBase_, ancestor.getTargetNodeSkeleton().skeletonJavaBlockBase_);

		JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock.NormalDataBlock normalBlock = this.skeletonJavaBlockBase_.addLocalChildRef(childName, localChildSkeleton).addNormal(
				ancestor.getImplementationChildRefInParentSkeleton(), pathToAncestorParent);

		// We still need to have the parent utilise this child

		return new LocalChild(childName, ancestor.getTargetNode(), parentLayer, childMaker, localChildSkeleton, normalBlock.getParent(), this.linkupTracker_, this.environment_);
	}

	/**
	 * Create a link to a child layer, include creating a new Skeleton block, and
	 * setting everything up in the parent block.
	 * 
	 * @param childName
	 * @param childLayer
	 * @param baseBuilder
	 * @param ancestor
	 * @param pathToAncestorParent
	 * @return
	 */
	// public LocalChild createLocalChild(String childName,SLayer parentLayer,
	// SLayer nextSideLayer, IRecordVariable
	// referencingVariableMayBeNull,LinkupTracker baseBuilder, AncestorConnection
	// ancestor, BaseTrackers.JavaVariablePath pathToAncestorParent) {
	// //Create new child layer block
	// JavaImplementationForm.BlockDataBlock newLayerSkeletonBase =
	// baseBuilder.allocateSpareNode(childName);
	//
	// //Set as local - this means setting the foundation node linked to the
	// layer, and the actual layer that is the parent of this child
	// JavaImplementationForm.BlockDataBlock.IsLocalDataBlock localChildSkeleton =
	// newLayerSkeletonBase.addIsLocalBlock(this.skeletonBase_,
	// ancestor.getTargetNodeSkeleton().skeletonBase_);
	// RedirectedChildSLayerMaker childMaker = new
	// RedirectedChildSLayerMaker(nextSideLayer,referencingVariableMayBeNull);
	//
	// JavaImplementationForm.BlockDataBlock.LocalChildRefDataBlock.NormalDataBlock
	// normalBlock = this.skeletonBase_.addLocalChildRefBlock(childName,
	// localChildSkeleton).addNormalBlock(ancestor.getImplementationChildRefInParentSkeleton(),
	// pathToAncestorParent);
	//
	// //We still need to have the parent utilise this child
	//
	// return new LocalChild(childName, ancestor.getTargetNode(),parentLayer,
	// childMaker, localChildSkeleton,normalBlock.getParent(),
	// this.linkupTracker_,this.environment_);
	//
	// }
	//

	public LocalChild createLocalChild(String childName, SLayer parentLayer, NonFoundationChildSLayerMaker childMaker, LinkupTracker baseBuilder, SLayer peerLayer, BaseTrackers.JavaVariablePath pathToAncestorParent) {
		// Create new child layer block
		// Debug.println("Create local 2:",childName);
		// Thread.dumpStack();
		// THIS NEEDS

		// BaseTrackers.JavaVariablePath pathToAncestorParent =
		// parentLayer.buildPathToPeerLayer(peerLayer.get);
		JavaImplementationFormBlock.BlockDataBlock newLayerSkeletonBase = baseBuilder.allocateSpareNode(childName, parentLayer.getFullName() + ":" + childName);

		// Set as local - this means setting the foundation node linked to the
		// layer, and the actual layer that is the parent of this child
		JavaImplementationFormBlock.BlockDataBlock.IsLocalDataBlock localChildSkeleton = newLayerSkeletonBase.addIsLocal(
				this.skeletonJavaBlockBase_, peerLayer.getBlockSkeleton().skeletonJavaBlockBase_);

		// This bit needs to be handled by asking the SLayer to do it (so it can
		// check if it is a foundation or local child.
		// rus
		JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton = this.skeletonJavaBlockBase_.addLocalChildRef(childName, localChildSkeleton);// .,
																																																																																						// peerLayer.getImplementationChildRefInParentSkeleton(),
																																																																																						// pathToAncestorParent);
		// BaseTrackers.JavaVariablePath pathToAncestorParent = new
		// BaseTrackers.JavaVariablePath();
		peerLayer.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, pathToAncestorParent);

		// TOD O :FINISH THIS - note for removal
		// We still need to have the parent utilise this child

		// throw Debug.finishMeMarker();
		LocalChild result = new LocalChild(childName, peerLayer, parentLayer, childMaker, localChildSkeleton, localChildBaseSkeleton, this.linkupTracker_, this.environment_);
		return result;
	}

	// public LocalChild createLocalChild(String childName,SLayer parentLayer,
	// ChildSLayerMaker childMaker,LinkupTracker baseBuilder, AncestorConnection
	// ancestor, BaseTrackers.JavaVariablePath pathToAncestorParent) {
	// //Create new child layer block
	// JavaImplementationForm.BlockDataBlock newLayerSkeletonBase =
	// baseBuilder.allocateSpareNode(childName);
	//
	// //Set as local - this means setting the foundation node linked to the
	// layer, and the actual layer that is the parent of this child
	// JavaImplementationForm.BlockDataBlock.IsLocalDataBlock localChildSkeleton =
	// newLayerSkeletonBase.addIsLocalBlock(this.skeletonBase_,
	// ancestor.getTargetNodeSkeleton().skeletonBase_);
	//
	// this.skeletonBase_.addLocalChildRefBlock(childName, localChildSkeleton,
	// ancestor.getImplementationChildRefInParentSkeleton(),
	// pathToAncestorParent);
	//
	// //We still need to have the parent utilise this child
	//
	// return new LocalChild(childName, ancestor.getTargetNode(),parentLayer,
	// childMaker, localChildSkeleton,this.linkupTracker_,this.environment_);
	// }
	public int nextGeneralIDIndex() {
		return this.generalIDIndex_++;
	}

	public JavaImplementationFormBlock.BlockDataBlock.GlobalMethodVariableDataBlock addGlobalMethodVariableBlock(String methodName) {
		return this.skeletonJavaBlockBase_.addGlobalMethodVariable(methodName, getRootBuilder().createCodeMethodChild());
	}

	public JavaImplementationFormBlock.BlockDataBlock.GlobalClassVariableDataBlock addGlobalClassVariableBlock(String className) {
		return this.skeletonJavaBlockBase_.addGlobalClassVariable(className, getRootBuilder().createBClassAccessChild());
	}

	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock addItemSkeleton() {
		return this.skeletonJavaBlockBase_.addItem();
	}

	public MacroCallInstance createMacroCallFinalBase(
			String variableName,
			MacroConstruction macro,
			MacroCallRequirementsAccess requirementsAccess,
			SLayer parentRecord
			) {
		MacroCallDataBlock macroCall = macro.createCallFinal(variableName + (macroCallCount_++), skeletonJavaBlockBase_.addItem(), requirementsAccess);
		return new MacroCallInstance(macro, macroCall, parentRecord);
	}

	// ==========================================================================

	private JavaParameterSettingsFormBlock.ParameterDataBlock newParameter(String parameterName) {
		return this.skeletonJavaBlockBase_.getParametersParameter().addParameter(parameterName);
	}

	// private final void setParameter(String name, VariableType parameterType) {
	// LocallyAccessRecordVariable variable = new
	// LocallyAccessRecordVariable(name,this.parent_);
	// variable.setUsageType(parameterType);
	//
	// if(!builderVariables_.containsKey(name)) {
	// //We force because we assume that at this point any repeated things are
	// sorted out at the design stage
	// parent_.forceIRecordVariable(RecordVariableTrace.getEmptyTrace(),
	// name,variable);
	// }
	// parent_.orderedImplementationParameters_.addElement(variable);
	// }
	private final TypicalBuilderIVariable addParameter(String name, BuilderVariableType type) {

		TypicalBuilderIVariable parameterVariable = new TypicalBuilderIVariable(name, type);
		// this.builderVariables_.force(name, parameterVariable);
		return parameterVariable;
	}

	public final ImplementationParameterTarget getAsParameterTarget(final VariableStore store) {
		return new ImplementationParameterTarget() {
			public void newStringParameter(String name) {
				store.forceBuilderVariable(name, addStringParameter(name));
			}

			public void newRecordParameter(String name, DesignRecordReference reference) {
				store.forceBuilderVariable(name, addRecordParameter(name, reference));
			}

			public void newIntParameter(String name) {
				store.forceBuilderVariable(name, addIntParameter(name));
			}

			public void newIntArrayParameter(String name) {
				store.forceBuilderVariable(name, addIntArrayParameter(name));
			}

			public void newDoubleParameter(String name) {
				store.forceBuilderVariable(name, addDoubleParameter(name));
			}

			public void newChainParameter(String name, ChainDefinitionBlock definition) {
				store.forceBuilderVariable(name, addChainParameter(name, definition));
			}

			public void newBooleanParameter(String name) {
				store.forceBuilderVariable(name, addBooleanParameter(name));
			}

			public void allParametersDeclared() {
			}

			public void newMappingParameter(String name,
					MappingDefinitionBlock definition) {
				store.forceBuilderVariable(name, addMappingParameter(name, definition));
			}

			public void newKeyParameter(String name, KeyDefinitionBlock definition) {
				store.forceBuilderVariable(name, addKeyParameter(name, definition));
			}
		};
	}

	// =======================================================================
	// ImplementationParameterTarget stuff

	public TypicalBuilderIVariable addIntParameter(String name) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.IntDataBlock parameterSkeleton = this.newParameter(name).addInt(getRootBuilder().createIntBuilderVariableAccessChild());
		return addParameter(name, PrimitiveVariableTypes.createIntVariableType(parameterSkeleton.getVariableStoreParameter()));

	}

	public TypicalBuilderIVariable addIntArrayParameter(String name) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.IntArrayDataBlock parameterSkeleton = this.newParameter(name).addIntArray(
				getRootBuilder().createIntArrayBuilderVariableAccessChild());
		return addParameter(name, PrimitiveVariableTypes.createIntArrayVariableType(parameterSkeleton.getVariableStoreParameter()));
	}

	public TypicalBuilderIVariable addStringParameter(String name) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.StringDataBlock parameterSkeleton = this.newParameter(name).addString(
				getRootBuilder().createStringBuilderVariableAccessChild());
		return addParameter(name, PrimitiveVariableTypes.createStringVariableType(parameterSkeleton.getVariableStoreParameter(), parameterSkeleton));

	}

	public TypicalBuilderIVariable addBooleanParameter(String name) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.BooleanDataBlock parameterSkeleton = this.newParameter(name).addBoolean(
				getRootBuilder().createBooleanBuilderVariableAccessChild());
		return addParameter(name, PrimitiveVariableTypes.createBooleanVariableType(parameterSkeleton.getVariableStoreParameter()));

	}

	public TypicalBuilderIVariable addDoubleParameter(String name) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.DoubleDataBlock parameterSkeleton = this.newParameter(name).addDouble(
				getRootBuilder().createDoubleBuilderVariableAccessChild());
		return addParameter(name, PrimitiveVariableTypes.createDoubleVariableType(parameterSkeleton.getVariableStoreParameter()));
	}

	public TypicalBuilderIVariable addRecordParameter(String name, DesignRecordReference reference) {

		final JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton = this.newParameter(name);
		FoundationNode relatedScaffoldNode = linkupTracker_.getImplementationContainer(reference.getRecordIndex());
		final JavaParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock baseRecordParameterSkeleton = relatedScaffoldNode.createRecordParameter(baseParameterSkeleton);

		return addParameter(name, new RecordType(name, relatedScaffoldNode, baseRecordParameterSkeleton));
	}

	public TypicalBuilderIVariable addGeneralRecordParameter(String name, FoundationNode record) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock.GeneralRecordDataBlock parameterBaseSkeleton = record.buildGeneralRecordRefParameter(this.newParameter(name));
		GeneralRecordVariableType variableType = new GeneralRecordVariableType(parameterBaseSkeleton, record, this.environment_);
		return addParameter(name, variableType);
	}

	public TypicalBuilderIVariable addChainParameter(String name, ChainDefinitionBlock definition) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton = this.newParameter(name);
		ChainImplementationConnection relatedChain = this.linkupTracker_.getChainLinkup(definition.getChainIndex());

		return addParameter(name, new ChainType(name, definition, relatedChain, baseParameterSkeleton));

	}

	public TypicalBuilderIVariable addMappingParameter(String name, MappingDefinitionBlock definition) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton = this.newParameter(name);

		return addParameter(name, new MappingType(name, definition, baseParameterSkeleton));

	}

	public TypicalBuilderIVariable addKeyParameter(String name, KeyDefinitionBlock definition) {
		final JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton = this.newParameter(name);

		return addParameter(name, new KeyType(name, definition, baseParameterSkeleton));

	}

	//
	// public IRecordVariable getBuilderVariable(String name) throws
	// ItemNotFoundException {
	// return this.builderVariables_.get(name);
	// }
	// public IRecordVariable getBuilderVariableQuiet(String name) {
	// return this.builderVariables_.getQuiet(name);
	// }

	// --------------------------------------------------------------------------
	private final static class MappingType extends AbstractBuilderVariableType implements BuilderVariableType, MappingReference {
		private final MappingDefinitionBlock definition_;
		private final String name_;
		private final JavaParameterSettingsFormBlock.ParameterDataBlock.MappingDataBlock mappingParameterSkeleton_;

		public MappingType(String name, MappingDefinitionBlock definition, JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton) {
			this.name_ = name;
			this.definition_ = definition;
			this.mappingParameterSkeleton_ = definition.addMappingParameter(baseParameterSkeleton);
		}

		public JavaParameterSettingsFormBlock.ParameterDataBlock.MappingDataBlock getMappingDefinitionSkeleton() {
			return this.mappingParameterSkeleton_;
		}

		public ExpressionDetailsFormBlock.ViaMappingDataBlock doAddViaMapping(ExpressionDetailsFormBlock expressionSkeleton, BaseTrackers.JavaVariablePath pathToMappingVariableDeclartion, KeyMarkerReference keyReference, BaseTrackers.JavaVariablePath pathToKeyVariableDeclartion) {
			ViaMappingDataBlock viaSkeleton = expressionSkeleton.addViaMapping(
					mappingParameterSkeleton_, pathToMappingVariableDeclartion,
					keyReference.getKeyDefinitionSkeleton(), pathToKeyVariableDeclartion
					);
			this.definition_.doSetType(viaSkeleton);
			return viaSkeleton;
		}

		public SLayer getAsFoundataionNodeReferenceVariableForType(JavaVariablePath pathToExtend) throws ParsingException {
			throw ParsingException.createInvalidType("mapping", "scaffold reference");
		}

		@Override
		public void getAsReference(IRecordTypeTarget target) {
			target.setAsMappingReference(this);
		}

		@Override
		public void addAsMacroRecordMappingParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			if (this.definition_ == type) {
				macroCallBlock.addCallParameter().addBlockMappingVariable(this.mappingParameterSkeleton_, variablePath);
			} else {
				throw ParsingException.createInvalidType(type.getName(), this.definition_.getName());
			}
		}

		public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Mapping variables cannot supply name block elements");
		}

		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("parameter variables cannot directly supply expression values");

		}

		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, IVariableType type)
				throws ParsingException {
			if (type == IVariableType.MAPPING_REFERENCE) {
				macroCallBlock.addCallParameter().addBlockMappingVariable(this.mappingParameterSkeleton_, variablePath);
			} else {
				throw ParsingException.createInvalidType(type.getName(), "mapping");
			}
		}

		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("parameter variables cannot directly supply expression values");
		}
	}

	// --------------------------------------------------------------------------
	private final static class KeyType extends AbstractBuilderVariableType implements BuilderVariableType, KeyMarkerReference {
		private final KeyDefinitionBlock definition_;
		private final String name_;
		private final JavaParameterSettingsFormBlock.ParameterDataBlock.KeyDataBlock keyParameterSkeleton_;

		public KeyType(String name, KeyDefinitionBlock definition, JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton) {
			this.name_ = name;
			this.definition_ = definition;
			this.keyParameterSkeleton_ = definition.addKeyParameter(baseParameterSkeleton);
		}

		@Override
		public void getAsReference(IRecordTypeTarget target) {
			target.setAsKeyReference(this);
		}

		@Override
		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, KeyDefinitionBlock type) throws ParsingException {
			if (this.definition_ == type) {
				macroCallBlock.addCallParameter().addBlockKeyVariable(this.keyParameterSkeleton_, variablePath);
			} else {
				throw ParsingException.createInvalidType(type.getName(), this.definition_.getName());
			}
		}

		public JavaParameterSettingsFormBlock.ParameterDataBlock.KeyDataBlock getKeyDefinitionSkeleton() {
			return keyParameterSkeleton_;
		}

		public SLayer getAsFoundataionNodeReferenceVariableForType(JavaVariablePath pathToExtend) throws ParsingException {
			throw ParsingException.createInvalidType("key", "scaffold reference");

		}

		public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("key variables cannot supply name block elements");
		}

		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("parameter variables cannot directly supply expression values");

		}

		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock,
				JavaVariablePath variablePath, IVariableType type)
				throws ParsingException {
			if (type == IVariableType.MAPPING_REFERENCE) {
				// macroCallBlock.addVariableBlock(this.keyParameterSkeleton_,
				// variablePath);
				Debug.finishMeMarker();
				throw ParsingException.createGeneralSyntax("Not finish");
			} else {
				throw ParsingException.createInvalidType(type.getName(), "mapping");
			}
		}

		public void addAsMacroRecordMappingParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createInvalidType("not mapping", "mapping");
		}

		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("parameter variables cannot directly supply expression values");
		}
	}

	// ---------------------------------------------------------------------------
	private static abstract class AbstractBuilderVariableType implements BuilderVariableType {
		public AbstractBuilderVariableType() {
		}

		public void addAsMacroRecordMappingParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, MappingDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable cannot provide mapping information");

		}

		public void addAsMacroRecordKeyParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, KeyDefinitionBlock type) throws ParsingException {
			throw ParsingException.createGeneralSyntax("variable cannot provide mapping information");

		}
	}

	// --------------------------------------------------------------------------
	private final static class ChainType extends AbstractBuilderVariableType implements BuilderVariableType {
		private final JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton_;
		private final ChainDefinitionBlock definition_;
		private final String name_;
		private final ChainImplementationConnection relatedChainImplementation_;
		private JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock parameterSkeletonNonDirect_;

		public ChainType(
				String name, ChainDefinitionBlock definition, ChainImplementationConnection relatedChain, JavaParameterSettingsFormBlock.ParameterDataBlock baseParameterSkeleton) {
			this.name_ = name;
			this.definition_ = definition;
			this.relatedChainImplementation_ = relatedChain;
			this.baseParameterSkeleton_ = baseParameterSkeleton;
			this.parameterSkeletonNonDirect_ =
					this.relatedChainImplementation_.setupParameter(baseParameterSkeleton_);
		}

		@Override
		public void getAsReference(IRecordTypeTarget target) {
			target.setNotReference();
		}

		@Override
		public SLayer getAsFoundataionNodeReferenceVariableForType(BaseTrackers.JavaVariablePath pathToExtend) throws ParsingException {
			throw ParsingException.createInvalidType("chain", "scaffold reference");
		}

		public JavaParameterSettingsFormBlock.ParameterDataBlock.ChainRecordDataBlock getParameterSkeleton() {
			// We could probably do this in the constructor...
			if (this.parameterSkeletonNonDirect_ == null) {
				this.parameterSkeletonNonDirect_ =
							this.relatedChainImplementation_.setupParameter(baseParameterSkeleton_);
			}
			return parameterSkeletonNonDirect_;
		}

		@Override
		public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Chain variables cannot supply name block elements");
		}

		@Override
		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			target.addElement().addChainParameter(getParameterSkeleton(), variablePathToThisVariable);
		}

		@Override
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			// TODO Auto-generated method stub
			Debug.finishMeMarker();
			// macroCallBlock.addGeneralChainVariableBlock(getParameterSkeleton(),
			// variablePath);
		}

		@Override
		public void updateExpressionFinal(CodeBlockEnvironment environment,
				ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			expressionSkeleton.addChainParameter(getParameterSkeleton(), variablePath);
		}

	} // End of class ChainType

	// --------------------------------------------------------------------------

	private final static class RecordType extends AbstractBuilderVariableType implements BuilderVariableType, SLayerRecordReference {
		private final JavaParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock baseParameterSkeleton_;
		private final String name_;
		private final FoundationNode relatedScaffoldNode_;

		public RecordType(String name, FoundationNode relatedScaffoldNode, JavaParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock baseParameterSkeleton) {
			this.name_ = name;
			this.relatedScaffoldNode_ = relatedScaffoldNode;
			this.baseParameterSkeleton_ = baseParameterSkeleton;
		}

		public void addAsPathElement(BaseTrackers.JavaVariablePath path) throws ParsingException {
			path.addBlockVariable(this.baseParameterSkeleton_);
		}

		public void doRecordThisMatchAsQueryParameter(
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton,
				BaseTrackers.JavaVariablePath variablePathToParameterFromSearchLayer,
				FoundationNode recordType,
				BaseTrackers.JavaVariablePath variablePathToRecordQueryValueFromSearchLayer
				) throws ParsingException {
			if (recordType != this.relatedScaffoldNode_) {
				throw ParsingException.createInvalidType(recordType.getFullName(), this.relatedScaffoldNode_.getFullName());
			} else {
				targetSkeleton.addThisRecordMatch(
						this.baseParameterSkeleton_, variablePathToParameterFromSearchLayer, recordType.getBaseBlockSkeleton(), variablePathToRecordQueryValueFromSearchLayer);
			}
		}

		@Override
		public void getAsReference(IRecordTypeTarget target) {
			target.setAsFoundationRecordReference(this);
		}

		@Override
		public void doVariableQueryMatchAsQueryValue(
				BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer,
				JavaParameterSettingsFormBlock.ParameterDataBlock.RecordDataBlock testParameter, JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton, BaseTrackers.JavaVariablePath variablePathToQueryMatch) {
			targetSkeleton.addRecordMatch(testParameter, variablePathToTestValueFromSearchLayer, this.baseParameterSkeleton_, variablePathToQueryMatch);
		}

		@Override
		public void doVariableQueryMatchAsQueryParameter(
				JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock targetSkeleton,
				BaseTrackers.JavaVariablePath variablePathToTestValueFromSearchLayer,
				IRecordVariable queryMatchValue, BaseTrackers.JavaVariablePath variablePathToQueryMatch) throws ParsingException {
			IRecordTypeTarget.generateTarget(queryMatchValue).getAsFoundationRecordReference().
					doVariableQueryMatchAsQueryValue(
							variablePathToTestValueFromSearchLayer, this.baseParameterSkeleton_, targetSkeleton, variablePathToQueryMatch
					);
		}

		@Override
		public SLayer getAsFoundataionNodeReferenceVariableForType(BaseTrackers.JavaVariablePath pathToExtend) throws ParsingException {
			if (pathToExtend != null) {
				pathToExtend.addBlockVariable(this.baseParameterSkeleton_);
			}
			return this.relatedScaffoldNode_.getRelatedLayerForLayerComponents();
		}

		@Override
		public void updateNameBlock(ComplexDataBlock target, JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createInvalidType("node", "String or int");
		}

		@Override
		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Non chain-element type");
		}

		@Override
		public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			if (type == IVariableType.RecordReference) {
				macroCallBlock.addCallParameter().addNormalParameterBlockVariable(baseParameterSkeleton_, variablePath);
			} else {
				throw IVariableType.RecordReference.createMismatchException(type);
			}
		}

		@Override
		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Record variables can't provide for target code");
		}

	} // End of class RecordType

	// --------------------------------------------------------------------------
	private static final class GeneralRecordVariableType extends AbstractBuilderVariableType implements BuilderVariableType {
		private final FoundationNode recordType_;
		private final FileEnvironment environment_;
		private final GeneralRecordDataBlock builtParameterSkeleton_;

		public GeneralRecordVariableType(
															final JavaParameterSettingsFormBlock.ParameterDataBlock.GeneralRecordDataBlock
																buildParameterSkeleton,
																FoundationNode recordType,
															FileEnvironment environment) {
			this.builtParameterSkeleton_ = buildParameterSkeleton;
			this.recordType_ = recordType;
			this.environment_ = environment;

		}

		public final void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock, JavaVariablePath variablePath, IVariableType type) throws ParsingException {
			if (type == IVariableType.RecordReference) {
				macroCallBlock.addCallParameter().addGeneralBlockVariable(builtParameterSkeleton_, variablePath);
			} else {
				throw IVariableType.RecordReference.createMismatchException(type);
			}
		}

		@Override
		public void getAsReference(IRecordTypeTarget target) {
			target.setNotReference();
		}

		public void updateCallChainFinal(BasicCallChainFormBlock target, JavaVariablePath variablePathToThisVariable) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Non chain-element type");
		}

		public void updateNameBlock(NameDetailsFormBlock.ComplexDataBlock target, BaseTrackers.JavaVariablePath variablePath, NameBlockDetails elementDetails) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Record variable cannot supply name element");
		}

		public void updateExpressionFinal(CodeBlockEnvironment environment, ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath variablePath) throws ParsingException {
			throw ParsingException.createGeneralSyntax("Record variables can't provide for target code");
		}

		public SLayer getAsFoundataionNodeReferenceVariableForType(JavaVariablePath pathToExtend) throws ParsingException {
			if(pathToExtend!=null) {
				pathToExtend.addGeneralBlockVariable(this.builtParameterSkeleton_);
			}
			return this.recordType_.getRelatedLayerForLayerComponents();
		}

	}

	// ==========================================================================
	// private static final class LocallyAccessRecordVariable implements
	// IRecordVariable {
	// private final String name_;
	// private VariableType usageType_;
	// private final BasicRecordContainer host_;
	// public LocallyAccessRecordVariable(String name, BasicRecordContainer host)
	// {
	// this.name_ = name;
	// this.host_ = host;
	// }
	// public String toString() {
	// return "{$"+name_+"}";
	// }
	// public IRecordVariable getResolvedVariableQuiet(String name,
	// JavaVariablePath variablePath) throws ParsingException {
	// return this.usageType_.getSubVariableQuiet(name, variablePath);
	// }
	// public void updateNameBlock(NameBuilderCode.ComplexDataBlock
	// target,BaseTrackers.JavaVariablePath variablePath, NameBlockDetails
	// elementDetails) throws ParsingException {
	// this.usageType_.updateNameBlock(target, variablePath,elementDetails);
	// }
	//
	//
	// public void setUsageType( VariableType usageType) {
	// this.usageType_ = usageType;
	// }
	//
	// public void addAsMacroRecordParameter(MacroCallDataBlock macroCallBlock,
	// JavaVariablePath variablePath,IVariableType type) throws ParsingException{
	// this.usageType_.addAsMacroRecordParameter(macroCallBlock,
	// variablePath,type);
	//
	// }
	//
	// public void updateCallChainFinal(BasicCallChainTemplate target,
	// JavaVariablePath variablePathToThisVariable) throws ParsingException {
	// this.usageType_.updateCallChainFinal(target, variablePathToThisVariable);
	// }
	//
	// public void updateExpressionFinal(CodeBlockEnvironment environment,
	// ExpressionDetailsFormBlock expressionSkeleton, JavaVariablePath
	// variablePath) throws ParsingException {
	// this.usageType_.updateExpressionFinal(environment,expressionSkeleton,
	// variablePath);
	//
	// }
	// public SLayer getValueRecordFinal(JavaVariablePath variablePath) throws
	// ParsingException {
	// return usageType_.getValueRecordFinal(variablePath);
	// }
	// public FoundationNode getAsFoundataionNodeReferenceVariable(
	// JavaVariablePath variablePath) throws ParsingException {
	// return usageType_.getAsFoundataionNodeReferenceVariable(pathToExtend);
	// }
	//
	//
	//
	//
	// }
}
/*
 * OUTUPUT! OUTUPUT! FileRoot(260):Tokens:[JAVA_KEYWORD line 6 1,
 * EXTERNAL_KEYWORD line 6 7, ID line 6 17, LEFT_CURLY line 6 26, ID line 7 2,
 * ASSIGNMENT line 7 10, JAVA_KEYWORD line 7 13, FULL_STOP_SYMBOL line 7 18, ID
 * line 7 19, FULL_STOP_SYMBOL line 7 24, ID line 7 25, LEFT_CURLY line 7 33, ID
 * line 8 4, ID line 8 12, LEFT_ROUND line 8 20, RIGHT_ROUND line 8 21,
 * SEMI_COLON_SYMBOL line 8 22, ID line 9 4, ID line 9 9, LEFT_ROUND line 9 16,
 * RIGHT_ROUND line 9 17, SEMI_COLON_SYMBOL line 9 18, ID line 10 4, ID line 10
 * 13, LEFT_ROUND line 10 20, ID line 10 21, ID line 10 29, RIGHT_ROUND line 10
 * 35, SEMI_COLON_SYMBOL line 10 36, ID line 11 4, ID line 11 13, LEFT_ROUND
 * line 11 30, ID line 11 31, ID line 11 39, RIGHT_ROUND line 11 45,
 * SEMI_COLON_SYMBOL line 11 46, RIGHT_CURLY line 12 2, ID line 13 2,
 * LEFT_TRIANGLE line 13 15, ID line 13 17, RIGHT_TRIANGLE line 13 22,
 * ASSIGNMENT line 13 25, ID line 13 28, FULL_STOP_SYMBOL line 13 34, ID line 13
 * 35, FULL_STOP_SYMBOL line 13 43, ID line 13 44, LEFT_CURLY line 13 58, ID
 * line 14 3, ID line 14 9, LEFT_ROUND line 14 20, ID line 14 21, ID line 14 27,
 * RIGHT_ROUND line 14 29, SEMI_COLON_SYMBOL line 14 30, ID line 15 3, ID line
 * 15 9, LEFT_ROUND line 15 13, ID line 15 14, ID line 15 19, RIGHT_ROUND line
 * 15 25, SEMI_COLON_SYMBOL line 15 26, RIGHT_CURLY line 16 2, ID line 17 2,
 * ASSIGNMENT line 17 10, JAVA_KEYWORD line 17 13, FULL_STOP_SYMBOL line 17 18,
 * ID line 17 19, FULL_STOP_SYMBOL line 17 24, ID line 17 25, LEFT_CURLY line 17
 * 33, PUBLIC_KEYWORD line 18 3, ID line 18 11, ID line 18 24, SEMI_COLON_SYMBOL
 * line 18 28, RIGHT_CURLY line 19 2, ID line 20 2, ASSIGNMENT line 20 15,
 * JAVA_KEYWORD line 20 18, FULL_STOP_SYMBOL line 20 23, ID line 20 24,
 * FULL_STOP_SYMBOL line 20 27, ID line 20 28, LEFT_CURLY line 20 41,
 * PUBLIC_KEYWORD line 21 3, ID line 21 11, ID line 21 17, LEFT_ROUND line 21
 * 25, ID line 21 26, ID line 21 34, RIGHT_ROUND line 21 40, SEMI_COLON_SYMBOL
 * line 21 41, PUBLIC_KEYWORD line 22 3, ID line 22 11, ID line 22 17,
 * LEFT_ROUND line 22 23, ID line 22 24, ID line 22 32, RIGHT_ROUND line 22 38,
 * SEMI_COLON_SYMBOL line 22 39, RIGHT_CURLY line 23 2, RIGHT_CURLY line 24 1,
 * OUTPUT_KEYWORD line 27 1, ID line 27 9, FULL_STOP_SYMBOL line 27 18, ID line
 * 27 19, FULL_STOP_SYMBOL line 27 25, ID line 27 26, SEMI_COLON_SYMBOL line 27
 * 31, DESIGN_KEYWORD line 33 1, ID line 33 9, LEFT_ROUND line 33 15,
 * RIGHT_ROUND line 33 16, LEFT_CURLY line 33 18, ID line 34 2, LEFT_ROUND line
 * 34 8, ID line 34 9, ID line 34 17, COMMA_SYMBOL line 34 22, ID line 34 24, ID
 * line 34 29, RIGHT_ROUND line 34 35, SEMI_COLON_SYMBOL line 34 36, ID line 35
 * 2, LEFT_ROUND line 35 28, RIGHT_ROUND line 35 29, LEFT_CURLY line 35 31, ID
 * line 36 4, LEFT_ROUND line 36 15, ID line 36 16, ID line 36 21, RIGHT_ROUND
 * line 36 27, SEMI_COLON_SYMBOL line 36 28, ID line 37 4, LEFT_ROUND line 37
 * 18, ID line 37 19, ID line 37 27, RIGHT_ROUND line 37 33, SEMI_COLON_SYMBOL
 * line 37 34, ID line 38 4, LEFT_ROUND line 38 19, ID line 38 20, ID line 38
 * 29, RIGHT_ROUND line 38 35, SEMI_COLON_SYMBOL line 38 36, ID line 39 4,
 * LEFT_ROUND line 39 14, RIGHT_ROUND line 39 15, SEMI_COLON_SYMBOL line 39 16,
 * ID line 39 18, LEFT_ROUND line 39 28, RIGHT_ROUND line 39 29,
 * SEMI_COLON_SYMBOL line 39 30, ID line 40 4, LEFT_ROUND line 40 11, ID line 40
 * 12, ID line 40 39, COMMA_SYMBOL line 40 54, ID line 40 56, ID line 40 83,
 * RIGHT_ROUND line 40 99, LEFT_CURLY line 40 101, ID line 41 5, LEFT_ROUND line
 * 41 14, RIGHT_ROUND line 41 15, SEMI_COLON_SYMBOL line 41 16, ID line 41 18,
 * LEFT_ROUND line 41 30, RIGHT_ROUND line 41 31, SEMI_COLON_SYMBOL line 41 32,
 * ID line 41 34, LEFT_ROUND line 41 49, RIGHT_ROUND line 41 50,
 * SEMI_COLON_SYMBOL line 41 51, ID line 41 53, LEFT_ROUND line 41 62,
 * RIGHT_ROUND line 41 63, SEMI_COLON_SYMBOL line 41 64, RIGHT_CURLY line 42 4,
 * RIGHT_CURLY line 43 3, RIGHT_CURLY line 44 1, OUTPUT_KEYWORD line 45 1, ID
 * line 45 9, FULL_STOP_SYMBOL line 45 18, ID line 45 19, FULL_STOP_SYMBOL line
 * 45 25, ID line 45 26, SEMI_COLON_SYMBOL line 45 31, JAVA_KEYWORD line 47 1,
 * ID line 47 7, IMPLEMENTS_KEYWORD line 47 13, ID line 47 25, EXTERNAL_KEYWORD
 * line 47 32, ID line 47 42, LEFT_CURLY line 47 53, MACRO_KEYWORD line 48 2, ID
 * line 48 9, LEFT_ROUND line 48 37, ID line 48 38, ID line 48 65, COMMA_SYMBOL
 * line 48 76, VAR_KEYWORD line 48 77, ID line 48 82, ID line 48 87, RIGHT_ROUND
 * line 48 94, LEFT_CURLY line 48 96, CODE_KEYWORD line 49 4, LEFT_CURLY line 49
 * 10, ID line 50 5, ID line 50 10, ASSIGNMENT line 50 18, INTEGER line 50 21,
 * SEMI_COLON_SYMBOL line 50 23, ID line 51 5, ASSIGNMENT line 51 13, INTEGER
 * line 51 15, SEMI_COLON_SYMBOL line 51 17, LINK_KEYWORD line 52 5,
 * DOLLAR_SYMBOL line 52 11, ID line 52 12, FULL_STOP_SYMBOL line 52 23,
 * FULL_STOP_SYMBOL line 52 24, FULL_STOP_SYMBOL line 52 25, LINK_KEYWORD line
 * 53 5, ID line 53 11, ID line 53 23, ASSIGNMENT line 53 31, DOLLAR_SYMBOL line
 * 53 34, ID line 53 35, SEMI_COLON_SYMBOL line 53 41, LINK_KEYWORD line 55 5,
 * ID line 55 11, ID line 55 23, PLUS_SYMBOL line 55 30, PLUS_SYMBOL line 55 32,
 * SEMI_COLON_SYMBOL line 55 34, LINK_KEYWORD line 56 6, ID line 56 12, ID line
 * 56 24, MINUS_SYMBOL line 56 31, MINUS_SYMBOL line 56 33, SEMI_COLON_SYMBOL
 * line 56 35, LINK_KEYWORD line 58 5, ID line 58 11, LEFT_CURLY line 58 19, ID
 * line 59 6, ID line 59 11, SEMI_COLON_SYMBOL line 59 16, ID line 60 6, ID line
 * 60 11, SEMI_COLON_SYMBOL line 60 17, MACRO_KEYWORD line 61 6, ID line 61 13,
 * LEFT_ROUND line 61 41, ID line 61 42, ASSIGNMENT line 61 53, DOLLAR_SYMBOL
 * line 61 55, ID line 61 56, COMMA_SYMBOL line 61 71, ID line 61 74, ASSIGNMENT
 * line 61 81, ID line 61 83, RIGHT_ROUND line 61 88, SEMI_COLON_SYMBOL line 61
 * 89, MACRO_KEYWORD line 62 6, ID line 62 13, LEFT_ROUND line 62 41, ID line 62
 * 42, ASSIGNMENT line 62 53, DOLLAR_SYMBOL line 62 55, ID line 62 56,
 * COMMA_SYMBOL line 62 72, ID line 62 74, ASSIGNMENT line 62 81, ID line 62 83,
 * RIGHT_ROUND line 62 89, SEMI_COLON_SYMBOL line 62 90, LINK_KEYWORD line 63 6,
 * ID line 63 12, ID line 63 22, ASSIGNMENT line 63 30, ID line 63 33,
 * PLUS_SYMBOL line 63 38, ID line 63 40, SEMI_COLON_SYMBOL line 63 46,
 * LINK_KEYWORD line 64 6, ID line 64 12, ID line 64 28, ASSIGNMENT line 64 36,
 * ID line 64 39, SEMI_COLON_SYMBOL line 64 46, RIGHT_CURLY line 65 5,
 * RIGHT_CURLY line 66 4, RIGHT_CURLY line 67 3, LINK_KEYWORD line 69 2, ID line
 * 69 8, LEFT_CURLY line 69 36, PUBLIC_KEYWORD line 70 4, CLASS_KEYWORD line 70
 * 12, ID line 70 19, LEFT_CURLY line 70 35, PRIVATE_KEYWORD line 71 5, ID line
 * 71 14, ID line 71 19, ASSIGNMENT line 71 26, INTEGER line 71 29,
 * SEMI_COLON_SYMBOL line 71 31, PUBLIC_KEYWORD line 72 5, ID line 72 13, ID
 * line 72 19, LEFT_ROUND line 72 30, RIGHT_ROUND line 72 31, LEFT_CURLY line 72
 * 33, ID line 73 5, ID line 73 10, ASSIGNMENT line 73 18, INTEGER line 73 21,
 * SEMI_COLON_SYMBOL line 73 23, ID line 74 6, ID line 74 11, ASSIGNMENT line 74
 * 19, INTEGER line 74 22, SEMI_COLON_SYMBOL line 74 24, ID line 75 6,
 * ASSIGNMENT line 75 14, INTEGER line 75 16, SEMI_COLON_SYMBOL line 75 18,
 * LINK_KEYWORD line 76 6, ID line 76 12, ID line 76 24, ASSIGNMENT line 76 32,
 * DOLLAR_SYMBOL line 76 35, ID line 76 36, SEMI_COLON_SYMBOL line 76 42,
 * LINK_KEYWORD line 78 6, ID line 78 12, ID line 78 24, PLUS_SYMBOL line 78 31,
 * PLUS_SYMBOL line 78 33, SEMI_COLON_SYMBOL line 78 35, LINK_KEYWORD line 79 7,
 * ID line 79 13, ID line 79 25, MINUS_SYMBOL line 79 32, MINUS_SYMBOL line 79
 * 34, SEMI_COLON_SYMBOL line 79 36, LINK_KEYWORD line 81 6, ID line 81 12,
 * LEFT_CURLY line 81 20, ID line 82 7, ID line 82 12, SEMI_COLON_SYMBOL line 82
 * 17, ID line 83 7, ID line 83 12, SEMI_COLON_SYMBOL line 83 18, MACRO_KEYWORD
 * line 84 7, ID line 84 14, LEFT_ROUND line 84 42, ID line 84 43, ASSIGNMENT
 * line 84 54, DOLLAR_SYMBOL line 84 56, ID line 84 57, COMMA_SYMBOL line 84 72,
 * ID line 84 75, ASSIGNMENT line 84 82, ID line 84 84, RIGHT_ROUND line 84 89,
 * SEMI_COLON_SYMBOL line 84 90, MACRO_KEYWORD line 85 7, ID line 85 14,
 * LEFT_ROUND line 85 42, ID line 85 43, ASSIGNMENT line 85 54, DOLLAR_SYMBOL
 * line 85 56, ID line 85 57, COMMA_SYMBOL line 85 73, ID line 85 75, ASSIGNMENT
 * line 85 82, ID line 85 84, RIGHT_ROUND line 85 90, SEMI_COLON_SYMBOL line 85
 * 91, LINK_KEYWORD line 86 7, ID line 86 13, ID line 86 23, ASSIGNMENT line 86
 * 31, ID line 86 34, PLUS_SYMBOL line 86 39, ID line 86 41, SEMI_COLON_SYMBOL
 * line 86 47, LINK_KEYWORD line 87 7, ID line 87 13, ID line 87 23, ASSIGNMENT
 * line 87 31, ID line 87 34, SEMI_COLON_SYMBOL line 87 41, RIGHT_CURLY line 88
 * 6, RIGHT_CURLY line 89 5, RIGHT_CURLY line 90 3, RIGHT_CURLY line 91 3,
 * LINK_KEYWORD line 92 2, ID line 92 8, LEFT_CURLY line 92 15, PUBLIC_KEYWORD
 * line 93 3, CLASS_KEYWORD line 93 11, ID line 93 18, LEFT_SQUARE line 93 22,
 * DOLLAR_SYMBOL line 93 23, ID line 93 24, COMMA_SYMBOL line 93 29, ID line 93
 * 30, RIGHT_SQUARE line 93 34, LEFT_CURLY line 93 37, PUBLIC_KEYWORD line 94 4,
 * CONSTRUCTOR_KEYWORD line 94 12, LEFT_ROUND line 94 24, ID line 94 25, ID line
 * 94 30, RIGHT_ROUND line 94 32, LEFT_CURLY line 94 34, RIGHT_CURLY line 95 4,
 * PUBLIC_KEYWORD line 96 4, ID line 96 12, ID line 96 18, LEFT_ROUND line 96
 * 22, ID line 96 23, ID line 96 28, RIGHT_ROUND line 96 30, LEFT_CURLY line 96
 * 32, RIGHT_CURLY line 98 4, RIGHT_CURLY line 99 3, RIGHT_CURLY line 100 2,
 * RIGHT_CURLY line 101 1] FileRoot(266):Stats for file:TestSkeletonX.txt
 * FileRoot(267):Number of tokens used:432 Number of columns created:12 Total
 * number of table accesses:331 ===================== Memorisation counts (of 41
 * concrete patterns): JavaCallChain:0 JavaImplementationIncludeFile:0
 * JavaMacroBlock:0 ForInitialStatement:0 JavaImplementatonBlock:0
 * TargetTypeReference:4 LinkChain:0 MacroCallParameter:0 ParameterBlock:0
 * TargetCallArguments:0 JavaExternalElement:0 BuilderExpression:0
 * MacroParameter:0 Import:0 TargetParameter:2 TargetCallComponent:0
 * JavaStatement:1 NameValue:0 JavaCodeBlock:0 JavaMethodCall:0
 * TemplateElement:0 SimpleParameter:0 BuilderVariableChain:0 LabelingSubs:0
 * MacroElement:0 ChainHookList:0 JavaClassInterfaceElement:0 Generics:0
 * JavaImplementationElement:0 File:0 PackagePathElement:0 JavaExpression:18
 * DesignBlock:0 BaseTargetTypeReference:0 JavaImplementationSpecificsBlock:0
 * BlockElement:0 Name:3 GeneralMacroCall:0 JavaExternalBlock:0 JavaInterface:0
 * JavaClass:0 ---------------------- Total reuse:28
 * 
 * ParserControl(88):Doing complete parsing!
 * GeneralMacroCall$MacroCallParameters(369):Target variable(61):expression
 * GeneralMacroCall$MacroCallParameters(369):Target variable(61):target
 * GeneralMacroCall$MacroCallParameters(369):Target variable(62):expression
 * GeneralMacroCall$MacroCallParameters(369):Target variable(62):target
 * ClassConstruction(176):Basic build:ExpressionTest
 * GeneralMacroCall$MacroCallParameters(369):Target variable(84):expression
 * GeneralMacroCall$MacroCallParameters(369):Target variable(84):target
 * GeneralMacroCall$MacroCallParameters(369):Target variable(85):expression
 * GeneralMacroCall$MacroCallParameters(369):Target variable(85):target
 * ClassConstruction(176):Basic build:Moo DualIndirectLayerType(160):Link to
 * parent failed:VB:$expression DualIndirectLayerType(161):Current
 * path:1:org.ffd2.skeletonx
 * .skeleton.BaseTrackers$JavaVariablePath$LocalParentLink@1055e55f
 * DualIndirectLayerType(160):Link to parent failed:VB:$expression
 * DualIndirectLayerType(161):Current
 * path:1:org.ffd2.skeletonx.skeleton.BaseTrackers$JavaVariablePath$LocalParentLink
 * @664883c DualIndirectLayerType(160):Link to parent failed:VB:$expression
 * DualIndirectLayerType(161):Current
 * path:1:org.ffd2.skeletonx.skeleton.BaseTrackers$JavaVariablePath$LocalParentLink
 * @322c1e6b GeneralMacroCall$MacroCallAccess(268):Getting record var
 * for:expression GeneralMacroCall$CodeBlockContext(137):Record
 * request:org.ffd2.skeletonx
 * .compile.java.GeneralMacroCall$TargetVariableCallParameter@39dd3812
 * XCodeCallChainBlock$VariableSegment(204):Doing variable access:61
 * XCodeCallChainBlock$VariableSegment(208):Path to
 * variable:1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@1e9e5c73
 * RecordVariableActions$GetRecordAction(22):Get record re do
 * Action:1:org.ffd2.skeletonx
 * .skeleton.BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@1e9e5c73
 * RecordVariableActions$GetRecordAction(26):Found
 * variable:Slayer:[Root]:PackratVariableExpression
 * ({StringLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@27b15692,
 * Decrement=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@6baa9f99,
 * Binary=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@ed42d08,
 * Increment=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@bd86fd3,
 * IntLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@2e958bb8,
 * BooleanLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@5c07f4ec}) in
 * (Type
 * =org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager$RecordType
 * @e60e128m name=leftExpression)
 * ImplementationBlockSkeletonManager$RecordType(642
 * ):FOUND!!!1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@1e9e5c73
 * XCodeCallChainBlock$VariableSegment(218):Path to variable
 * post:1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@1e9e5c73
 * GeneralMacroCall$MacroCallAccess(268):Getting record var for:expression
 * GeneralMacroCall$CodeBlockContext(137):Record
 * request:org.ffd2.skeletonx.compile.java.
 * GeneralMacroCall$TargetVariableCallParameter@3abc8e1e
 * XCodeCallChainBlock$VariableSegment(204):Doing variable access:62
 * XCodeCallChainBlock$VariableSegment(208):Path to
 * variable:1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@311671b2
 * RecordVariableActions$GetRecordAction(22):Get record re do
 * Action:1:org.ffd2.skeletonx
 * .skeleton.BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@311671b2
 * RecordVariableActions$GetRecordAction(26):Found
 * variable:Slayer:[Root]:PackratVariableExpression
 * ({StringLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@27b15692,
 * Decrement=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@6baa9f99,
 * Binary=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@ed42d08,
 * Increment=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@bd86fd3,
 * IntLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@2e958bb8,
 * BooleanLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@5c07f4ec}) in
 * (Type
 * =org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager$RecordType
 * @3882764bm name=rightExpression)
 * ImplementationBlockSkeletonManager$RecordType
 * (642):FOUND!!!1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@311671b2
 * XCodeCallChainBlock$VariableSegment(218):Path to variable
 * post:1:org.ffd2.skeletonx.skeleton
 * .BaseTrackers$JavaVariablePath$LocalGlobalPeerLink@311671b2
 * DualIndirectLayerType(160):Link to parent failed:VB:$expression
 * DualIndirectLayerType(161):Current
 * path:2:org.ffd2.skeletonx.skeleton.BaseTrackers$JavaVariablePath$LocalParentLink
 * @7d2452e8 DualIndirectLayerType(160):Link to parent failed:VB:$expression
 * DualIndirectLayerType(161):Current
 * path:2:org.ffd2.skeletonx.skeleton.BaseTrackers$JavaVariablePath$LocalParentLink
 * @5bbf3d87 ClassConstruction(247):Doing final class construction:null
 * ClassConstruction(249):Simple class name:ExpressionTest(70)
 * JavaClassElements$VariableElement(456):Simple variable:frog_(71)
 * GeneralMacroCall$MacroCallAccess(268):Getting record var for:expression
 * GeneralMacroCall$CodeBlockContext(137):Record
 * request:org.ffd2.skeletonx.compile.java.
 * GeneralMacroCall$TargetVariableCallParameter@62f47396
 * XCodeCallChainBlock$VariableSegment(204):Doing variable access:84
 * XCodeCallChainBlock$VariableSegment(208):Path to variable:0:null
 * RecordVariableActions$GetRecordAction(22):Get record re do Action:0:null
 * RecordVariableActions$GetRecordAction(26):Found
 * variable:Slayer:[Root]:PackratVariableExpression
 * ({StringLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@27b15692,
 * Decrement=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@6baa9f99,
 * Binary=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@ed42d08,
 * Increment=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@bd86fd3,
 * IntLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@2e958bb8,
 * BooleanLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@5c07f4ec}) in
 * (Type
 * =org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager$RecordType
 * @e60e128m name=leftExpression)
 * ImplementationBlockSkeletonManager$RecordType(642):FOUND!!!0:null
 * XCodeCallChainBlock$VariableSegment(218):Path to variable post:0:null
 * GeneralMacroCall$MacroCallAccess(268):Getting record var for:expression
 * GeneralMacroCall$CodeBlockContext(137):Record
 * request:org.ffd2.skeletonx.compile.java.
 * GeneralMacroCall$TargetVariableCallParameter@1ed0af9b
 * XCodeCallChainBlock$VariableSegment(204):Doing variable access:85
 * XCodeCallChainBlock$VariableSegment(208):Path to variable:0:null
 * RecordVariableActions$GetRecordAction(22):Get record re do Action:0:null
 * RecordVariableActions$GetRecordAction(26):Found
 * variable:Slayer:[Root]:PackratVariableExpression
 * ({StringLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@27b15692,
 * Decrement=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@6baa9f99,
 * Binary=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@ed42d08,
 * Increment=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@bd86fd3,
 * IntLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@2e958bb8,
 * BooleanLiteral=org.ffd2.skeletonx.compile.java.layer.NodeLayerChild@5c07f4ec}) in
 * (Type
 * =org.ffd2.skeletonx.compile.java.layer.ImplementationBlockSkeletonManager$RecordType
 * @3882764bm name=rightExpression)
 * ImplementationBlockSkeletonManager$RecordType(642):FOUND!!!0:null
 * XCodeCallChainBlock$VariableSegment(218):Path to variable post:0:null
 * ClassConstruction(247):Doing final class
 * construction:org.ffd2.skeletonx.compile.java.NameBlock@2b313906 NameBlock(41):Adding
 * complex
 * NameBlock(43):Element:org.ffd2.skeletonx.compile.java.NameBlock$VariableOption
 * @60f47bf5 NameBlock(43):Element:org.ffd2.skeletonx.compile.java.NameBlock$3@24f9fdc
 * NameBlock(37):Adding simple:x NameBlock(37):Adding simple:y
 * ParserControl(72):Outputing all! No system errors. No user errors. No
 * warnings.
 */