package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.compile.java.VariableStore;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

public class MacroRootType implements LayerType {
	private final SLayer	peerMacroLayer_;
	private final SLayer parentLayer_;
	private final VariableStore	parameterVariables_;
	private final JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock	macroBlockSkeleton_;
	private final String	macroName_;
	public MacroRootType( String macroName, VariableStore parameterVariables, SLayer parentLayer, SLayer globalRootLayer,JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock macroBlockSkeleton, NonFoundationChildSLayerMaker childMaker,ImplementationBlockSkeletonManager blockSkeleton, FileEnvironment environment) {
		this.macroName_ = macroName;
		this.macroBlockSkeleton_ = macroBlockSkeleton;
		this.peerMacroLayer_ = new SLayer(this,new MacroChildMaker(childMaker,this),blockSkeleton,parentLayer.getRelatedTracker(),environment);
		this.parameterVariables_ = parameterVariables;

		this.parentLayer_ = parentLayer;
	}
	public IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
		pathToVariable.addDefiningLayerPeer(macroBlockSkeleton_);
		if(name.equals("baseLex")) {
			Thread.dumpStack();
		}
		IRecordVariable parameterVariable = this.parameterVariables_.getBuilderVariableQuiet(name);
		if(parameterVariable!=null) {
			return parameterVariable;
		} else {
			return parentLayer_.getLayerStrictBuilderVariableQuiet(name, pathToVariable);
		}
	}
	public String getFullName() { return parentLayer_.getFullName()+":[MacroRoot]"+macroName_; }
	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
		pathToFoundationLayer.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getRelatedFoundationNode(pathToFoundationLayer);
	}
	public SLayer getAncestorNodeSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		pathToParentLayer.addDefiningLayerPeer(macroBlockSkeleton_);
		return this.parentLayer_.getAncestorNodeSLayer(name, pathToParentLayer);
	}
	public SLayer getAncestorVariableSLayerBackwards(String name, JavaVariablePath pathToParentLayer) throws ParsingException {
		pathToParentLayer.addDefiningLayerPeer(macroBlockSkeleton_);
		return this.parentLayer_.getAncestorVariableSLayer(name, pathToParentLayer);
	}
	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		throw new RuntimeException("Not expected!");
	}
	private final static class MacroChildMaker implements NonFoundationChildSLayerMaker {
		private final NonFoundationChildSLayerMaker	peerBaseMaker_;
		private final MacroRootType	parent_;
		/**
		 * 
		 * @param peerBaseMaker the child layer maker that we'd normally use (from the context the macro is defined within)
		 */
		public MacroChildMaker(NonFoundationChildSLayerMaker peerBaseMaker, MacroRootType parent) {
			this.peerBaseMaker_ = peerBaseMaker;
			this.parent_ = parent;
			
		}
		
		public NodeLayerChild createNewNodeLayerChild(
					String name, ImplementationBlockSkeletonManager blockSkeleton, LinkupTracker relatedTracker, 
					SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber
				) throws ParsingException {
			BaseTrackers.JavaVariablePath pathToVariableDefinition = new BaseTrackers.JavaVariablePath();
//		final SLayer childLayer = parent.childLayer_;
			//TODO: FINISH
			Debug.finishMeMarker();
			//So, this doesn't quite work right
			//pathToVariableDefinition.addDefiningLayerPeer(this.parent_.macroBlockSkeleton_);
			AncestorConnection ancestor = parentLayerType.getAncestorFoundationNodeQuiet(name, pathToVariableDefinition,true);
			if(ancestor==null) {
				throw ParsingException.createObjectNotFound("node child", name);
			} else {	
				BaseTrackers.JavaVariablePath childPathToPeer = new BaseTrackers.JavaVariablePath();
				LocalChild childRef = 
						blockSkeleton.createLocalChild(
									name, parentLayer,
									MacroGeneralLayerType.createForNodeChild(ancestor.getTargetNode().getRelatedLayerForLayerComponents(), childPathToPeer,fileEnvironment), relatedTracker, ancestor);//,pathToVariableDefinition);
				SLayer childLayer = childRef.getChildLayer();
				childLayer.buildHorizontalPathToPeerLayer(ancestor.getTargetNode().getRelatedLayerForLayerComponents(), childPathToPeer);

				NodeLayerChild createdRecord = NodeLayerChild.createLocalChild(name, childLayer,childRef);
				
				return createdRecord;
				
//				
//				DualIndirectLayerType parentBridgeForChild = 
//					DualIndirectLayerType.buildLocalNodeChildBased(
//							childRef,
//							childLayer,
//							parentLayer,
//							MacroGeneralLayerType.createForNodeChild(parentLayer,fileEnvironment),
//							fileEnvironment
//						);
//				SLayer childLayer = parentBridgeForChild.getChildLayer();
//				return new VariableChild(name, childLayer,resolvedVariable, fileEnvironment,pathToVariableDefinition);
		
				//throw ParsingException.createGeneralSemantic("Node children not supported in macro roots yet!");

//				SLayer childLayer = childRef.getChildLayer();
//				NodeLayerChild createdRecord = NodeLayerChild.createNonFoundationLayerChild(name, childLayer);
//				return createdRecord;
			}
		}
		public VariableChild createNewVariableLayerChild(String name, IRecordVariable resolvedVariable, BaseTrackers.JavaVariablePath pathToVariableDefinition,
				SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber)
				throws ParsingException {	
			SLayer referencedFoundationNode = resolvedVariable.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
			BaseTrackers.JavaVariablePath childPathToPeer = new BaseTrackers.JavaVariablePath();

			//TODO: FINISH THIS!
			//THhis the part we have to change - we need the Child maker to be different here
			DualIndirectLayerType parentBridgeForChild = 
				DualIndirectLayerType.buildVariableBased(
						pathToVariableDefinition,
						referencedFoundationNode,
						parentLayer,
						MacroGeneralLayerType.createForVariableChild(referencedFoundationNode,childPathToPeer,resolvedVariable,fileEnvironment),//,pathToVariableDefinition),
						resolvedVariable,
						fileEnvironment
					);
			SLayer childLayer = parentBridgeForChild.getChildLayer();
			childLayer.buildHorizontalPathToPeerLayer(referencedFoundationNode, childPathToPeer);
			return new VariableChild(name, childLayer,resolvedVariable, fileEnvironment,pathToVariableDefinition);
		}
		public VariableChild 
						createNewVariableLayerChildNonLocal(
							String name, SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber
						) throws ParsingException {

			final BaseTrackers.JavaVariablePath pathToVariableDefinition = new BaseTrackers.JavaVariablePath();
			assert parentLayer==parent_.peerMacroLayer_: "Should match!";
			IRecordVariable foundParameter = parent_.parameterVariables_.getBuilderVariableQuiet(name);
			if(foundParameter!=null) {
				SLayer referencedFoundationNode = foundParameter.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
				BaseTrackers.JavaVariablePath childPathToPeer = new BaseTrackers.JavaVariablePath();

				//THhis the part we have to change - we need the Child maker to be different here
				DualIndirectLayerType parentBridgeForChild = 
					DualIndirectLayerType.buildVariableBased(
							pathToVariableDefinition,
							referencedFoundationNode,
							parentLayer,
							MacroGeneralLayerType.createForVariableChild(referencedFoundationNode,childPathToPeer,foundParameter,fileEnvironment),//,pathToVariableDefinition),
							foundParameter,
							fileEnvironment
						);
				SLayer childLayer = parentBridgeForChild.getChildLayer();
				//TODO Not sure if right
				childLayer.buildHorizontalPathToPeerLayer(referencedFoundationNode, childPathToPeer);

				return new VariableChild(name, childLayer,foundParameter, fileEnvironment,pathToVariableDefinition);
			} else {
//				IRecordVariable resolvedVariable = parentLayer.getBuilderVariableQuiet(name, pathToVariableDefinition,true);
//				if(resolvedVariable==null) {
					throw ParsingException.createObjectNotFound("skeleton variable", name);
//				} else {
//	
//					FoundationNode referencedFoundationNode = resolvedVariable.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
//					//TODO: FINISH THIS!
//					//THhis the part we have to change - we need the Child maker to be different here
//					
//					DualIndirectLayerType parentBridgeForChild = DualIndirectLayerType.buildVariableBased(pathToVariableDefinition,referencedFoundationNode,parentLayer,new MacroGeneralLayerType(parentLayer),fileEnvironment);
//					SLayer childLayer = parentBridgeForChild.getChildLayer();
//					return new VariableChild(name, childLayer, fileEnvironment);
//				}
			}
		}
	}
	
	// ==========================================================================
	
	public SLayer getPeerLayer() { return this.peerMacroLayer_; }
	public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
  	return parentLayer_.getChainDefinition(name);
  }
	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
  	return parentLayer_.getMappingDefinition(name);
  }
	public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
  	return parentLayer_.getKeyDefinition(name);
  }
	public FoundationNode getRootFoundationNodeBasic() { return parentLayer_.getRootFoundationNodeBasic(); }


	
	public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		IRecordVariable base = this.parameterVariables_.getBuilderVariableQuiet(name) ;
		if(base!=null) {
			return base;
		} else {
			pathToVariable.addDefiningLayerPeer(this.macroBlockSkeleton_);
			return parentLayer_.getBuilderVariableQuiet(name, pathToVariable, isFirstLayer);
		}
	}

	public boolean buildPathToType(TargetType type, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		pathToDeclaration.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.buildPathToType(type, pathToDeclaration, isFirstLayer);
	}

	public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables, 	TargetType outputClass, String variableName, JavaVariablePath chainToVariable, boolean isFirstLayer) {
		TargetVariable base = null;
		
		if(allowLocalVariables) {
			base = this.parameterVariables_.getTargetVariableQuiet(variableName) ;
		}
		if(base!=null ) {
			return base;
		} else {
			chainToVariable.addDefiningLayerPeer(this.macroBlockSkeleton_);
			return parentLayer_.getTargetVariableQuiet(allowLocalVariables, outputClass, variableName, chainToVariable, false);
		}
	}

	public TargetMethod getTargetMethodQuiet(TargetType outputClass, String name, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		pathToDeclaration.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getTargetMethodQuiet(outputClass, name, pathToDeclaration, isFirstLayer);
	}

	public TargetType getTargetTypeQuiet(String typeName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		// TODO: link in with parameter types
		pathToDeclaration.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getTargetTypeQuiet(typeName, pathToDeclaration, false);
//		return null;
	}

	public MacroConstruction getMacroConstructionFinal(String name,BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		pathToContainingLayer.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getMacroConstructionFinal(name,pathToContainingLayer);
	}

	public GlobalClassReference getGlobalClassReferenceQuiet(String className, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		pathToDeclaration.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getGlobalClassReferenceQuiet(className,pathToDeclaration,isFirstLayer);
	}

	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		pathToVariable.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getGlobalMethodReferenceQuiet(baseClass,methodName,pathToVariable,isFirstLayer);
	
	}

	public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope, String variableName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		pathToDeclaration.addDefiningLayerPeer(this.macroBlockSkeleton_);
		return parentLayer_.getLocalVariableQuiet(relatedScope,variableName,pathToDeclaration,isFirstLayer);
	}

	public boolean isLocalVariableQuiet(ScopeContainer relatedScope,String variableName, boolean isFirstLayer) {
		return parentLayer_.isLocalTargetVariableQuiet(relatedScope,variableName,isFirstLayer);
	}

	public void buildPathToUpperLayerBack(SLayer ancestor, JavaVariablePath path) {
		path.addDefiningLayerPeer(this.macroBlockSkeleton_);
	  parentLayer_.buildPathToUpperLayer(ancestor, path);
	}
	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path) {
		path.addDefiningLayerPeer(this.macroBlockSkeleton_);
	  return parentLayer_.buildPathToFoundationNode(target, path);
	}
	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		path.addDefiningLayerPeer(this.macroBlockSkeleton_);
	  return parentLayer_.buildPathToPeerLayer(target, path);
	  
	}
	public boolean buildHorizontalPathToPeerLayer(SLayer target, JavaVariablePath path) {
		return false;
	}
	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
		Thread.dumpStack();
		return false;
	}
	public AncestorConnection getAncestorFoundationNodeQuiet( String foundationNodeName, JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer) {
		pathToAncestorNodeParent.addDefiningLayerPeer(this.macroBlockSkeleton_);
	  return parentLayer_.getAncestorFoundationNodeQuiet( foundationNodeName, pathToAncestorNodeParent,isFirstLayer);
	}
	


}
