package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.DesignRecordLink;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.DataBlockEnvironment;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;

import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;

/**
 * This is the bridge to the "super root", above the layer representing the root. 
 * For the most part this means acting as a terminator for back access methods
 * Note: the RootLayer/Node has no parent (so most of the get methods only signal "item not found", or pass on query to peer Foundation Node
 * @author Matthew Goode
 *
 */
public class RootLayerType implements LayerType {
	private final FoundationNode	peerNode_;
	private final SLayer	rootLayer_;
	private final DesignRecordLink designLink_;
	private final DataBlockEnvironment	parentEnvironment_;
	
	public RootLayerType(FoundationNode rootNodePeer, ImplementationBlockSkeletonManager blockSkeleton,DesignRecordLink designLink, DataBlockEnvironment parentEnvironment, LinkupTracker relatedLinkup, FileEnvironment fileEnvironment) {
		this.peerNode_ = rootNodePeer;
		this.designLink_ = designLink;
		this.parentEnvironment_ = parentEnvironment;
		this.rootLayer_ = new SLayer(this,SLayer.NORMAL_CHILD_MAKER,blockSkeleton,relatedLinkup,fileEnvironment);
	}
	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
		return peerNode_;
	}
	public SLayer getRootLayer() {
		return rootLayer_;
	}
	public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
  	return designLink_.getChainDefinition(name);
  }
	public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
  	return designLink_.getMappingDefinition(name);
  }
	public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
  	return designLink_.getKeyDefinition(name);
  }
	public FoundationNode getRootFoundationNodeBasic() { return peerNode_; }

	public void buildPathToUpperLayerBack(SLayer ancestor, JavaVariablePath path) {
		//If this happens then that means some layer has a funny idea of its ancestors....
		throw new RuntimeException("THIS SHOULD NOT HAPPEN!");
	}
	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path) {
		//If this happens then that means some layer has a funny idea of its ancestors....
		return false;
	}
	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		//Thread.dumpStack();
		return false;
	}
	public boolean buildHorizontalPathToPeerLayer(SLayer target, JavaVariablePath path) {
		return false;
	}
	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager base, ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
		Thread.dumpStack();
		return false;
	}
	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		throw new RuntimeException("Not expected!");
	}
	public SLayer getAncestorNodeSLayerBackwards(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		throw ParsingException.createObjectNotFound("Node Layer", name);
	}
	public SLayer getAncestorVariableSLayerBackwards(String name, JavaVariablePath pathToParentLayer) throws ParsingException {
		throw ParsingException.createObjectNotFound("Variable SLayer", name);
	}
	public boolean buildPathToType(TargetType type, JavaVariablePath pathStore) {
		//This may happen for Fixed types (primitives/externals)
		Debug.println("Hmmmm?");
		// Maybe remove elements from pathStore?
		// Should try to remove.
		return true;
		//throw new RuntimeException("THIS SHOULD NOT HAPPEN!");
	}
	public String getFullName() { return "[Root]"; }

	public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		int mark = pathToVariable.mark();
		
		IRecordVariable viaPeer = peerNode_.getBuilderParameterQuiet(name, pathToVariable, true);
		if(viaPeer!=null) {
			return viaPeer;
		} else {
			pathToVariable.unmark(mark);
			return this.parentEnvironment_.getBuilderVariableQuiet(name, pathToVariable);
		}
	}
	public boolean buildPathToType(TargetType type, JavaVariablePath pathStore, boolean isFirstLayer) {
		return false;//peerNode_.buildPathToType(type, pathStore, true);
	}
	public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables, TargetType outputClass, String variableName, JavaVariablePath chainToVariable, boolean isFirstLayer) {
		return this.parentEnvironment_.getTargetVariableQuiet(allowLocalVariables, outputClass, variableName, chainToVariable);
	}
	public TargetMethod getTargetMethodQuiet(TargetType outputClass, String name, JavaVariablePath chainToVariable, boolean isFirstLayer) {
		return this.parentEnvironment_.getTargetMethodQuiet(outputClass, name, chainToVariable);
	}
	public TargetType getTargetTypeQuiet(String name, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		return this.parentEnvironment_.getTargetTypeQuiet(name, pathToDeclaration);
	}
	public MacroConstruction getMacroConstructionFinal(String name,BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		throw new ItemNotFoundException(name,"macro");
	}
	public GlobalClassReference getGlobalClassReferenceQuiet(String className, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		return null; //peerNode_.getGlobalClassReferenceQuiet(className, pathToVariable);
	}
	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		return null; //peerNode_.getGlobalMethodReferenceQuiet(baseClass, methodName, pathToVariable);
	}
	public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope, String variableName, JavaVariablePath pathToDeclaration, 	boolean isFirstLayer) {
		return null;
	}
	public AncestorConnection getAncestorFoundationNodeQuiet( String foundationNodeName, JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer) {
		return null;
	}
	public boolean isLocalVariableQuiet(ScopeContainer relatedScope, String variableName, boolean isFirstLayer) {
		return false;
	}
	
	public IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
		return null;
	}
}
