package org.ffd2.skeletonx.compile.java.layer;

import org.ffd2.solar.exceptions.ItemAlreadyExistsException;
import org.ffd2.solar.exceptions.ItemNotFoundException;
import org.ffd2.solar.exceptions.ParsingException;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.language.AccessStoreX;
import org.ffd2.solar.language.SpecificCommonErrorOutput;

import org.ffd2.skeletonx.compile.impl.ChainDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.FileEnvironment;
import org.ffd2.skeletonx.compile.impl.KeyDefinitionBlock;
import org.ffd2.skeletonx.compile.impl.MappingDefinitionBlock;
import org.ffd2.skeletonx.compile.java.DataBlockEnvironment;
import org.ffd2.skeletonx.compile.java.DirectTargetElementAccess;
import org.ffd2.skeletonx.compile.java.GlobalClassReference;
import org.ffd2.skeletonx.compile.java.GlobalMethodReference;
import org.ffd2.skeletonx.compile.java.IRecordVariable;
import org.ffd2.skeletonx.compile.java.JavaConcreteChainLink;
import org.ffd2.skeletonx.compile.java.LinkupTracker;
import org.ffd2.skeletonx.compile.java.MacroCallInstance;
import org.ffd2.skeletonx.compile.java.MacroCallRequirementsAccess;
import org.ffd2.skeletonx.compile.java.MacroConstruction;
import org.ffd2.skeletonx.compile.java.SVariableHolder;
import org.ffd2.skeletonx.compile.java.ScopeContainer;
import org.ffd2.skeletonx.compile.java.TargetMethod;
import org.ffd2.skeletonx.compile.java.TargetType;
import org.ffd2.skeletonx.compile.java.TargetVariable;
import org.ffd2.skeletonx.compile.java.VariableStore;
import org.ffd2.skeletonx.skeleton.BaseBuilder;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.GeneralExpressionDefinitionDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedArgumentsStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedCodeBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedForBlockStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedParametersStoreDataBlock;
import org.ffd2.skeletonx.skeleton.JavaImplementationFormHolder.JavaImplementationFormBlock.BlockDataBlock.LinkedTypeListDataBlock;
import org.ffd2.skeletonx.skeleton.BaseTrackers;
import org.ffd2.skeletonx.skeleton.BaseTrackers.JavaVariablePath;

public class SLayer  {
	private final FileEnvironment	fileEnvironment_;

	private final AccessStoreX<VariableChild> variableChildren_;
	//This is for all the node children that were first created via a call from the implementation 
	// (so may include more than the design specifications - being errors - and maybe less than all actual foundation/variable references
	private final AccessStoreX<NodeLayerChild> nodeChildrenWithLinkUsageReferences_;

//	private final AccessStore<InstantiatedIVariable> instantiatedIVariables_ = new AccessStore<InstantiatedIVariable>();
	
  private final AccessStoreX<SVariableHolder> classVariables_;
  private final AccessStoreX<SLocalVariableHolder> localVariables_;
  private final AccessStoreX<SMethodHolder> classMethods_;
  
  //Globals
  private final AccessStoreX<GlobalClassReference> globalClassReferences_;
  private final AccessStoreX<MethodReferenceHolder> globalMethodReferences_;

  
  private final AccessStoreX<TargetType> types_;
 
  private final AccessStoreX<MacroConstruction> macros_;
  private final AccessStoreX<IRecordVariable> layerBuilderVariables_;
	
	private final LayerType layerType_;
//	private FoundationNode referencingFoundationNode_;
	
	private final ImplementationBlockSkeletonManager blockSkeleton_;
	
	private SimpleVector<LinkHolder> concreteChainLinks_ = null;
	private final DataBlockEnvironment asDataBlockEnvironment_;

	private final LinkupTracker	relatedTracker_;

	// =========================================================================
	public static final NonFoundationChildSLayerMaker NORMAL_CHILD_MAKER = new NonFoundationChildSLayerMaker() {
//		public VariableChild createNewVariableLayerChildOld(String name, SLayer parentLayer, FileEnvironment fileEnvironment) throws ParsingException {
//			final BaseTrackers.JavaVariablePath pathToVariableDefinition = new BaseTrackers.JavaVariablePath();
//			IRecordVariable resolvedVariable = parentLayer.getBuilderVariableQuiet(name, pathToVariableDefinition,true);
//			
//			if(resolvedVariable==null) {
//				throw ParsingException.createObjectNotFound("skeleton variable", name);
//			} else {
//
//				FoundationNode referencedFoundationNode = resolvedVariable.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
//				DualIndirectLayerType parentBridgeForChild = DualIndirectLayerType.buildVariableBased(pathToVariableDefinition,referencedFoundationNode,parentLayer,this,fileEnvironment);
//				SLayer childLayer = parentBridgeForChild.getChildLayer();
//				return new VariableChild(name, childLayer, fileEnvironment);
//			}
//		}

		public VariableChild createNewVariableLayerChildNonLocal(String name, SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber)
				throws ParsingException {

			final BaseTrackers.JavaVariablePath pathToVariableDefinition = new BaseTrackers.JavaVariablePath();
			IRecordVariable resolvedVariable = parentLayer.getBuilderVariableQuiet(name, pathToVariableDefinition,true);
			if(resolvedVariable==null) {
				throw ParsingException.createObjectNotFound("skeleton variable", name);
			} else {
				return createNewVariableLayerChild(name, resolvedVariable, pathToVariableDefinition, parentLayer, parentLayerType, fileEnvironment, lineNumber);
			}
		}
		public VariableChild createNewVariableLayerChild(String name, IRecordVariable resolvedVariable, BaseTrackers.JavaVariablePath pathToVariableDefinition,
				SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber)
				throws ParsingException {		
			SLayer nextLayer = resolvedVariable.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
			DualIndirectLayerType parentBridgeForChild = 
					DualIndirectLayerType.buildVariableBased(pathToVariableDefinition,nextLayer,parentLayer,new RedirectedChildSLayerMaker(nextLayer,resolvedVariable,pathToVariableDefinition),resolvedVariable,fileEnvironment);
			SLayer childLayer = parentBridgeForChild.getChildLayer();
			return new VariableChild(name, childLayer,resolvedVariable, fileEnvironment,pathToVariableDefinition);
		}

		public NodeLayerChild createNewNodeLayerChild(String name, ImplementationBlockSkeletonManager blockSkeleton, LinkupTracker	relatedTracker, SLayer parentLayer, LayerType parentLayerType, FileEnvironment fileEnvironment, int lineNumber) throws ParsingException {
			BaseTrackers.JavaVariablePath path = new BaseTrackers.JavaVariablePath();
//		final SLayer childLayer = parent.childLayer_;
			AncestorConnection ancestor = parentLayerType.getAncestorFoundationNodeQuiet(name, path,true);
			if(ancestor==null) {
				throw ParsingException.createObjectNotFound("node child", name);
			} else {				
				//This might be a bit wrong becuase 
				LocalChild childRef = blockSkeleton.createLocalChild(name, parentLayer,new RedirectedChildSLayerMaker(parentLayer,null,path), relatedTracker, ancestor);//,path);
				SLayer childLayer = childRef.getChildLayer();
				path.addLocalGlobalPeer(childRef.getIsLocalSkeleton());
				NodeLayerChild createdRecord =NodeLayerChild.createLocalChild(name, childLayer,childRef);
				return createdRecord;
			}
		}
	};		
	private final NonFoundationChildSLayerMaker childLayerMaker_;
	
	// =========================================================================
	// Constructors
	/**
	 * Main constructor
	 * @param environment
	 * @param requestErrors this may be null if this Container is created specifically (as opposed to input file suggestion)
	 * @param baseError the base error used for creating other errors (does not have to specifically be for any particular line)
	 */
	public SLayer(LayerType layerType,NonFoundationChildSLayerMaker childMaker,ImplementationBlockSkeletonManager blockSkeleton, LinkupTracker relatedTracker, FileEnvironment environment) {
		this.blockSkeleton_ = blockSkeleton;
		this.childLayerMaker_= childMaker;
		this.layerType_ = layerType;
		this.relatedTracker_ = relatedTracker;
		this.fileEnvironment_ = environment;
		this.variableChildren_ = environment.createEntityStore("variable child");
		this.nodeChildrenWithLinkUsageReferences_ = environment.createEntityStore("node child");

		classVariables_  = environment.createEntityStore("class variable");
	  localVariables_  = environment.createEntityStore("local variable");
	  this.globalClassReferences_ = environment.createEntityStore("global class");
	  this.globalMethodReferences_ = environment.createEntityStore("global method");
	  classMethods_  = environment.createEntityStore("class method");
	  this.layerBuilderVariables_ = environment.createEntityStore("builder variable");
	  types_  = environment.createEntityStore("type");
	  macros_  = environment.createEntityStore("macro");
	  this.asDataBlockEnvironment_ = new DataBlockEnvironment() {
			public IRecordVariable getBuilderVariableQuiet(String name, JavaVariablePath pathToVariable){
				return SLayer.this.getBuilderVariableQuiet(name, pathToVariable, true);
			}
			public TargetVariable getTargetVariableQuiet(boolean allowLocal, TargetType targetType, String name, JavaVariablePath pathToDeclaration) {
				return SLayer.this.getTargetVariableQuiet(allowLocal, targetType, name, pathToDeclaration, true);
			}
			
			public TargetType getTargetTypeQuiet(String name, JavaVariablePath pathToDeclaration) {
				return SLayer.this.getTargetTypeQuiet(name, pathToDeclaration,true);
			}
			public TargetMethod getTargetMethodQuiet(TargetType targetType, String name, JavaVariablePath pathToDeclaration) {
				return SLayer.this.getTargetMethodQuiet(targetType, name, pathToDeclaration, true);
			}
		};
	}
	public String toString() { return "Slayer:"+layerType_.getFullName()+"("+nodeChildrenWithLinkUsageReferences_+")"; }
	public FoundationNode getRootFoundationNodeBasic() {
		return this.layerType_.getRootFoundationNodeBasic();
	}
	public FileEnvironment getFileEnvironment() {
		return this.fileEnvironment_;
	}
	public DataBlockEnvironment getAsDataBlockEnvironment() {
		return this.asDataBlockEnvironment_;
	}
	/**
	 * Find the underlying Foundation layer for this SLayer (so, if variable, find foundation layer referenced, if foundation layer, return 
	 * direct, if reused node child, a reference back to defining FoundationNode
	 * @param pathToFoundationLayer the path back to the defining layer
	 * @return the related FoundationNode
	 */
	public FoundationNode getRelatedFoundationNode(BaseTrackers.JavaVariablePath pathToFoundationLayer) {
		return this.layerType_.getRelatedFoundationNode(pathToFoundationLayer);
	}
	public String getFullName() {
		return this.layerType_.getFullName();
	}
	public final LinkupTracker getRelatedTracker() {
		return this.relatedTracker_;
	}
//Use wisely
  public JavaImplementationFormBlock.BlockDataBlock	getSkeletonBase() { return this.blockSkeleton_.getSkeletonBase(); }

	public void setLocalChildTypeAsPeerLayer(JavaImplementationFormBlock.BlockDataBlock.LocalChildRefDataBlock localChildBaseSkeleton, BaseTrackers.JavaVariablePath targetVariablePathToFoundationChildrenChainTracker) {
		this.layerType_.setLocalChildTypeAsPeerLayer(localChildBaseSkeleton, targetVariablePathToFoundationChildrenChainTracker);
	}
	// ==========================================================================
	// Stuff to be removed at some point (or shifted to ImplementationSkeleton thingy
//	public int nextIDIndex() {
//		return this.blockSkeleton_.nextGeneralIDIndex();
//	}
	// ==========================================================================
	/**
	 * Build the path to the specified type
	 * @return true if type found
	 */
	public boolean buildPathToType(TargetType type, BaseTrackers.JavaVariablePath pathStore, boolean isFirstLayer) {
		if(this.globalClassReferences_.isContainsValue(type)) {
			return true; //Caller is assumed to be "pointing" at this node
		} else if(this.types_.isContainsValue(type)) {
			return true;
		} else {
			return layerType_.buildPathToType(type, pathStore,isFirstLayer);
		}
	}

	// ==========================================================================
//	public static final SLayer createRootContainer(FileEnvironment environment, SpecificCommonErrorOutput baseError) {
//		return new SLayer(environment,  baseError);
//	}
	
	public void buildPathToUpperLayer(SLayer ancestor, BaseTrackers.JavaVariablePath path) {
		//Maybe this needs to check the Global track (in variable layers)
		if(this!=ancestor) {
			layerType_.buildPathToUpperLayerBack(ancestor, path);
		}
	}
	public BaseTrackers.JavaVariablePath buildPathToUpperLayer(SLayer ancestor ) {
		BaseTrackers.JavaVariablePath path = new BaseTrackers.JavaVariablePath();
		buildPathToUpperLayer(ancestor,path);
		return path;
	}
	public boolean buildPathToFoundationNode(FoundationNode target, BaseTrackers.JavaVariablePath path) {
		//Maybe this needs to check the Global track (in variable layers)
		if(target.getGlobalSLayer()==this) {
			return true;
		} else {
			return layerType_.buildPathToFoundationNode(target, path);
		} 
	}
	public BaseTrackers.JavaVariablePath buildPathToFoundationNode(FoundationNode target ) {
		BaseTrackers.JavaVariablePath path = new BaseTrackers.JavaVariablePath();
		buildPathToFoundationNode(target,path);
		return path;
	}
	public boolean buildHorizontalPathToPeerLayerViaSkeleton(SLayer target, BaseTrackers.JavaVariablePath path) {
		return buildHorizontalPathToPeerSkeleton(target.blockSkeleton_, path);
	}
	public boolean buildHorizontalPathToPeerSkeleton(ImplementationBlockSkeletonManager target, BaseTrackers.JavaVariablePath path) {
		if(target==this.blockSkeleton_) {
			return true;
		} else {
			return layerType_.buildHorizontalPathToPeerSkeleton(this.blockSkeleton_, target, path);
		}
	}
	public boolean buildHorizontalPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		if(target==this) {
			return true;
		} else {
			return layerType_.buildHorizontalPathToPeerLayer(target, path);
		}
	}
	public boolean buildPathToPeerLayer(SLayer target, BaseTrackers.JavaVariablePath path) {
		//Maybe this needs to check the Global track (in variable layers)
		if(target==this) {
			return true;
		}
		return layerType_.buildPathToPeerLayer(target, path);
		
	}
	public BaseTrackers.JavaVariablePath buildPathToPeerLayer(SLayer target ) {
		
		BaseTrackers.JavaVariablePath path = new BaseTrackers.JavaVariablePath();
		if(!buildPathToPeerLayer(target,path)) {
			throw new RuntimeException("This should not happen:"+ path);
		}
		
		return path;
	}
	public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock addItemSkeleton() {
		return this.blockSkeleton_.addItemSkeleton();
	}
	//Use sparingly....
	public ImplementationBlockSkeletonManager getBlockSkeleton() {
		return this.blockSkeleton_;
	}
	/**
	 * Only guarranteed to be meaningful if called after basicBuild() has been called on this layer
	 * (so may be called from the basicBuild() methods of children on their ancestors
	 * @return
	 */
//	public FoundationNode getReferencingFoundationNode() {
//		return this.referencingFoundationNode_;
//	}
	public AncestorConnection getAncestorFoundationNodeQuiet(String foundationNodeName, BaseTrackers.JavaVariablePath pathToAncestorNodeParent, boolean isFirstLayer) {
		return this.layerType_.getAncestorFoundationNodeQuiet(foundationNodeName, pathToAncestorNodeParent, isFirstLayer);
		
	}
	// =========================================================================
// To be classified
	public void addConcreteChainLink(String chainName, String linkName, JavaConcreteChainLink link, SpecificCommonErrorOutput definitionError) throws ParsingException {
  	if(this.concreteChainLinks_==null) {
  		this.concreteChainLinks_ = new SimpleVector<LinkHolder>();
  	}
  	this.concreteChainLinks_.addElement(new LinkHolder(chainName, linkName, link,definitionError));
  }
  public JavaConcreteChainLink getChainLinkFinal(String chainName, String linkName, BaseTrackers.JavaVariablePath pathToVariablesStore) throws ParsingException {
  	if(concreteChainLinks_!=null) {
  		for(LinkHolder link : this.concreteChainLinks_) {
  			if(link.isMatchingLink(chainName, linkName)) {
  				return link.getLink();
  			}
  		}
  	}
  	//TODO: backtrack to parent...
  	throw ParsingException.createObjectNotFound("link", "Chain:"+chainName+", link:"+linkName);
  }

	// =========================================================================
	// Resolution methods
	/**
	 * Because this is called after the initial setup stuff, the base Skeleton should be
	 * set up if one of the "design" layers, or alternatively set up as we recurse.
	 * @param relatedTracker
	 */
//	 void basicBuild( LayerType myType, ImplementationBlockSkeletonManager blockSkeleton, LinkupTracker skeletonBuilder) {
//		setType(myType);
//		this.blockSkeleton_ = blockSkeleton;
//		
//		//Need to build root elements as well
//		myType.basicBuildNodeChildren( skeletonBuilder,blockSkeleton, this.nodeChildrenWithLinkUsageReferences_);
//
//		for(VariableChild child : this.variableChildren_) {
//			child.basicBuild(blockSkeleton,skeletonBuilder, this);
//		}
////		for(ReferencedBuildVariable variable : this.referencedBuilderVariables_) {
////			variable.basicBuild(relatedTracker);
////		}
//		//TODO: add stuff for variables.
//		//Not yet - assume Design variables are built in FoundationNode (even though we know about them). 
//	}
	// ==========================================================================
  public LinkedCodeBlockStoreDataBlock allocateLinkedCode( ) {
		return this.blockSkeleton_.allocateLinkedCode( );	
	}
  public GeneralExpressionDefinitionDataBlock allocateExpressionDefinition() {
  	return this.blockSkeleton_.allocateExpressionDefinition();
  }
  
  public LinkedForBlockStoreDataBlock allocateForBlock( ) {
		return this.blockSkeleton_.allocateForBlock( );	
	}
	public  JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock.RootTypeDataBlock addRootTypeSearchBlock(FoundationNode type, int searchLineNumber) {
		return type.addSearchBlockForThisNodeAsTarget(this.blockSkeleton_,searchLineNumber); //back and forth stuff - just got to make sure there is no "double back" calling
	}
	public  JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock addSearchBlock(SLayer type, int searchLineNumber) {
		return type.addSearchBlockForThisNodeAsTarget(this.blockSkeleton_,searchLineNumber); //back and forth stuff - just got to make sure there is no "double back" calling
	}
	public  JavaImplementationFormBlock.BlockDataBlock.SearchRecordDataBlock addSearchBlockForThisNodeAsTarget(ImplementationBlockSkeletonManager targetBuilder, int searchLineNumber) {
		return targetBuilder.addSearchBlock(this.blockSkeleton_,searchLineNumber);
	}
  public LinkedArgumentsStoreDataBlock allocateLinkedArgumentSetter() {
		return this.blockSkeleton_.allocateLinkedArgumentSetter() ;	
  }
  public LinkedTypeListDataBlock allocateLinkedTypeList() {
		return this.blockSkeleton_.allocateLinkedTypeList() ;	

  }
  public LinkedParametersStoreDataBlock allocateLinkedParameters() {
		return this.blockSkeleton_.allocateLinkedParameters() ;	
  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedArgumentsSetDataBlock 
  		createArgumentLinkage( 
  			LinkedArgumentsStoreDataBlock targetArgumentSet, BaseTrackers.JavaVariablePath pathToTargetRecord
  		) {
  	
		return this.blockSkeleton_.createArgumentLinkage(targetArgumentSet.getVariableStoreParameter(), pathToTargetRecord);
  }
//  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.LinkedTypeListSetDataBlock createTypeListLinkage( LinkedTypeListDataBlock targetArgumentStore, BaseTrackers.JavaVariablePath pathToTargetRecord) {
//		return this.blockSkeleton_.createTypeListLinkage(targetArgumentStore.getVariableStoreParameter(), pathToTargetRecord);
//  }
  public JavaImplementationFormBlock.BlockDataBlock.ItemDataBlock.GeneralTargetVariableDataBlock addGeneralTargetVariableCodeVariable(String variableName) {
  	return this.blockSkeleton_.addGeneralTargetVariableCodeVariable(variableName);
  }
  
 
	
	// ==========================================================================
  /**
   * Build up a path to a TargetType declaration, starting from this layer
   * Checks if type is DirectAccess (in which case, no path is needed) - if you
   * don't need this check, use the accessTypeInternal() method
   * @see accessInternalType()
   */
  
  //Replaced by buildPathStuff()
//  public final void accessGeneralType(TargetType outputClass, BaseTrackers.JavaVariablePath chainToVariableStore) {
//    if(outputClass.getDirectElementAccessQuiet()==null) {
//	    accessInternalType(outputClass, chainToVariableStore);
//    }
//  }
//  public final void accessInternalType(TargetType outputClass, BaseTrackers.JavaVariablePath chainToVariableStore) {
//  	boolean found = this.queryAccessTypeInternal(outputClass, chainToVariableStore);
//    if(!found) {
//      this.layerType_.accessInternalType(outputClass, chainToVariableStore);
//    }
//  }

  
	// =========================================================================
	// Variable stuff
	
	// --------------------------------------------------------------------------
	
	
//	public final IRecordVariable getIVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
//		InstantiatedIVariable local = this.instantiatedIVariables_.getValueQuiet(name);
//		if(local!=null) {
//			return local;
//		} else {
//			return this.parentContainer_.getIVariableBackwardsQuiet(name, pathToVariable);
//		}
//	}
  
  /**
   * Add a builder variable, defined by the Java Implmentation (as opposed to the Design stuff)
   */
  public void addBuilderVariable(String name,IRecordVariable variable) throws ItemAlreadyExistsException{
  	this.layerBuilderVariables_.put(name, variable);
  }
  public ChainDefinitionBlock getChainDefinition(String name) throws ParsingException {
  	return this.layerType_.getChainDefinition(name);
  }
  public MappingDefinitionBlock getMappingDefinition(String name) throws ParsingException {
  	return this.layerType_.getMappingDefinition(name);
  }
  public KeyDefinitionBlock getKeyDefinition(String name) throws ParsingException {
  	return this.layerType_.getKeyDefinition(name);
  }
  /**
   * Get a builder variable defined in this layer. 
   * @param name
   * @param pathToVariable
   * @return
   * @throws ParsingException
   */
  public final IRecordVariable getLayerDirectBuilderVariable(String name, BaseTrackers.JavaVariablePath pathToVariable) throws ParsingException {
  	return this.layerBuilderVariables_.get(name);
  }
	public final IRecordVariable getBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) {
		//Check if there are any layer defined Builder Variables
		IRecordVariable result = this.layerBuilderVariables_.getQuiet(name);
		//If no then check more generally
		if(result==null) {
			result = this.layerType_.getBuilderVariableQuiet(name,pathToVariable,isFirstLayer);
		}
		return result;
	}
	/**
	 * Get a builder variable that is defined strictly within this layer set (eg, no parallel layers are searched).
	 * @param name
	 * @param pathToVariable
	 * @return
	 */
	public final IRecordVariable getLayerStrictBuilderVariableQuiet(String name, BaseTrackers.JavaVariablePath pathToVariable) {
		//Check if there are any layer defined Builder Variables
		IRecordVariable result = this.layerBuilderVariables_.getQuiet(name);
		//If no then check more generally
		if(result==null) {

			result = this.layerType_.getLayerStrictBuilderVariableQuiet(name,pathToVariable);
		}
		return result;
	}
	public final IRecordVariable getBuilderVariable(String name, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer) throws ParsingException {
		//Check if there are any layer defined Builder Variables
		IRecordVariable result = getBuilderVariableQuiet(name, pathToVariable, isFirstLayer);
		if(result!=null) {
			return result;
		} else {
			throw ParsingException.createObjectNotFound("builder variable", name);
		}
	}
	// -------------------------------------------------------------------------
	//==========================================================================
	//Things RecordAccess used to do
	public MacroCallInstance createMacroCallFinal(
			String variableName, MacroConstruction macro, MacroCallRequirementsAccess requirementsAccess
		) {
		return this.blockSkeleton_.createMacroCallFinalBase(variableName, macro, requirementsAccess, this);
	} 
	// --------
	public MacroConstruction getMacroConstructionFinal(String name, BaseTrackers.JavaVariablePath pathToContainingLayer) throws ItemNotFoundException {
		MacroConstruction local = this.macros_.getQuiet(name);
		if(local!=null) { 
			return local;
		} else {
		  return this.layerType_.getMacroConstructionFinal(name,pathToContainingLayer); 
		}
	}
	/**
	 * Replace this with a direct set, after, or maybe dump
	 * @param name
	 * @param childNode
	 * @param childNodeReferenceInParent
	 * @return
	 */
	
//	public SLayer getFoundationLayerChild(String name, FoundationNode childNode, FoundationNodeChildReference childNodeReferenceInParent) {
//	  NodeLayerChild foundRecord = this.nodeChildrenWithLinkUsageReferences_.getQuiet(name);
//		SLayer childLayer;
//		if(foundRecord!=null) {
//			//If was actually used by implementation
//			childLayer = foundRecord.getBase();
//			//Inform link that it is a link between two Foundation Layers
//			//foundRecord.setAsFoundationLink(childNode,childNodeReferenceInParent);
//		} else {
//			//Otherwise, this foundation node was never directly used, so now we just make one.
//			childLayer = new SLayer(this.fileEnvironment_);
//			//The linked errors bit is a bit of a hack.
//			foundRecord = new NodeLayerChild(name,this.fileEnvironment_,childNode);
//			this.nodeChildrenWithLinkUsageReferences_.force(name, foundRecord);
//		}
//		foundRecord.setAsFoundationLink(childNode,childNodeReferenceInParent);
//
//		return childLayer;
//	}
	// ==========================================================================
//	public static final class FoundationLink {
//		private final SLayer	baseLayer_;
//		private final AccessStoreX<FoundationLink> childrenLinks_;
//		private final FoundationNode	baseNode_;
//		private FoundationLink(SLayer baseLayer, FoundationNode baseNode, FileEnvironment environment) {
//			this.baseLayer_ = baseLayer;
//			this.baseNode_ = baseNode;
//			this.childrenLinks_ = environment.createEntityStore("foundation layer");
//		}
//		public SLayer getBaseLayer() {
//			return this.baseLayer_;
//		}
//		/**
//		 * This is intented to be called by the FoundationNode class during contruction, as a means of linking up FoundationNodes to SLayers. Thus,
//		 * it is expected this method is called once and only once for each named child.
//		 * @param name
//		 * @return
//		 */
//		public void createFoundationNodeChild(String name, FoundationNode childNode) {
//			final NodeChild foundRecord = baseLayer_.nodeChildrenWithLinkUsageReferences_.getQuiet(name);
//			SLayer childLayer;
//			if(foundRecord!=null) {
//				//If was actually used by implementation
//				childLayer = foundRecord.getBase();
//				//Inform link that it is a link between two Foundation Layers
//				foundRecord.setAsFoundationLink(childNode);
//			} else {
//				//Otherwise, this foundation node was never directly used, so now we just make one.
//				childLayer = new SLayer(baseLayer_.fileEnvironment_,baseLayer_.baseError_);
//			}
//			childLayer.setFoundationParentBridge(this.baseLayer_,this.baseNode_,childNode);
//			
//			//Since we are a "Foundation Layer", we just connect directly to the skeleton stuff of the related foundation node (instead of creating our own)
//			childLayer.blockSkeleton_ = childNode.getSkeletonManager(); 
//			FoundationLink childLink = new FoundationLink(childLayer,childNode, baseLayer_.fileEnvironment_);
//			this.childrenLinks_.force(name,childLink);
//			return childLink;
//		}
//		public FoundationLink getCreatedNodeChild(String name) throws ItemNotFoundException{
//			return this.childrenLinks_.get(name);
//		}
//		public FoundationLink getCreatedNodeChildQuiet(String name) {
//			return this.childrenLinks_.getQuiet(name);
//		}
//		
//	}
	//==========================================================================
	// General methods
	
// DELETE:
//	public SLayer createMacroRoot(MacroConstruction baseConstruction,SpecificCommonErrorOutput requestError) {
//		final SLayer child = new SLayer(this.fileEnvironment_,requestError);
//		child.setType(new MacroRootType(this,baseConstruction, get));
//		return child;
//	}
	void forceFoundationNodeChild(String name, FoundationNode peerFoundationNode, SLayer peerLayer,FoundationNodeChildReference childNodeReferenceInParent) {
		this.nodeChildrenWithLinkUsageReferences_.force(
				name,NodeLayerChild.createFoundationNodeChild(name, peerLayer,childNodeReferenceInParent));
		
	}
	
	public SLayer getNodeChild(String name, int lineNumber) throws ParsingException {
		return getNodeLayerChild(name,lineNumber).getBase();
	} 
	public SLayer getAncestorNodeSLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		return this.layerType_.getAncestorNodeSLayerBackwards(name,pathToParentLayer);
	}
	public SLayer getAncestorVariableSLayer(String name, BaseTrackers.JavaVariablePath pathToParentLayer) throws ParsingException {
		return this.layerType_.getAncestorVariableSLayerBackwards(name,pathToParentLayer);
	}
	
//	private NodeLayerChild getNodeLayerChildNew(String name) throws ParsingException {
//		
//		final NodeLayerChild foundRecord = this.nodeChildrenWithLinkUsageReferences_.getQuiet(name);
//		if(foundRecord!=null) {
//			return foundRecord;
//		} else {
//			NodeLayerChild createdRecord = childLayerMaker_.createNewNodeLayerChild(name, this.blockSkeleton_, this.relatedTracker_, this,this.layerType_);
//			this.nodeChildrenWithLinkUsageReferences_.force(name, createdRecord);
//			return createdRecord;
//		}
//	}


	public NodeLayerChild getNodeLayerChild(String name, int lineNumber) throws ParsingException {
		final NodeLayerChild foundRecord = this.nodeChildrenWithLinkUsageReferences_.getQuiet(name);
		if(foundRecord!=null) {
			return foundRecord;
		} else {
			NodeLayerChild createdRecord = this.childLayerMaker_.createNewNodeLayerChild(name, this.blockSkeleton_, this.relatedTracker_, this, this.layerType_,this.fileEnvironment_, lineNumber);
			this.nodeChildrenWithLinkUsageReferences_.force(name, createdRecord);
			return createdRecord;

		}
	} 
	/*
	 * 			//BaseTrackers.JavaVariablePath path = new BaseTrackers.JavaVariablePath();
//		final SLayer childLayer = parent.childLayer_;
//			AncestorConnection ancestor = getAncestorFoundationNodeQuiet(name, path,true);
//			if(ancestor==null) {
//				throw ParsingException.createObjectNotFound("node child", name);
//			} else {				
				//LocalChild childRef = this.blockSkeleton_.createLocalChild(name, this, this.childLayerMaker_, this.relatedTracker_, ancestor,path);
				// childLayer = childRef.getChildLayer();
				//NodeLayerChild createdRecord = NodeLayerChild.createNonFoundationLayerChild(name, childLayer);
	 */
	public SLayer getVariableChild(String name, int lineNumber) throws ParsingException {
		return this.getVariableChildNode(name,lineNumber).getBase();
	}
	public VariableChild forceVariableChild(String name, VariableChild createdRecord) {
		this.variableChildren_.force(
			name,createdRecord
		);
		return createdRecord;
	}
	public VariableChild getVariableChildNodeQuiet(String name, int lineNumber)    {
		//TODO: optimise...
		try {
			return getVariableChildNode(name, lineNumber);
		} catch (ParsingException e) {
			return null;
		}
	}
	public VariableChild getVariableChildNode(String name, int lineNumber)  throws ParsingException  {
		
		final VariableChild foundRecord = this.variableChildren_.getQuiet(name);
		if(foundRecord!=null) {
			return foundRecord;
		} else {
			IRecordVariable local = this.layerBuilderVariables_.getQuiet(name);
			VariableChild createdRecord;

			
			if(local!=null) {
				Debug.finishMeMarker();
				//TODO: allow local back checking to (change the new BaseTrackers.JavaVariablePath() stuff)
				JavaVariablePath pathToVariableInParent = new BaseTrackers.JavaVariablePath();
				local.getAsFoundataionNodeReferenceVariable(pathToVariableInParent);
				createdRecord = this.childLayerMaker_.createNewVariableLayerChild(name, local, new BaseTrackers.JavaVariablePath(),this, this.layerType_, this.fileEnvironment_, lineNumber);
			} else {
//				if(name.equals("parameterType")) {
////					if(this.childLayerMaker_ instanceof MacroGeneralLayerType) {
////						throw ParsingException.createGeneralSemantic("found variable:"+this.childLayerMaker_);
////					}
//				}
				createdRecord = this.childLayerMaker_.createNewVariableLayerChildNonLocal(name, this, this.layerType_, this.fileEnvironment_, lineNumber);
			}
			this.variableChildren_.force(
				name,createdRecord
			);
			return createdRecord;
		}
	}
	public SLayer getVariableChildOld(String name) throws ParsingException {
		final VariableChild foundRecord = this.variableChildren_.getQuiet(name);
		if(foundRecord!=null) {
			SLayer found = foundRecord.getBase();
			return found;
		} else {
			
			final BaseTrackers.JavaVariablePath pathToVariableDefinition = new BaseTrackers.JavaVariablePath();
			IRecordVariable resolvedVariable = getBuilderVariableQuiet(name, pathToVariableDefinition,true);
			
			if(resolvedVariable==null) {
				throw ParsingException.createObjectNotFound("skeleton variable", name);
			} else {

				SLayer referencedFoundationNode = resolvedVariable.getAsFoundataionNodeReferenceVariable(pathToVariableDefinition);
				DualIndirectLayerType parentBridgeForChild = DualIndirectLayerType.buildVariableBased(pathToVariableDefinition,referencedFoundationNode,this,this.childLayerMaker_,resolvedVariable, fileEnvironment_);
				SLayer childLayer = parentBridgeForChild.getChildLayer();
				this.variableChildren_.force(
						name,new VariableChild(name, childLayer, resolvedVariable,this.fileEnvironment_,pathToVariableDefinition)
				);
				return childLayer;
				//				this.baseLayer_.basicBuild(parentBridgeForChild ,blockSkeleton /* we stay in the same skeleton block */, nodeBuilder);
				//Need to get Design Layer stuff from variable (if available). 
				// Store related SLayer, and get type for the following...
			//	mySkeleton.addIsLocalBlock(parent.blockSkeleton_, globalBlockParameter);
			}
		}
	}
	// ==========================================================================

	// --------------------------------------------------------------------------


	// ==========================================================================

	// ==========================================================================
	// Basic container stuff
	
	// --------------------------------------------------------------------------
	// GET stuff
	
	public GlobalClassReference getGlobalClassReferenceQuiet(String className, BaseTrackers.JavaVariablePath pathToVariable, boolean isFirstLayer)  {
		GlobalClassReference local = this.globalClassReferences_.getQuiet(className);
		if(local!=null) {
			return local;
		}
		return this.layerType_.getGlobalClassReferenceQuiet(className, pathToVariable,isFirstLayer);
	}
	
	public TargetMethod getTargetMethodQuiet(TargetType targetType, String name, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
		MethodReferenceHolder base = this.globalMethodReferences_.getQuiet(name);
		if(base!=null) {
			TargetMethod method = base.getMethodQuiet(targetType);
			if(method!=null) {
				return method;
			}
		}
		if(isFirstLayer) {
			//There is no point doing this over and over again for each layer.
 	 		DirectTargetElementAccess directAccess = targetType.getDirectElementAccessQuiet();
	    if(directAccess!=null) {
	    	return directAccess.getTargetMethodQuiet(name,pathToDeclaration);
	    }
 	 	}
 	 	SMethodHolder holder = this.classMethods_.getQuiet(name);
	  if(holder!=null) {
	  	TargetMethod result = holder.getMethodQuiet(targetType );
	  	if(result!=null) { return result; }
	  } 
	  return this.layerType_.getTargetMethodQuiet(targetType, name, pathToDeclaration,isFirstLayer);
	  
  }
	

	public TargetVariable getTargetVariableQuiet(boolean allowLocalVariables, TargetType targetType, String name, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
  	if(isFirstLayer) {
			//There is no point doing this more than once.
  		DirectTargetElementAccess directAccess = targetType.getDirectElementAccessQuiet();
	    if(directAccess!=null) {
	    	return directAccess.getTargetVariableQuiet(name,pathToDeclaration);
	    }
  	}
  	SVariableHolder holder = this.classVariables_.getQuiet(name);
    if(holder!=null) {
    	TargetVariable result = holder.getVariable(targetType);
    	if(result!=null) {
    		return result;
    	}
    } 
    return this.layerType_.getTargetVariableQuiet(allowLocalVariables, targetType, name, pathToDeclaration, isFirstLayer);
  }

	
  public TargetVariable getLocalVariableQuiet(ScopeContainer relatedScope,  String variableName, JavaVariablePath pathToDeclaration, boolean isFirstLayer) {
  	SLocalVariableHolder holder = this.localVariables_.getQuiet(variableName);
  	
    if(holder!=null) {
    	TargetVariable result = holder.getVariable(relatedScope);
    	if(result!=null) {
    		return result;
    	}
    } 
    return this.layerType_.getLocalVariableQuiet(relatedScope, variableName, pathToDeclaration, isFirstLayer);
    
  }
  
  public TargetType getTargetTypeQuiet( String typeName, BaseTrackers.JavaVariablePath chainToVariable, boolean isFirstLayer) {
  	GlobalClassReference localClass = this.globalClassReferences_.getQuiet(typeName);
		if(localClass!=null) {
			return localClass;
		} else {
			TargetType type = this.types_.getQuiet(typeName);
	    if(type!=null) {
	      return type;
	    } else {
	    	return this.layerType_.getTargetTypeQuiet(typeName,chainToVariable, isFirstLayer);
	    }
		}
	}
//  public TargetVariable getClassVariable(TargetType outputClass, String variableName, BaseTrackers.JavaVariablePath chainToVariable) throws ItemNotFoundException {
//    SVariableHolder holder = this.classVariables_.get(variableName);
//    TargetVariable variable = null;
//    if(holder!=null) {
//       variable = holder.getVariable(outputClass);
//    } 
//    return variable;
//  }
  // -------------------------------------------------------------------------
  // QUERY stuff
  public boolean queryAccessTypeInternal(TargetType outputClass, BaseTrackers.JavaVariablePath chainToVariableStore) {
    //TODO: optimise - how?
  	return this.types_.isContainsValue(outputClass);
  }
  // -------------------------------------------------------------------------
  // IS stuff
  public boolean isLocalTargetVariableQuiet(ScopeContainer scope,  String variableName, boolean isFirstLayer) {
  	SLocalVariableHolder holder = this.localVariables_.getQuiet(variableName);
    if(holder!=null) {
      return holder.isContainsVariableInScope(scope);
    } else {
    	return this.layerType_.isLocalVariableQuiet(scope, variableName, isFirstLayer);
    }
  }

  // --------------------------------------------------------------------------
  // ADD methods
  
  public void addType(TargetType targetType, String typeName)  throws ItemAlreadyExistsException {
    this.types_.put(typeName, targetType);
  }
  
	public SLayer addMacroLayer(
			String name,  final MacroConstruction mc, VariableStore parameterVariables, 
			ImplementationBlockSkeletonManager blockSkeleton,
			JavaImplementationFormBlock.BlockDataBlock.IsMacroDataBlock macroBlockSkeleton,
			LinkupTracker skeletonBuilder
		) throws ParsingException {
  	MacroRootType childType = new MacroRootType(name, parameterVariables, this, this.getRootFoundationNodeBasic().getGlobalSLayer(),macroBlockSkeleton,this.childLayerMaker_,blockSkeleton,this.fileEnvironment_);
  	SLayer peerLayer = childType.getPeerLayer();
		this.macros_.put(name, mc);
		return peerLayer;
  }
	
  public void addClassVariable(TargetType containingTargetClass, String variableName, TargetVariable variable) throws ItemAlreadyExistsException {
    SVariableHolder holder = this.classVariables_.getQuiet(variableName);
    if(holder!=null&&holder.isContains(containingTargetClass)) {
      throw new ItemAlreadyExistsException();
    } else {
      holder = new SVariableHolder(containingTargetClass, variable, holder);
      this.classVariables_.force(variableName, holder); //Could optimise
      Debug.futureFeatureMarker("Could optimise!");
    }
  }
  public void addClassMethod(TargetType containingTargetClass, String methodName, TargetMethod method) throws ItemAlreadyExistsException {
    SMethodHolder holder = this.classMethods_.getQuiet(methodName);
    
    if(holder!=null&&holder.isContains(containingTargetClass)) {
      throw new ItemAlreadyExistsException();
    } else {
      holder = new SMethodHolder(containingTargetClass, method, holder);
      this.classMethods_.force(methodName, holder); //Could optimise
      Debug.futureFeatureMarker("Could optimise!");
    }
  }

  public void addLocalVariable(ScopeContainer relatedScope, String variableName, TargetVariable variable) throws ItemAlreadyExistsException {
    SLocalVariableHolder holder = this.localVariables_.getQuiet(variableName);
    if(holder!=null) {
    	holder.addVariableInScope(variableName,relatedScope,variable);
    } else {
      holder = new SLocalVariableHolder(relatedScope, variable);
      this.localVariables_.force(variableName, holder); 
     }
    
  }
 
	// ==========================================================================



	private static final class LinkHolder {
		private final JavaConcreteChainLink link_;
		private final String linkName_;
		private final String chainName_;
		private boolean used_ = false;
		private final SpecificCommonErrorOutput definitionError_;
		/**
		 * 
		 * @param chainName the name of the chain 
		 * @param linkName the specific name of the link
		 * @param link the actual link
		 * @param definitionError where the link was defined
		 */
		public LinkHolder(String chainName, String linkName, JavaConcreteChainLink link, SpecificCommonErrorOutput definitionError) {
			this.chainName_ = chainName;
			this.linkName_ = linkName;
			this.link_ = link;
			this.definitionError_ = definitionError;
		}
		public boolean isMatchingLink(String chainName, String linkName) {
			return this.chainName_.equals(chainName)&&this.linkName_.equals(linkName);
		}
		public JavaConcreteChainLink getLink() {
			this.used_ = true;
			return this.link_;
		}
		public void checkUsed() {
			if(!this.used_) {
				
				//	this.definitionError_.unexpectedObjectError("chain link", "[chain:"+this.chainName_+", link:"+this.linkName_+']');
			
			}
		}
		
	} // end of class LinkHolder


	// ==========================================================================
//==========================================================================
  //Global stuff
	
  public GlobalClassReference getGlobalClassReference(String className, BaseTrackers.JavaVariablePath pathToVariable) throws ItemNotFoundException {
		return this.globalClassReferences_.get(className);
	}
  /**
	 * MethodAccess base = this.getTargetMethodQuiet(baseClass, methodName, classPath);
		if(base!=null) {
			return base.getMethod().getAsGlobalQuiet();
		} else {
			return null;
			*/
	/**
	 * @param name
	 * @param reference
	 * @return
	 * @throws ItemAlreadyExistsException
	 */
	public JavaImplementationFormBlock.BlockDataBlock.GlobalClassVariableDataBlock addGlobalClassReference(String name, GlobalClassReference reference) throws ItemAlreadyExistsException {
		JavaImplementationFormBlock.BlockDataBlock.GlobalClassVariableDataBlock skeleton = 
			this.blockSkeleton_.addGlobalClassVariableBlock(name);

		this.globalClassReferences_.put(name, reference);
//		this.basicContainer_.addType(reference, name);
		// We do this later when attaching to a layer...
		return skeleton;
	}
	public JavaImplementationFormBlock.BlockDataBlock.GlobalMethodVariableDataBlock addGlobalMethodReference(String methodName,GlobalClassReference baseClass, GlobalMethodReference method ) throws ItemAlreadyExistsException {

		JavaImplementationFormBlock.BlockDataBlock.GlobalMethodVariableDataBlock skeleton = this.blockSkeleton_.addGlobalMethodVariableBlock(methodName) ;
		MethodReferenceHolder holder = this.globalMethodReferences_.getQuiet(methodName);
    if(holder!=null&&holder.isContains(baseClass)) {
      throw new ItemAlreadyExistsException();
    } else {
      holder = new MethodReferenceHolder(baseClass, method, holder);
      this.globalMethodReferences_.force(methodName, holder); //Could optimise
      Debug.futureFeatureMarker("Could optimise!");
      return skeleton;
    }
  }
  
	 
	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath classPath) {
		return getGlobalMethodReferenceQuiet(baseClass, methodName, classPath,true);
	}
	public GlobalMethodReference getGlobalMethodReferenceQuiet( GlobalClassReference baseClass, String methodName, JavaVariablePath pathToVariable, boolean isFirstLayer) {
		MethodReferenceHolder base = this.globalMethodReferences_.getQuiet(methodName);
		if(base!=null) {
			return base.getMethodQuiet(baseClass);
		} else {
			return this.layerType_.getGlobalMethodReferenceQuiet(baseClass,methodName, pathToVariable,isFirstLayer);
		}
	}
	
	private final static class MethodReferenceHolder {
    private final GlobalMethodReference method_;
    private MethodReferenceHolder next_;
    private final GlobalClassReference targetType_;

    public MethodReferenceHolder(GlobalClassReference targetType, GlobalMethodReference method, MethodReferenceHolder next) {
      this.targetType_ = targetType;
      this.method_ = method;
      this.next_ = next;
    }
    public boolean isContains(TargetType queryTargetType) {
      if(this.targetType_ == queryTargetType) {
        return true;
      } else if(next_==null) {
        return false;
      } else {
        return next_.isContains(queryTargetType);
      }
    }
    public GlobalMethodReference getMethodQuiet(TargetType queryTargetType) {
      if(this.targetType_ == queryTargetType) {
        return this.method_;
      } else if(next_==null) {
        return null;
      } else {
        return next_.getMethodQuiet(queryTargetType);
      }
    }
  }
}
