package org.ffd2.skeletonx.compile.library;

import java.io.IOException;

import org.ffd2.solar.compile.GlobalEnvironment;
import org.ffd2.solar.compile.ParserBase;
import org.ffd2.solar.general.Debug;
import org.ffd2.solar.general.SimpleVector;
import org.ffd2.solar.io.FileLocator;
import org.ffd2.solar.language.BasicErrorTracker;

import org.ffd2.skeletonx.compile.impl.FileRoot;
import org.ffd2.skeletonx.compile.impl.OutputGrouping;
import org.ffd2.skeletonx.skeleton.BaseBuilder;


public class ParserControl { //implements IModuleParser<FileRoot>{
  private final GlobalEnvironment definitionEnvironment_;
  private final GlobalEnvironment implementationEnvironment_;
//  public void completePrimaryParse() {
//    unresolved_.fixate();
//  }
  private final BasicErrorTracker errorTracker_;

 // private final ModuleHub<FileRoot> hub_;
  private final SimpleVector<OutputGrouping> baseBuilders_ = new SimpleVector<OutputGrouping>();
	private final ParserBase<FileRoot>	parserBase_;
  
  
  public ParserControl(ParserBase<FileRoot> parserBase,BasicErrorTracker errorTracker) {
    this.parserBase_ = parserBase;
    this.errorTracker_ = errorTracker;
    this.definitionEnvironment_ = new GlobalEnvironment(errorTracker);
    this.implementationEnvironment_ = new GlobalEnvironment(errorTracker);
   // this.hub_ = new ModuleHub<FileRoot>(source,library,this);

  }
  public ParserBase<FileRoot> getParserBase() {
  	return this.parserBase_;
  }
  public OutputGrouping getOutputGrouping(String[] path, boolean doOutput) {
    for(OutputGrouping og : this.baseBuilders_) {
    	if(og.isMatching(path)) {
    		return og;
    	}
    }
    //(path,this.outputContextGlobalBase_);
  	OutputGrouping result = new OutputGrouping(path,doOutput,new BaseBuilder());
    this.baseBuilders_.addElement(result);
    return result;
  }
//  public VariableGraph getOutputContextGlobalBase() {
//    return this.outputContextGlobalBase_;
//  }
 
  public GlobalEnvironment getDefinitionEnvironment() {
    return this.definitionEnvironment_;
  }
  public GlobalEnvironment getImplementationEnvironment() {
    return this.implementationEnvironment_;
  }
  
  public void finish(FileLocator target) throws IOException {
    boolean cleanAll = false;
  	if(!this.errorTracker_.isHasHadErrors()) {
    	completeParsing();
    }
    if(!this.errorTracker_.isHasHadErrors()) {
      for(OutputGrouping base : this.baseBuilders_) {
        base.output( target);
      }
      Debug.println("Outputing all!");
    }
  }
  public void output(FileLocator target, String[] basePackage, boolean cleanAll) throws IOException {
  	 if(!this.errorTracker_.isHasHadErrors()) {
       for(OutputGrouping base : this.baseBuilders_) {
         base.output( target);
       }
       Debug.println("Outputing all!");
     }
//  	if(!this.errorTracker_.isHadError()) {
//  		this.packageTracker_.buildAndOutputAll(target,basePackage, cleanAll);
//  	}
  }

//  public ParseAccess<FileRoot> formAccess( String fileName, ParserToHubConnection<FileRoot> moduleHub) {
//
////    GeneralCommonErrorOutput error = new GeneralCommonErrorOutput(StringUtils.toString(fileName,"."),errorTracker_);
//    Debug.println("Forming access for:",fileName);
//    ParseAccessImpl result = new ParseAccessImpl(
//      fileName,moduleHub,
//      this);
//    return result;
//  }


  public boolean completeParsing() {
  	Debug.println("Doing complete parsing!");
  	SimpleVector<FileRoot> files = parserBase_.getAllParserFiles();
		Debug.println("RP Design");
		for(FileRoot f : files) {
  		f.resolutionPassDesign(); 
  	}
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("IP Design");

  	for(FileRoot f : files) { f.intermediatePassDesign(); }
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("FP Implementation ");
  	for(FileRoot f : files) { f.finalPassDesign(); }
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("RP Implementation");
  	for(FileRoot f : files) { f.resolutionPassImplementation(); }
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("IP Implementation");
  	for(FileRoot f : files) { f.intermediatePassImplementation(); }
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("FP Implementation");
  	for(FileRoot f : files) { f.finalPassImplementation(); }
  	if(this.errorTracker_.isHasHadErrors()) { return false; }
		Debug.println("All good");
    return !this.errorTracker_.isHadError();

  }
/*
 * public void startParse(String text, ParserToHubConnection<FileRoot> hubConnection) throws IOException  {
      //Debug.println("Text to parse:",text);
      SkeletonTable table = new SkeletonTable();
      SkeletonLexParser lex = new SkeletonLexParser();
      GeneralErrorCatcher errorCatcher = new GeneralErrorCatcher(error_);
      lex.doParseForNormal(new StringBasedCharacterSource(text),table,errorCatcher);
      if(!errorCatcher.isInError()) {
        Source<SkeletonToken> tokens = table.getTokens();
        Debug.println("Tokens:",tokens);
        //        System.out.println("Read:"+tokens.length);

    //      System.out.println("tokens:"+solar.general.StringUtils.toString(tokens));
        SkeletonPackrat builder = new SkeletonPackrat(tokens);
        builder.buildFile(root_,errorCatcher);
      }
    }
 */

}
